#pragma ident "$Id: string.c,v 1.6 2004/06/21 19:46:44 dechavez Exp $"
/*======================================================================
 *
 *  ISI related string functions
 *
 *====================================================================*/
#include "isi.h"
#include "util.h"

char *isiReqStatusString(int value)
{
static char    *unknown = "UNKNOWN";
static char   *complete = "complete";
static char *incomplete = "incomplete";

    switch (value) {
      case ISI_COMPLETE:    return complete;
      case ISI_INCOMPLETE: return incomplete;
    }

    return unknown;
}

char *isiFormatString(int value)
{
static char  *unknown = "UNKNOWN";
static char    *undef = "undef";
static char  *generic = "generic";
static char   *native = "native";

    switch (value) {
      case ISI_FORMAT_UNDEF:   return undef;
      case ISI_FORMAT_GENERIC: return generic;
      case ISI_FORMAT_NATIVE:  return native;
    }

    return unknown;

}

char *isiCompressString(int value)
{
static char *unknown = "UNKNOWN";
static char   *undef = "undef";
static char    *none = "uncomp";
static char     *ida = "comp";
static char  *steim1 = "steim1";
static char  *steim2 = "steim2";

    switch (value) {
      case ISI_COMP_UNDEF:  return undef;
      case ISI_COMP_NONE:   return none;
      case ISI_COMP_IDA:    return ida;
      case ISI_COMP_STEIM1: return steim1;
      case ISI_COMP_STEIM2: return steim2;
    }

    return unknown;

}

char *isiPolicyString(int value)
{
static char   *unknown = "UNKNOWN";
static char     *undef = "undef";
static char      *none = "none";
static char   *min_gap = "minimal gap";
static char *min_delay = "minimal delay";

    switch (value) {
      case ISI_RECONNECT_POLICY_UNDEF:     return undef;
      case ISI_RECONNECT_POLICY_NONE:      return none;
      case ISI_RECONNECT_POLICY_MIN_GAP:   return min_gap;
      case ISI_RECONNECT_POLICY_MIN_DELAY: return min_delay;
    }

    return unknown;

}

static char *TypeString(int value)
{
static char  *unknown = "UNKNOWN";
static char    *undef = "undef";
static char     *int8 = "int8";
static char    *int16 = "int16";
static char    *int32 = "int32";
static char    *int64 = "int64";
static char   *real32 = "real32";
static char   *real64 = "real64";
static char     *ida8 = "ida8";
static char    *ida10 = "ida10";

    switch (value) {
      case ISI_TYPE_UNDEF:  return undef;
      case ISI_TYPE_INT8:   return int8;
      case ISI_TYPE_INT16:  return int16;
      case ISI_TYPE_INT32:  return int32;
      case ISI_TYPE_REAL32: return real32;
      case ISI_TYPE_REAL64: return real64;
      case ISI_TYPE_IDA8:   return ida8;
      case ISI_TYPE_IDA10:  return ida10;
    }

    return unknown;
}

char *isiStreamString(ISI_STREAM_NAME *name, char *buf)
{
    if (name->oldchn[0] != 0) {
        sprintf(buf, "%s:%s", name->sta, name->oldchn);
    } else if (strlen(name->loc) != 0 && strcmp(name->loc, ISI_BLANK_LOC) != 0) {
        sprintf(buf, "%4s.%s.%s", name->sta, name->chn, name->loc);
    } else {
        sprintf(buf, "%4s.%s   ", name->sta, name->chn);
    }
    return buf;
}

char *isiTstampString(ISI_TSTAMP *tstamp, char *buf)
{
    utilDttostr(tstamp->value, 0, buf);
    sprintf(buf+strlen(buf), " 0x%04x", tstamp->status);
    return buf;
}

char *isiDescString(ISI_DATA_DESC *desc, char *buf)
{
    buf[0] = 0;
    sprintf(buf+strlen(buf), "%s", isiCompressString(desc->comp));
    sprintf(buf+strlen(buf), ":%s", TypeString(desc->type));
    if (!isiIsNative(desc)) {
        sprintf(buf+strlen(buf), ":%s", desc->order == ISI_ORDER_BIGENDIAN ? "nbo" : "obn");
    }
    return buf;
}

char *isiGenericTsHdrString(ISI_GENERIC_TSHDR *hdr, char *buf)
{
char tbuf[128];

    buf[0] = 0;
    isiStreamString(&hdr->name, buf);
    sprintf(buf+strlen(buf), " %s", isiTstampString(&hdr->tofs, tbuf));
    sprintf(buf+strlen(buf), " %s", isiTstampString(&hdr->tols, tbuf));
    sprintf(buf+strlen(buf), " %6.3lf", isiSrateToSint(&hdr->srate));
    sprintf(buf+strlen(buf), " %3d", hdr->nsamp);
    sprintf(buf+strlen(buf), " %4d", hdr->nbytes);
    sprintf(buf+strlen(buf), " %s", isiDescString(&hdr->desc, tbuf));
    return buf;
}

/* Revision History
 *
 * $Log: string.c,v $
 * Revision 1.6  2004/06/21 19:46:44  dechavez
 * recognize blank loc in isiStreamString()
 *
 * Revision 1.5  2003/11/26 21:20:52  dechavez
 * fixed order test in isiDescString
 *
 * Revision 1.4  2003/11/19 23:48:01  dechavez
 * include util.h to calm compiler
 *
 * Revision 1.3  2003/11/04 19:59:38  dechavez
 * added isiReqStatusString(), changed isiStreamString to use sta/chan/loc if
 * oldchn is missing
 *
 * Revision 1.2  2003/11/03 23:14:06  dechavez
 * removed sample size from isiDescString(), added nbytes to isiGenericTsHdrString()
 *
 * Revision 1.1  2003/10/16 15:38:52  dechavez
 * Initial release
 *
 */
