#pragma ident "$Id: msg.c,v 1.2 2003/12/22 18:45:51 dechavez Exp $"
/*======================================================================
 *
 * Log a message string
 *
 *====================================================================*/
#include "logio.h"

FILE *logioOpenLogFile(LOGIO *lp)
{
    if (strcasecmp(lp->path, "stderr") == 0) return stderr;
    if (strcasecmp(lp->path, "stdout") == 0) return stdout;
    if (strcasecmp(lp->path, "-") == 0)      return stdout;
    return fopen(lp->path, "a+");
}

#define TSLEN 64
#define PID_STR_LEN 64
#define PIDAMBLE_LEN LOGIO_MAX_PREFIX_LEN + LOGIO_MAX_PNAME_LEN + PID_STR_LEN + 3
#define TMPBUF_LEN   (PIDAMBLE_LEN + LOGIO_MAX_MSG_LEN)

static void DeliverMessage(LOGIO *lp, int level)
{
FILE *fp;
time_t now;
char timestamp[TSLEN];
char pidamble[PIDAMBLE_LEN];
#ifdef HAVE_SYSLOG
char tmpbuf[TMPBUF_LEN];
#endif /* HAVE_SYSLOG */

    /* build pidamble */

    pidamble[0] = 0;
    if (strlen(lp->prefix) != 0) sprintf(pidamble+strlen(pidamble), "%s ", lp->prefix);
    if (strlen(lp->pname) != 0) sprintf(pidamble+strlen(pidamble), "%s", lp->pname);
    sprintf(pidamble+strlen(pidamble), "[%d:%d]", (int) getpid(), THREAD_SELF());

    /* using syslog */

    if (lp->syslog) {
#ifdef HAVE_SYSLOG
        sprintf(tmpbuf, "%s %s", pidamble, lp->msg);
        syslog(level, tmpbuf);
#endif /* HAVE_SYSLOG */
        return;
    }

    /* using user supplied function */

    if (lp->func != NULL) {
        (*lp->func)(lp->msg);
        return;
    }

    /* logging to a file */

    if ((fp = logioOpenLogFile(lp)) == NULL) return;

    /* timestamp */
    now = time(NULL);
    if (strftime(timestamp, TSLEN, lp->tfmt, localtime(&now))) fprintf(fp, "%s ", timestamp);

    /* pidamble */
    fprintf(fp, "%s ", pidamble);

    /* message */
    fprintf(fp, "%s\n", lp->msg);
    logioCloseFile(fp);
}

VOID logioMsg(LOGIO *lp, int level, CHAR *format, ...)
{
int i;
va_list marker;
CHAR *ptr;

    if (lp == NULL) return;

/* Ignore the message if it is beyond our threshold */

    if (level > logioThreshold(lp)) return;

/* Build and deliver the message string */

    MUTEX_LOCK(&lp->mutex);
        memset((void *) lp->msg, 0, LOGIO_MAX_MSG_LEN);
        if (strlen(lp->prefix) > 0) sprintf(lp->msg, "%s ", lp->prefix);

        ptr = lp->msg;
        va_start(marker, format);
        vsprintf(ptr, format, marker);
        va_end(marker);

        for (i = strlen(lp->msg) - 1; i >= 0 && lp->msg[i] == '\n'; i--) lp->msg[i] = 0;
        DeliverMessage(lp, level);
    MUTEX_UNLOCK(&lp->mutex);
}

/* Revision History
 *
 * $Log: msg.c,v $
 * Revision 1.2  2003/12/22 18:45:51  dechavez
 * include pname[pid:tid] in syslog messages
 *
 * Revision 1.1  2003/06/09 23:48:00  dechavez
 * initial release
 *
 */
