#pragma ident "$Id: misc.c,v 1.6 2004/06/10 17:18:38 dechavez Exp $"
/*======================================================================
 *
 *  Misc helper functions for cross platform convenience
 *
 *====================================================================*/
#include "util.h"

BOOL utilNetworkInit(VOID)
{
#ifdef WIN32
WSADATA lpWSAData;

    return WSAStartup(MAKEWORD(2,2), &lpWSAData) == 0 ? TRUE : FALSE;
#else
    return TRUE;
#endif
}

/* Set socket for non-blocking I/O */

VOID utilSetNonBlockingSocket(int sd)
{
#ifdef WIN32
unsigned long wl; 
    wl=1;
    ioctlsocket(sd, FIONBIO,&wl);
#elif defined VXWORKS
    ioctl(sd, FIONBIO, 0);
#else
    fcntl(sd, F_SETFL, O_NONBLOCK);
#endif
    return;
}

SOCKET utilCloseSocket(int sd)
{
    if (sd != INVALID_SOCKET) {
#ifdef WIN32
        closesocket(sd);
#else
        close(sd);
#endif
    }
    return INVALID_SOCKET;
}

BOOL utilFileExists(char *path)
{
BOOL status;

#ifdef WIN32

struct _stat buf;

    if (path == (char *) NULL) return FALSE;

    if (_stat(path, &buf) != 0) {
        status = FALSE;
    } else if (buf.st_mode & _S_IFREG) {
        status = TRUE;
    } else {
        status = FALSE;
    }

#else

struct stat buf;

    if (path == (char *) NULL) return FALSE;

    if (stat(path, &buf) != 0) {
        status = FALSE;
    } else if (buf.st_mode & S_IFREG) {
        status = TRUE;
    } else {
        status = FALSE;
    }

#endif

    return status;
}

/* Get socket address via specification by either name or dot decimal address */

BOOL utilSetHostAddr(struct sockaddr_in *in_addr, char *host, int port)
{
BOOL status;
int sockAddrSize;
#ifndef _WIN32
int h_errno;
#endif
unsigned long addr;
struct hostent *hp, result;
char buffer[MAXPATHLEN];

    in_addr->sin_family = AF_INET;
    in_addr->sin_port = htons((UINT16) port);

/* Begin by assuming host name is given in text form */

#ifdef _WIN32
    hp = gethostbyname(host);
#else
    gethostbyname_r(host, &result, buffer, MAXPATHLEN, &hp, &h_errno);
#endif

/* If that fails, try again assuming host name is in dot decimal form */

    if (hp == NULL && (addr = inet_addr(host)) != -1) {
#ifdef _WIN32
        hp = gethostbyaddr((char *) &addr, sizeof(addr),AF_INET);
#else
        gethostbyaddr_r(
            (char *) &addr, sizeof(addr),
            AF_INET,
            &result,
            buffer,
            MAXPATHLEN,
            &hp,
            &h_errno
        );
#endif
    }

    if (hp == (struct hostent *) NULL) {
        status = FALSE;
    } else {
        memcpy(&in_addr->sin_addr, hp->h_addr, hp->h_length);
        status = TRUE;
    }

    return status;
}

/* Revision History
 *
 * $Log: misc.c,v $
 * Revision 1.6  2004/06/10 17:18:38  dechavez
 * fixed utilSetNonBlockingSocket() for WIN32
 *
 * Revision 1.5  2004/06/08 18:45:56  dechavez
 * ifdef h_error out of utilSetHostAddr() for Windows builds (aap)
 *
 * Revision 1.4  2004/06/04 22:49:46  dechavez
 * various AAP windows portability modifications
 *
 * Revision 1.3  2003/11/04 00:35:50  dechavez
 * utilCloseSocket() changed to return INVALID_SOCKET
 *
 * Revision 1.2  2003/10/16 15:50:52  dechavez
 * added tests for invalid arguments
 *
 * Revision 1.1  2003/06/10 00:05:41  dechavez
 * initial release
 *
 */
