
/* file SCUSRDEF.H
 *      ==========
 *
 * version 2, 6-JUL-91
 *
 * prototypes of module SHCORR.C
 * K. Stammler, 22-MAY-91
 */


#ifndef __SCUSRDEF
#define __SCUSRDEF

#ifndef __SHCONST
#include "shconst.h"
#endif

/*---------------------------------------------------------------------------*/


void sc_set_corrm( int mode, STATUS *status );

/* set correlation mode
 *
 * parameters of routine
 * int       mode;     input; correlation mode (see above)
 * STATUS    *status;  output; return status
 */


/*---------------------------------------------------------------------------*/


void sc_set_corrl( REAL from_t, REAL to_t, STATUS *status );

/* sets correlation length for all following correlations
 *
 * parameters of routine
 * REAL       from_t;     input; rel. start time
 * REAL       to_t;       input; rel. end time
 * STATUS     *status;    output; return status
 */


/*---------------------------------------------------------------------------*/


long sc_restrclth( REAL dt );

/* returns length of result trace of correlation
 *
 * parameters of routine
 * REAL       dt;            input; sample distance
 */


/*---------------------------------------------------------------------------*/


void sc_do_corr( SAMPLE *wav, long wav_s, long wav_e, SAMPLE *trc,
	long maxtrc, REAL dt, SAMPLE *res, STATUS *status );

/* computes crosscorrelation of two traces "wav" and "trc".  The
 * wavelet wav[wav_s..wav_e] is correlated with the trace windows
 * trc[wav_s-cls..wav_e-cls], trc[wav_s-cls+1..wav_e-cls+1], ...
 * up to trc[wav_s+cle..wav_e+cle].  Not existing samples of the
 * "trc"-array are replaced by zeroes.
 *
 * parameters of routine
 * SAMPLE     *wav;          input; (whole) wavelet trace
 * long       wav_s, wav_e;  input; wavelet window (samples)
 * SAMPLE     *trc;          input; 2. correlation trace
 * long       maxtrc;        input; length of 2. correlation trace (samples)
 * REAL       dt;            input; sample distance in sec
 * SAMPLE     *res;          output; result trace
 * STATUS     *status;       output; return status
 */


/*---------------------------------------------------------------------------*/


void sc_polar2( SAMPLE zarr[], SAMPLE rarr[], long lth, REAL *angle );

/* computes main direction of polarisation in 2-dimensional case.
 * Result is the angle "angle" to the z-direction (in degrees)
 *
 * the coherence matrix
 *
 *          -               -        -        -
 *         |  <Z*Z>   <Z*R>  |      |  z    c  |
 *   C  =  |                 |  =:  |          |
 *         |  <Z*R>   <R*R>  |      |  c    r  |
 *          -               -        -        -
 *
 * is diagonalised.  The direction of the eigenvector with the
 * largest eigenvalue is the desired direction
 *
 * Let
 *   k := (z - r) / c
 * and
 *   alpha[1] := 1/2 * (k + sqrt(k*k+4))
 *   alpha[2] := 1/2 * (k - sqrt(k*k+4))
 *
 * then the eigenvalues lambda[i] and eigenvectors v[i] (i = 1,2)
 * are given by
 *
 *   lambda[i] = r + c*alpha[i]
 *   v[i] = ( alpha[i], 1 )
 *
 *
 * parameters of routine
 * SAMPLE     zarr[];  input; samples of z-trace
 * SAMPLE     rarr[];  input; samples of r-trace
 * long       lth;     input; length of traces in samples
 * REAL       *angle;  output; computed angle
 */


/*---------------------------------------------------------------------------*/


void sc_polar3( SAMPLE zarr[], SAMPLE narr[], SAMPLE earr[],
	long lth, REAL *azim, REAL *inci, STATUS *status );

/* computes main direction of polarisation in 3-dimensional case.
 * Result is azimuth and angle of incidence (in degrees).
 *
 * the coherence matrix
 *
 *          -                       -         -              -
 *         |  <Z*Z>   <N*Z>   <E*Z>  |      |  zz    zn   ze  |
 *         |                         |      |                 |
 *   C  =  |  <Z*N>   <N*N>   <E*N>  |  =:  |  zn    nn   ne  |
 *         |                         |      |                 |
 *         |  <Z*E>   <N*E>   <E*E>  |      |  ze    ne   ee  |
 *          -                       -        -               -
 *
 * is diagonalised.  The direction of the eigenvector with the
 * largest eigenvalue is the desired direction
 *
 * parameters of routine
 * SAMPLE     zarr[];  input; samples of z-trace
 * SAMPLE     narr[];  input; samples of n-trace
 * SAMPLE     earr[];  input; samples of e-trace
 * long       lth;     input; length of traces in samples
 * REAL       *azim;   output; computed azimuth
 * REAL       *inci;   output; computed angle of incidence
 * STATUS     *status; output; return status
 */


/*---------------------------------------------------------------------------*/

#endif /* __SCUSRDEF */
