#!/usr/bin/env python

import os, tempfile
import seis, seis.ah

# check pql version
#### pqlversion = None
#### __vv = " ".join(os.popen("pql").readlines()).split()
#### for __v in __vv:
####     if __v[:2]=="20" and __v[4] == ".":
####         pqlversion = __v
####         break

# The TempDir is used because otherwise if the script is interrupted the
# temporary directory is not removed automatically. TempDir, in contrast,
# removes the associated directory if its instance ceases to exist.
class __TempDir(object):

    def __init__(self):
        self.name = tempfile.mkdtemp()

    def __del__(self):
        import os # helps if called after parts of 'os' are already deleted
        os.system("rm -fr '%s'" % self.name) # XXX this is just a crude hack

class Defaults(dict):

    AbsTime =  "0"
    RelTime =  "1"
    NumRecs = "12"

    def __init__(self, f=None):
        self.keys = []
        try:
            self.read(os.path.expanduser("~/.pqldefaults"))
        except IOError:
            raise IOError,  "Failed to read ~/.pqldefaults; to fix, " \
                            "run 'pql' once and 'set defaults'"

    def read(self, fname):
        for line in file(fname).xreadlines():
            line = line.split()
            key  = line[0]
            self[key] = " ".join(line[1:])
            if key not in self.keys:
                self.keys.append(key)

    def write(self, fname):
        keys = self.keys
        f = file(fname, "w")
        f.write("\n".join(["%s %s" % (key, self[key]) for key in keys]))
        f.close()

def show(traces, prefix="", maxnum=100, defaults=None):

    tmpdir = __TempDir()
    fmt = "%%0%dd" % len(str(len(traces)))

    filenames = []
    for i,trc in enumerate(traces):
        if isinstance(trc, seis.Trace):
            filename = fmt % (i+1)
        elif isinstance(trc, tuple):
            trc,filename = trc
        else:
            raise TypeError, "Cannot plot '%s'" % type(trc)
        f = file(tmpdir.name + "/" + filename,"w")
        seis.ah.write(trc, f)
        f.close()
        filenames.append(filename)

    filenames = " ".join(["'%s'" % f for f in filenames])

    if not defaults:
        defaults = Defaults()
        defaults['Number']      = str(min(maxnum, len(traces)))
        defaults['Time']        = Defaults.AbsTime
        defaults['Label']       = "1"
        defaults['AutoScale']   = "1"
    defaults.write(tmpdir.name + "/pqldefaults")
    os.system("cd '%s'; pql -c pqldefaults -ah %s" % (tmpdir.name, filenames))

if __name__ == "__main__":

    # self test

    from Numeric import arange, sin
    from time import time

    traces = []
    for i in xrange(15):
        x = 0.05*arange(10000.)
        y = (20.+i)*sin((0.1+0.02*i)*x)
        t = seis.Time(time1970 = time()+i*20)
        trc = seis.Trace(data=y, delta=0.05, time=t)
        traces.append(trc)
    traces[6] = (traces[6], "Fancy label")

    show(traces)
