import time as _time
import seiscomp.detector

from seiscomp.detector import Trigger as Pick

class Origin(object):

    __slots__ = ["detected","bad","status","id",
        "lat", "lat_err", "lon", "lon_err", "dep", "dep_err",
        "time", "time_err", "rms_res", "confreg_semmaj", "confreg_semmin",
        "associations",
        "confreg_majaxi", "confreg_timerr", "trgs", "arr",
        "mag_mb", "amp_mb", "per_mb", "mag_ml", "amp_ml",
        "mb_avg", "mb_nst", "ml_avg", "ml_nst", "m_med", "m_nst","md5hash" ]
    
    def __init__(self, id="", time=None, lat=0., lon=0., dep=0.):
        self.id,self.time,self.lat,self.lon,self.dep = id,time,lat,lon,dep

        self.confreg = OriginConfRegion()
        self.error   = OriginError()

        self.rms_res = 0 # XXX
        self.detected = _time.gmtime() # XXX
        self.status = "N" 
        self.ignored = 0

    def remove_arrival(self, id):
        self.trgs = [ trg for trg in self.trgs if trg.id != id ]
        self.arr  = [ arr for arr in self.arr  if arr.id != id ]

    def compute_magnitudes(self):
        # determine the magnitudes mb and ml and store them in dictionaries.
        _mlist_all = []
        self.mag_mb, self.amp_mb, self.per_mb = compute_magnitude_mb(self)
        self.mag_ml, self.amp_ml              = compute_magnitude_ml(self)

        _mlist = [m for m in self.mag_mb.values() if m != 0.]
        _mlist_all.extend(_mlist)
        self.mb_nst = len(_mlist)
        if self.mb_nst == 0:
                self.mb_avg = 0.
        else:   self.mb_avg = average(_mlist)

        _mlist = [m for m in self.mag_ml.values() if m != 0.]
        _mlist_all.extend(_mlist)
        self.ml_nst = len(_mlist)
        if self.ml_nst == 0:
                self.ml_avg = 0.
        else:   self.ml_avg = average(_mlist)

        self.m_nst = len(_mlist_all)
        if self.m_nst == 0:
                self.m_med = 0.
        else:   self.m_med = median(_mlist_all)

    def compute_residuals(self):
        raise NotImplementedError

    def get_region(self):
        import seiscomp.fe
        return seiscomp.fe.region(self.lat, self.lon)

    region = property(get_region)


class OriginError(object):

    def __init__(self, time_err=0., lat_err=0., lon_err = 0., dep_err = 0.):
        self.time_err = time_err
        self.lat_err  = lat_err
        self.lon_err  = lon_err
        self.dep_err  = dep_err


class OriginConfRegion(object):

    def __init__(self, timerr=0., semmaj=0., semmin = 0., majaxi = 0.):
        self.timerr = timerr
        self.semmaj = semmaj
        self.semmin = semmin
        self.majaxi = majaxi

class Association(object):

    __slots__ = ["origin_id", "pick_id", "phase", "residual", "used", "db" ]

    def __init__(self, origin_id=None, pick_id=None, phase=None,
                        residual=None, used=None, db=None):
        self.origin_id, self.pick_id, self.phase, self.residual, self.used = \
                origin_id, pick_id, phase, residual, used
        self.db = db

