import string, re
import seis

class Trigger:
    pass

def read_eventfile(f):
    """
    Parses an event file as generated by autoloc. It skips any line
    before the one starting with "LocSAT", and is therefore also
    suitable for files generated by LocSAT itself.
    """

    lines = file(f).readlines()
    regex = re.compile(" +(.*[A-Z]) +[mM][bsSL]=(\d.\d) +(\d+/../.. +" \
                       "..:..:..) +(\d\d.\d) (.) +(\d+.\d) (.).*")
    for i in range(len(lines)):
        line = string.split(lines[i])
        if len(line) == 0:
            continue
        if line[0] != "LocSAT":
            continue

        # Parse the event line
        evt = seis.Event()
        m = regex.match(lines[i+2])
        if not m: break # this is practically an error
        m = m.groups()
        evt.com = m[0]
        evt.time   = seis.Time(m[2])
        evt.coord  = float(m[3]), float(m[5]), 33.
        if m[4]=='S': evt.lat = -evt.lat
        if m[6]=='W': evt.lon = -evt.lon

        # The LocSAT line was found; now scan the rest of the file
        triggers = {}
        for ii in range(i+5, len(lines)):
            line = string.split(lines[ii])
            if len(line) == 0:
                break
            trg = Trigger()
            trg.sta, trg.net = line[0:2]
            trg.time = seis.Time("%s %s" % tuple(line[2:4]))
            trg.amp, trg.per, trg.resi, trg.dist, trg.azi, trg.mb, trg.ml \
                = map(float, line[4:])
            triggers[trg.sta] = trg

        break

    return evt, triggers


def read_station_inf(station_inf):
    """
    Reads an Autoconf-style "station.inf" file and returns a dictionary
    with NET_STA being the key, and the entries containing tuples of
    ( net, sta, lat, lon, alt, weight ), where 
 
    sta     station code 
    net     network code 
    lat     latitude in degrees
    lon     latitude in degrees
    alt     altitude in meters 
    weight  0, 1 or 2 
    """ 
 
    sta_dict = {} 
 
    if type(station_inf) is str:
        station_inf = file(station_inf)
    assert type(station_inf) is file
 
    for line in station_inf.xreadlines():
        sta, lat, lon, alt, weight, net = line.split()
        lat, lon, alt = float(lat), float(lon), float(alt)
        weight = int(weight)
 
        key = "%s_%s" % (net, sta)
        sta_dict[key] = ( net, sta, lat, lon, alt, weight )
 
    return sta_dict
