from Numeric import array, cumsum, concatenate, average
from seiscomp import mseed
import seis

def group_streams(records):

    d = dict()
    for rec in records:
        id = rec.stream_id
        if id not in d:
            d[id] = []
        d[id].append(rec)

    return d

def group_continuous(records, dtmax=0.1):

    # determine time gaps
    t0 = records[0].begtime
    t1 = array([ r.begtime-t0 for r in records ])
    t2 = array([ r.endtime-t0 for r in records ])
    dt = t2[:-1] - t1[1:]

    # determine sequence number increments (normally 1)
    nn = array([ r.seq_no for r in records ])
    dn = nn[1:] - nn[:-1]

    # a gap is believed to be only where the time gap exceeeds dtmax
    # *and* there is a discontinuous sequence number
    indices = [0] + list(cumsum(abs(dt)>dtmax and dn!=1))
    ngroups = indices[-1]+1
    groups  = [ [] for i in xrange(ngroups) ]
    for i, rec in zip(indices, records):
        groups[i].append(rec)

    return groups

def concat(records):

    # determine time gaps
    t0 = records[0].begtime
    t1 = [ r.begtime-t0 for r in records ]
    t2 = [ r.endtime-t0 for r in records ]
    dt = array(t2[:-1]) - array(t1[1:])
#   print average(dt)

    # TODO: determine and apply appropriate time correction
    first = records[0]
    data = concatenate([rec.data for rec in records])
    time = first.time
    cha  = first.cha
    sta  = seis.Station(code=first.sta)
    trc  = seis.Trace(data=data, time=time, cha=cha, sta=sta, fsamp=first.fsamp)
    return trc

#records = [ r for r in mseed.Input("malt.bhz") ]
#groups  = group_continuous(records)
#
#for grp in groups:
#    trc = concat(grp)
#    fname = "%04d%02d%02d%02d%02d%02d" % trc.time.asDate
#    fname = fname[2:] + ".%s.%s.AH" % (trc.sta.code, trc.cha)
#    seis.ah.write(trc, fname)
