/***************************************************************************
 *   Copyright (C) 2007 by seismo   *
 *   seismo@pclaptop   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
/***************************************************************************
 * rtnet.c
 * SeedLink client using libslink (Chad Trabant).
 *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <math.h>		                 // For math routines (such as sqrt & trig).
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>

#include <libslink.h>

#ifdef WIN32                                     // WINDOWS
  #include "glut.h"
  #include <process.h>
#else
  #include <GL/glut.h>                           // LINUX
#endif


#ifndef WIN32                                    
  #include <signal.h>

  static void term_handler (int sig);
#else
  #define strcasecmp _stricmp
#endif

#define PACKAGE "rtnet"
#define VERSION LIBSLINK_VERSION

static short int verbose  = 0;
static short int ppackets = 0;
static char * statefile   = 0;	    /* state file for saving/restoring state */
static char *dumpfile     = 0;	    /* output file for data dump */

static char *location     = "1";    // switch to read data for plotting
static FILE *outfile      = 0;      /* the descriptor for the dumpfile */
static int yheight        = 0;      /* max height in pixels */
static int xlenght        = 0;      /* max length in pixels */

int minutter = 0;                                // MINUTES PER SCREEN

#define nch 35                                   // MAX NUMBER OF CHANNELS
#define MAXSR 100                                // MAX SAMPLES PER SECOND
#define NBSCALE 8                                // NUMBER OF BUFFERS SCALING

//****************  FILTER  **********************************
/* Type-independant absolute value for C standard types */
#ifndef abs
#define ABS(x) ((x)<0?-(x):(x))
#endif
#define TRUE 1
static  int nsects;
static  float sn[5*3+1],sd[5*3+1];

float   snc[nch][16];                            // filter coefficients
float   sdc[nch][16];                            // filter coefficients
int     cset[nch];

static  float save_y2[nch][16];                  // overlap between 2 filter buffers
static  float save_y1[nch][16];
static  float save_x2[nch][16];
static  float save_x1[nch][16];

float   fl;
float   fh;
float   eqavst;
int     orden;

int     xo=0;                          // coordinates for position on screen
int     yo=0;                          // coordinates for position on screen
int     sindob    = 0;                 // single or double buffering

int     OS = 1;
int     printarg = 1;

typedef struct {float r, i;} complex;

/* Table of constant values */
static  float c_b12 = 2.;
static  complex c_b43 = {1.,0.};


/* Local function prototypes */
static  double warp();
static  int chebparm(), lptbp(), lptbr(), lpthp(), bilin2(), lp();
static  int c1roots(), c2roots();
static  int cutoffs(), beroots();
static  complex cmul(), cpowi(), csqrt1(), conjg(), cdiv();

//****************  END FILTER  **********************************


static  SLCD * slconn;	                         // connection parameters 
SLpacket * slpack;
int     seqnum;
int     ptype;
int     fnd=0;

char    *topdir_rt       = 0;               // top directory RTQUAKE
/*****************/
/* COLOR SCHEMES */
/*****************/
float BGND_D_R = 0.0;                            // DEFAULT BACKGROUND  : BLACK
float BGND_D_G = 0.0;
float BGND_D_B = 0.0;

float DRAW_D_R = 1.0;                            // DEFAULT DRAW        : OKER
float DRAW_D_G = 0.6;
float DRAW_D_B = 0.0;

float NAME_D_R = 0.5;                            // DEFAULT STATION-NAME: YELLOW
float NAME_D_G = 0.7;
float NAME_D_B = 0.7;

float DATE_D_R = 0.0;                            // DEFAULT DATE        : GREEN
float DATE_D_G = 1.0;
float DATE_D_B = 0.0;

float LATE_D_R = 0.0;                            // DEFAULT LATENCY     : GREEN
float LATE_D_G = 1.0;
float LATE_D_B = 0.0;

float DETE_D_R = 1.0;                            // DEFAULT DETECTION   : RED
float DETE_D_G = 0.0;
float DETE_D_B = 0.0;

float RATI_D_R = 0.0;                            // DEFAULT RATIO       : GREEN
float RATI_D_G = 1.0;
float RATI_D_B = 0.0;

float PICK_D_R = 1.0;                            // DEFAULT PICKS       : RED
float PICK_D_G = 0.0;
float PICK_D_B = 0.0;

float CURN_D_R = 1.0;                            // DEFAULT CURRENT NAME: WHITE
float CURN_D_G = 1.0;
float CURN_D_B = 1.0;

float AXIS_D_R = 0.3;                            // DEFAULT AXIS        : GRAY
float AXIS_D_G = 0.3;
float AXIS_D_B = 0.3;

float   FL_PRESET = 2.0;                         // DEFAULT LOW-PASS FREQUENCY
float   FH_PRESET = 8.0;                         // DEFAULT HIGH-PASS FREQUENCY
int     WINH      = 400;                         // DEFAULT WINDOW HEIGHT IN PIXELS
int     WINL      = 1000;                        // DEFAULT WINDOW LENGTH IN PIXELS
float   YSCALE    = 400.0;                       // DEFAULT FACTOR FOR SCALING
float   AMP_STEP  = 4.0;                         // DEFAULT AMPLIFICATION STEP
int     TOTALSAMP = 90000;                       // DEFAULT TOTAL SAMPLES PER SCREEN
float   TOTALMINU = 15.0;                        // DEFAULT TOTAL MINUTES PER SCREEN



int   TOTSMP    = 0;
float MINUTES   = 0.0;


/*****************/
/* COLOR SCHEMES */
/*****************/
float BGND_R;
float BGND_G;
float BGND_B;

float DRAW_R;
float DRAW_G;
float DRAW_B;

float NAME_R;
float NAME_G;
float NAME_B;

float DATE_R;
float DATE_G;
float DATE_B;

float LATE_R;
float LATE_G;
float LATE_B;

float DETE_R;
float DETE_G;
float DETE_B;

float RATI_R;
float RATI_G;
float RATI_B;

float PICK_R;
float PICK_G;
float PICK_B;

float CURN_R;
float CURN_G;
float CURN_B;

float AXIS_R;
float AXIS_G;
float AXIS_B;

char ADDR_SERVER[200];
char STREAMS_SERVER[200];
char STATIONS_SERVER[200];

//TOTSMP = 60 * MINUTES * MAXSR;
int    sw2[nch];
int    sw3[nch];
int    sw4[nch];
int    sw5[nch];
int    sw1[nch];
int    sw50[nch];
int    rotate[nch];
int    sum[nch];
int    teller[nch];
int    avg[nch];

float amp=1.0;
float values[nch][900000];
int   trgflg[nch][900000];
int   trgmrk[nch][900000];
char  window_title[80];
int k;
int i;
int m;
int j;
int stopwatch = 0;
int stno = 0;
int only_one = 0;
int demo_version = 0;
int auto_scaling = 0;
int antall = 0;

int    init_sw=0;
int    ns[nch];

float  promedio[nch];
float  factor[nch];
float  scale_factor[nch];
long   ryrn[nch];
long   rmon[nch];
long   rday[nch];
long   rhrn[nch];
long   rmin[nch];

int   iryrn[nch];
int   irmon[nch];
int   irday[nch];
int   irhrn[nch];
int   irmin[nch];

int    risec[nch];
float  rsek[nch];
int    rsecs[nch];
float  ftime[nch];
float  stpix[nch];
int    sr[nch];
float  amp_single[nch];
float  latsnitt[nch];
int    trigger[nch];
int    active[nch];

float  csta[nch];
float  clta[nch];
int    hold[nch];
float  rat[nch];

float  v[nch];
float  incr[nch];
char   rtid[nch][80];
char   lat[nch][80];
char   snr[nch][80];
char   tid[nch][80];
int    ix[nch];
int    time_quality[nch];

int    counter[nch];
int    current_channel = 0;
int    filter_rescale_counter = 0;
int    rescale_counter = 0;

float R=0.0;
float G=0.0;
float B=0.0;

  double TIMESTART;
  double TIMESTOPP;
  double TIMEDIFF;

int    start_stop = 0;
char str[80];
char last[80];
int  seismo[1000];
float fseismo[1000];
int RunMode = 1;		// Used as a boolean (1 or 0) for "on" and "off"
float xold=-500.0;
float xnew=-500.0;
float yold=0.0;
float ynew=0.0;
int nsold=0;

char toplot[nch][20];
char fullna[nch][80];
int no_to_plot;


char tags[20][50];
float tags_ix[50];
int tag_index=0;

 

static int packet_handler (char *msrecord, int packet_type,
			    int seqnum, int packet_size);
static int parameter_proc (int argcount, char **argvec);
static char *getoptval (int argcount, char **argvec, int argopt);
static void usage (void);
static void FilterRescale(void);
static void Rescale(void);
static void appl(int fnd);
static int  DET_MEM(int kan);
static void DTE();
static void TIMSEC();
static void strip(int line);
static void bitmap_output(int x, int y, char *string,void *font);
static void STA_LTA();
static int  unpack_samples();
static void design();
static int  buroots();
//static void main(int argc,char **argv);






void drawScene(void)
{
  int ch;
  int ixtrg;
  int DDAY,MMTH;
  int DY;
  int YR;
  int xmax=0;
  int marker1=0;
  int marker2=0;
  double MSECS;
  double MSECSR;

  long   yrn;
  long   mon;
  long   day;
  long   hrn;
  long   min;

  int   iyrn;
  int   imon;
  int   iday;
  int   ihrn;
  int   imin;
  float  sek;
  char dum[50];
  /* Loop with the connection manager */
//  while ( sl_collect (slconn, &slpack) )

  if( sl_collect (slconn, &slpack))
  {
    if(start_stop == 0)
    {
      ptype  = sl_packettype (slpack);
      seqnum = sl_sequence (slpack);
      fnd=packet_handler ((char *) &slpack->msrecord, ptype, seqnum, SLRECSIZE);
      if(fnd != -1)                              // PACKET FOUND ?
      {

        ch=fnd;                                  // YES, SAVE STATION NUMBER
//printf("%2d packet found NS: %d\n",ch,ns[ch]);
		fnd=-1;     
//incrtu                             // RESET 'fnd'
        xmax=(int)((float)WINL / incr[ch]);      // MAX X-VALUE
//printf("%2d packet found NS: %d xmax: %5d incr: %10.5f\n",ch,ns[ch],xmax,incr[ch]);
        for(k=0;k<ns[ch];k++)                    // REMOVE DC
          fseismo[k]=(float)(seismo[k]-avg[ch]);



        if(fl != 0.0 && fh != 0.0)               // FILTER DATA ?
        {
          if(ch == 0)                            // IF CHANNEL 0 ?
          {
            filter_rescale_counter++;            // INCREMENT COUNTER
            if(filter_rescale_counter >= 50)    // FILTER-RESCALE ?
            {
              printf("Rescale-Filter\n");
              FilterRescale();
              filter_rescale_counter = 0;
            }
          }
          for(k=0;k<ns[ch];k++)                  // ADD AMPLITUDE TO CONSERVE ENERGI
            fseismo[k] = fseismo[k] * 100.0;
          appl(ch);                              // FILTER ONE BUFFER
          ixtrg=DET_MEM(ch);                     // COMPUTE STA/LTA
          if(rat[ch] >= 4.5)
          {
            trigger[ch] = 1;                     // TRIGGER ON
            if(sw50[ch] == 0)
            {
              marker1=ix[ch];                    // MARKER FOR START OF BUFFER WITH DETECTION
              marker2=ix[ch]+ixtrg;              // MARKER FOR SAMPLE WITH DETECTION
              if(marker1 < xmax && marker2 < xmax)
              {
                trgflg[ch][marker1] = 1;
                trgmrk[ch][marker2] = 1;
                marker1 = 0;
                marker2 = 0;
              }

              sprintf(dum,"play %s/glasses.wav &",topdir_rt);
//              system(dum);
//              printf("%s %s  ix: %6d ixtrg: %3d  %6.1f\n",toplot[ch],rtid[ch],ix[ch],ixtrg,rat[ch]);
              sprintf(last,"Last trigger: %s %s  ix: %6d ixtrg: %3d  %6.1f",toplot[ch],rtid[ch],ix[ch],ixtrg,rat[ch]);
              sw50[ch]=1;
            }
          }else{
            if(trigger[ch] == 1)                 // TRIGGER ON ?
            {
              if(rat[ch] < 1.5)
              {
//                printf("turn off trigger\n");
                trigger[ch] = 0;
                sw50[ch]=0;
              }
            }
          }

          if(sw4[ch] < NBSCALE)                        // SCALE DATA OVER 5 BUFFERS ?
          {
            for(k=0;k<ns[ch];k++)
            {
              v[ch]=v[ch]+(abs)(fseismo[k]);
              teller[ch]++;
            }
            promedio[ch]=(float)v[ch]/(float)teller[ch];

            factor[ch] = YSCALE / (100.0 * promedio[ch]);
			if(sw4[ch]<(NBSCALE-3))
			  scale_factor[ch]=factor[ch];

            sw4[ch]++;
          }
        }
        if(fl == 0.0 && fh == 0.0)               // FILTER DATA ?
        {

          if(ch == 0)                            // IF CHANNEL 0 ?
          {
            rescale_counter++;            // INCREMENT COUNTER
            if(rescale_counter >= 500)    // FILTER-RESCALE ?
            {
              printf("Rescale\n");
              Rescale();
              rescale_counter = 0;
            }
          }
        }

        for(k=0;k<ns[ch];k++)
        {
          if(auto_scaling == 0)
          {
            values[ch][ix[ch]]=fseismo[k] * scale_factor[ch] * amp * amp_single[ch];
          }else{
            values[ch][ix[ch]]=fseismo[k];
          }
          ix[ch]++;
          if(ix[ch] >= xmax)
          {
            for(m=0;m<(TOTSMP);m++)              // SHIFT ALL SAMPLES 'NS' LEFT
            {
              values[ch][m]=values[ch][m+ns[ch]];
              trgflg[ch][m]=trgflg[ch][m+ns[ch]];
              trgmrk[ch][m]=trgmrk[ch][m+ns[ch]];
            }

            if(marker1 != 0 && marker2 != 0)     // IF MARKERS ON, PLOT THEM
            { 
              trgflg[ch][marker1-ns[ch]]=1;
              trgmrk[ch][marker2-ns[ch]]=1;
            }

            ix[ch] = ix[ch] - ns[ch];
            stpix[ch] = (60.0 - ftime[ch]) * incr[ch] * sr[ch];            
          }

        }

        if(sw1[ch]==0)
        {
          yold=values[ch][0];
          ynew=values[ch][0];
          stpix[ch] = (60.0 - ftime[ch]) * incr[ch] * sr[ch];
          sw1[ch]=1;
        }

        sscanf(tid[ch],"%4d%1c%3d%1c%2d%1c%2d%1c%f4.1",&YR,dum,&DY,dum,&ihrn,dum,&imin,dum,&sek);
        hrn=ihrn;
        min=imin;
//  printf("%4d %3d %2d %2d %4.1f\n",YR,DY,hrn,min,sek);
//-- convert from doy to date  
        DTE (DY,&DDAY,&MMTH,YR);
        yrn=YR;
        mon=MMTH;
        day=DDAY;
//      printf("TIME: %4d %2d %2d %2d %2d %4.1f\n",yrn,mon,day,hrn,min,sek);
        TIMSEC(yrn,mon,day,hrn,min,sek,&MSECS);
		if(stopwatch == 0)
		{
		  TIMESTART = MSECS;
		  stopwatch = 1;
		}
		TIMESTOPP = MSECS;
		TIMEDIFF = TIMESTOPP - TIMESTART;
//-- convert from doy to date  
        DTE (rday[ch],&DDAY,&MMTH,ryrn[ch]);
        yrn=ryrn[ch];
        mon=MMTH;
        day=DDAY;
        rsek[ch]=risec[ch]+(float)(rsecs[ch]/10000.0);
//      printf("TIME: %4d %2d %2d %2d %2d %4.1f\n",yrn,mon,day,rhrn[ch],rmin[ch],rsek[ch]);
        TIMSEC(yrn,mon,day,rhrn[ch],rmin[ch],rsek[ch],&MSECSR);
//        printf("ch: %2d  Latency: %6.2f\n",ch,MSECSR-MSECS);
        latsnitt[ch]=latsnitt[ch]+MSECSR-MSECS;
        counter[ch]++;
        sprintf(dum,"%6.2f",MSECSR-MSECS);
        sprintf(lat[ch],"%s",dum);


        strip(ch);                               // PLOT THIS CHANNEL IN CORRESPONDING VIEWPORT

// Flush the pipeline, swap the buffers
        glFlush();
        glutSwapBuffers();
        if ( RunMode==1 ) 
        {
          glutPostRedisplay();	// Trigger an automatic redraw for animation
        }
      }
    }else{
#ifdef WIN32
  Sleep(1000);
#else
     sleep(1);                                  // WHEN IN 'HOLD MODE'
#endif
    }

    if ( RunMode==1 ) 
    {
      glutPostRedisplay();	// Trigger an automatic redraw for animation
    }
  }
}



void strip(int line)
{
  int DDAY,MMTH;
  int DY;
  int YR;
  long   yrn;
  long   mon;
  long   day;
  long   hrn;
  long   min;

  int   iyrn;
  int   imon;
  int   iday;
  int   ihrn;
  int   imin;
  float  sek;
  char dum[50];
  char   timestamp[80];
  int space;
  int x_txt=0;
  int y_txt=0;
  float x_tid=0.0;
  float y_tid=0.0;
  float y_station=0.0;

  float x1,y1,x2;
  float min_mark;
  float add=0.0;
  float maxval=0.0;
  float max=0.0;
//  float smp;
  int   xpos;
  int   p;
  char  tekst[80];
  int minut=(int)MINUTES;
//printf("WINH: %d no_to_plot:%d\n",WINH,no_to_plot);
  space = WINH/no_to_plot;

  if(init_sw == 0)                               // CLEAR VIEWPORT INITIALLY
  {
    x_txt=-WINL/2+4;
    y_txt=WINH/2-15 * no_to_plot;

    for(p=0;p<no_to_plot;p++)
    {
      xpos=WINH-(p+1)*space;
      glViewport(0,xpos,WINL,space);
      glColor3f(BGND_R, BGND_G, BGND_B);
      glRecti(-WINL/2,-WINH/2,WINL/2,WINH/2);
      if(demo_version == 0)
      {
        glColor3f(DETE_R, DETE_G, DETE_B);       // PLOT STATION NAME IN RED INITIALLY
        sprintf(tekst,"%s",toplot[p]);
//  bitmap_output(x_txt, y_txt, tekst,GLUT_BITMAP_HELVETICA_18);
        bitmap_output(x_txt, y_txt, tekst,GLUT_BITMAP_9_BY_15 );
      }else{
        glColor3f(DETE_R, DETE_G, DETE_B);       // PLOT STATION FULL NAME IN RED INITIALLY
        sprintf(tekst,"%s",fullna[p]);
        bitmap_output(x_txt, y_txt, tekst,GLUT_BITMAP_HELVETICA_18);
//        bitmap_output(x_txt, y_txt, tekst,GLUT_BITMAP_9_BY_15 );
      }
      glFlush();
    }
    glFlush();
    init_sw=1;
  }

  xpos=WINH-(line+1)*space;

  x_txt=-WINL/2+4;
  y_txt=WINH/2-15 * no_to_plot;

  y_station=WINH/2-20 * no_to_plot;

  y_tid=space;
  x_tid=(float)WINL/2.0;

  glViewport(0,xpos,WINL,space);

  x1=-WINL/2;
  x2= WINL/2;
  y1= 1.0 - (line+1) * space * 2.0/(float)space;

  glColor3f(BGND_R, BGND_G, BGND_B);             // ERASE VIEWPORT WITH FILLED RECTANGLE
  glRecti(-WINL/2,-WINH/2,WINL/2,WINH/2);


  glBegin( GL_LINES );                           // PLOT THE SIGNAL
  if(trigger[line] == 1)                         // IF IN DETECTION MODE CHANGE COLOR
  {
    glColor3f(DETE_R, DETE_G, DETE_B);
  }else{
    glColor3f(DRAW_R, DRAW_G, DRAW_B);           // IF NOT, USE NORMAL DRAWING COLOR
  }
  if(auto_scaling == 1)
  {
    max=0.0;
    for(i=0;i<TOTSMP;i++)
    {
      if(fabs((double)values[line][i]) > max)
        max=fabs((double)(values[line][i]));
    }
  }


//printf("TOTSMP:%d\n",TOTSMP);

  for(i=0;i<TOTSMP;i++)                          // DRAW EACH SAMPLE
  {
    glVertex2f( xold, yold );
    glVertex2f( xnew, ynew );
    xold=xnew;
    yold=ynew;
    xnew=xnew+incr[line];
    if(auto_scaling == 1)
    {
      ynew=values[line][i]/max * ((float)WINH/2-2.0);
    }else{
      ynew=values[line][i];
    }
  }
  glEnd();

  xold=-(WINL/2);                                // SET X BACK TO START OF VIEWPORT
  xnew=-(WINL/2);
  glBegin( GL_LINES );                           // PLOT PICKS IF ANY
  glColor3f(DETE_R, DETE_G, DETE_B);
  for(i=0;i<TOTSMP;i++)
  {
    if(trgflg[line][i]==1)
    {
      
      glVertex2f( xold, WINH/2 );
      glVertex2f( xnew, -WINH/2 );
    }
    xold=xnew;
    yold=ynew;
    xnew=xnew+incr[line];

  }
  glEnd();


  xold=-(WINL/2);                                // SET X BACK TO START OF VIEWPORT
  xnew=-(WINL/2);
  glBegin( GL_LINES );                           // PLOT PICKS , AT 'IXTRG'
  glColor3f(PICK_R, PICK_G, PICK_B);
  for(i=0;i<TOTSMP;i++)
  {
    if(trgmrk[line][i]==1)
    {
      
      glVertex2f( xold, WINH/2-2 );
      glVertex2f( xnew, -WINH/2+2 );
    }
    xold=xnew;
    yold=ynew;
    xnew=xnew+incr[line];

  }
  glEnd();


  glBegin( GL_LINES );                           // PLOT THE AXIS
  glColor3f(AXIS_R, AXIS_G, AXIS_B);
  glVertex2f( -WINL/2, -WINH/2+1 );
  glVertex2f( WINL/2, -WINH/2+1 );
  for(i=0;i<minut;i++)                           // PLOT MINUTE MARKS
  {
    add=i * 60.0 * sr[line] * incr[line];
    min_mark=-WINL/2 + stpix[line]+add;
    glVertex2f( min_mark, -WINH/2+50 );
    glVertex2f( min_mark, -WINH/2 );
  }
  glEnd();
  
  if(demo_version == 0)
  {
    glColor3f(NAME_R, NAME_G, NAME_B);           // PLOT STATION NAME
    if(current_channel == line)                  // CURRENT (FIRST) IN WHITE
      glColor3f(CURN_R, CURN_G, CURN_B);         // THE REST IN GREEN
    sprintf(tekst,"%s",toplot[line]);
//    bitmap_output(x_txt, y_txt, tekst,GLUT_BITMAP_HELVETICA_18);
    bitmap_output(x_txt, y_txt, tekst,GLUT_BITMAP_9_BY_15 );
  }else{
    glColor3f(NAME_R, NAME_G, NAME_B);           // PLOT STATION FULL NAME
    if(current_channel == line)                  // CURRENT (FIRST) IN WHITE
      glColor3f(CURN_R, CURN_G, CURN_B);         // THE REST IN GREEN
    sprintf(tekst,"%s",fullna[line]);
    bitmap_output(x_txt, y_txt, tekst,GLUT_BITMAP_HELVETICA_18);
//    bitmap_output(x_txt, y_txt, tekst,GLUT_BITMAP_9_BY_15 );
  }

  glColor3f(DATE_R, DATE_G, DATE_B);             // PLOT TIME


  sscanf(rtid[line],"%4d%1c%3d%1c%2d%1c%2d%1c%f6.3",&YR,dum,&DY,dum,&ihrn,dum,&imin,dum,&sek);
  hrn=ihrn;
  min=imin;
//  printf("%4d %3d %2d %2d %4.1f\n",YR,DY,hrn,min,sek);
//-- convert from doy to date  
  DTE (DY,&DDAY,&MMTH,YR);
  yrn=YR;
  yrn=yrn-2000;
  mon=MMTH;
  day=DDAY;
  iday=day;
  imon=mon;
  iyrn=yrn;
  ihrn=hrn;
  imin=min;
  sprintf(timestamp,"%2d/%2d/%2d %2d:%2d:%6.3f",iday,imon,iyrn,ihrn,imin,sek);
  if(timestamp[0] == ' ') timestamp[0] = '0';
  if(timestamp[3] == ' ') timestamp[3] = '0';
  if(timestamp[6] == ' ') timestamp[6] = '0';
  if(timestamp[9] == ' ') timestamp[9] = '0';
  if(timestamp[12] == ' ') timestamp[12] = '0';
  if(timestamp[15] == ' ') timestamp[15] = '0';
  if(timestamp[18] == ' ') timestamp[18] = '0';
  if(timestamp[19] == ' ') timestamp[19] = '0';
  if(timestamp[20] == ' ') timestamp[20] = '0';

  sprintf(tekst,"%s",rtid[line]);

  if(demo_version == 0)
  {
    bitmap_output(WINL/2-290, y_txt, tekst,GLUT_BITMAP_8_BY_13 );
    glColor3f(LATE_R, LATE_G, LATE_B);           // PLOT LATENCY
    sprintf(tekst,"%s",lat[line]);
    bitmap_output(WINL/2-100, y_txt, tekst,GLUT_BITMAP_8_BY_13 );

    glColor3f(RATI_R, RATI_G, RATI_B);           // PLOT SNR
    sprintf(tekst,"%s",snr[line]);
    bitmap_output(WINL/2-50, y_txt, tekst,GLUT_BITMAP_8_BY_13 );
  }else{
    bitmap_output(WINL/2-200, y_txt, timestamp,GLUT_BITMAP_HELVETICA_18);
  }
  xold=-(float)(WINL/2);                         // SET X BACK TO START OF VIEWPORT
  xnew=-(float)(WINL/2);

  if(time_quality[line] == 1)                    // PLOT SQUARE TO MARK GPS STATUS
  {
    glColor3f(DETE_R, DETE_G, DETE_B);           // ONLY MARK IT IF OUT OF SYNC
	glRecti(WINL/2-10,WINH/2-100,WINL/2,WINH/2);
  }else{
    glColor3f(BGND_R, BGND_G, BGND_B);
	glRecti(WINL/2-10,WINH/2-100,WINL/2,WINH/2);
  }

  
  if(TIMEDIFF > 60.0)
  {
	stopwatch = 0;
	for(p=0;p<no_to_plot;p++)
	{
      xpos=WINH-(p+1)*space;
      glViewport(0,xpos,WINL,space);
	  if(counter[p] == 0)
	  {
        glColor3f(DETE_R, DETE_G, DETE_B); 
        sprintf(tekst,"NOT RECEIVING DATA");
        bitmap_output(-100, y_txt, tekst,GLUT_BITMAP_8_BY_13 );
	  }
	}
	for(p=0;p<no_to_plot;p++)
	  counter[p] = 0;
  }




}

int DET_MEM(int kan)
{
  float       sta1=0.0;               // STA at current sample
  float       lta1=0.0;               // LTA at current sample
  float       stamax;             // Maximum STA within buffer
  int nsta=4*sr[kan];                       // number of samples sta
  int nlta=400*sr[kan];                       // number of samples lta
  int ixmax;
  int ixtrg;
  sta1=csta[kan];
  lta1=clta[kan];
  STA_LTA(&sta1,&lta1,&stamax,kan,ns[kan],nsta,nlta,&ixmax,&ixtrg);
  if(hold[kan] < 5)
  {
    csta[kan]=sta1;
    clta[kan]=sta1;
    hold[kan]++;
  }else{
    csta[kan]=sta1;
    clta[kan]=lta1;
  }
  sprintf(snr[kan],"%4.1f",csta[kan]/clta[kan]);
  rat[kan]=csta[kan]/clta[kan];
//printf("%s\n",snr[kan]);
  return(ixtrg);
}
tu_read_stationlist ( const char * streamfile)
{
  FILE *streamfp;
  char linje[80];
  char station[80];
  char navn[80];
  char tempo[80];
  char filnam1[80];
//  char ch;
  int i,l,no;
  int br=0;
/* Open the station list file */
  sprintf(filnam1,"%s/par/%s",topdir_rt,streamfile);
  printf("Stationfile: %s\n",filnam1);
  if ((streamfp = fopen (filnam1, "rb")) == NULL)
  {
    printf("Can't open: %s\n",filnam1);
    exit(0);
  }
  if(antall == 1)
  {
    no = no_to_plot;
  }else{
    no = 200;
  }

  i=0;
	l=0;
  for(i=0;i<no;i++)
  {

	if((fgets(linje, 70, streamfp)) != NULL)
	{

      printf("%2d %s",i,linje);
	  sprintf(tempo,"%s",linje);
      for(l=0;l<80;l++)
	  {
	    station[l]='\0';
	    navn[l]='\0';
	  }
      strncpy(station,linje,10);
      sprintf(toplot[i], "%s",station);
//      printf("%s\n",toplot[i]);
      for(l=0;l<40;l++)
	    navn[l]=linje[l+11];
	  sprintf(fullna[i],"%s",navn);
	  for(l=0;l<40;l++)
	    if(fullna[i][l] == '\n') fullna[i][l] = '\0';
//	  printf("%s\n",fullna[i]);
	}else{
      break;
	}

  }
    
      printf("End of file. File contains: %2d items\n",i);
      no_to_plot = i;

    
    
}
/*--------------------------------------------------------------------------*/
/* Compute STA and LTA and store max STA                                    */
/*--------------------------------------------------------------------------*/
void STA_LTA(sta1,lta1,stamax,chan,ns,nsta,nlta,ixmax,ixtrg)


  float* sta1;              /* lta and sta sent as from the last data buffer*/
  float* lta1;              /* to be returned as the new values             */
  float* stamax;            /* to be returned as the max sta found          */
  int chan;                 /* number of channel to be analysed             */
  int ns;                   /* last sample to be analysed                   */
  int nsta;
  int nlta;
  int* ixmax;
  int* ixtrg;
{
register    int ix;
float       idx;                    /* Average difference in data value     */
                                    /* between 2 samples.                   */
//float*      firstbufptr;
//float*      nextbufptr;             /* pointer  next sample for the channel */
//int         total;                  /* total number of times around the loop*/
                                    /* is number of samples between the last*/
                                    /* and first sample for that channel,   */
                                    /* divided by the no_chan               */
int sww=0;               
int no_sta = nsta;
int no_lta = nlta;
float value;
/*----initialise max sta----*/

*stamax=0.0;
*ixmax=0;
*ixtrg=-1;
/*----analyse the samples belonging to chan for the length of the buffer--*/

for(ix=0; ix<ns; ix++)
{
/*--find the difference, then slide along ready for the next two points--*/

    idx=(float)(fabs((double)(fseismo[ix])));
    idx=idx * idx;

/*--calculate sta and lta, and assign max if necessary--*/
    *sta1 += (idx - *sta1)/no_sta;
    *lta1 += (*sta1 - *lta1)/no_lta+0.000001;
    if(*sta1 > *stamax)
    {
        *stamax = *sta1;
        *ixmax=ix;
    }
    if(sww==0)
    {
      value=*sta1 / *lta1;
      if(value > 2.5)
      {
        *ixtrg=ix;
        sww=1;
      }
    }

}
} /* end of sta_lta procedure */





stroke_output(GLfloat x, GLfloat y, char *format,...)
{
  va_list args;
  char buffer[200], *p;

  va_start(args, format);
  vsprintf(buffer, format, args);
  va_end(args);
  glPushMatrix();
  glTranslatef(x, y, 0);
  glScalef(0.005, 0.005, 0.005);
  for (p = buffer; *p; p++)
    glutStrokeCharacter(GLUT_STROKE_ROMAN, *p);
  glPopMatrix();
}
void bitmap_output(int x, int y, char *string, void *font)
{
  int len, i;

  glRasterPos2f(x, y);
  len = (int) strlen(string);
  for (i = 0; i < len; i++) {
    glutBitmapCharacter(font, string[i]);
  }
}
void 
my_reshape(int w, int h)
{
   GLdouble size;
   GLdouble aspect;

   /* Use the whole window. */
   glViewport(0, 0, w, h);

   /* We are going to do some 2-D orthographic drawing. */
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   size = (GLdouble)((w >= h) ? w : h) / 2.0;
   if (w <= h) {
      aspect = (GLdouble)h/(GLdouble)w;
      glOrtho(-size, size, -size*aspect, size*aspect, -100000.0, 100000.0);
   }
   else {
      aspect = (GLdouble)w/(GLdouble)h;
      glOrtho(-size*aspect, size*aspect, -size, size, -100000.0, 100000.0);
   }

   /* Make the world and window coordinates coincide so that 1.0 in */
   /* model space equals one pixel in window space.                 */
   glScaled(aspect, aspect, 1.0);

   /* Now determine where to draw things. */
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();

}

Set_Def_Color()
{
   BGND_R = BGND_D_R;
   BGND_G = BGND_D_G;
   BGND_B = BGND_D_B;

   DRAW_R = DRAW_D_R;
   DRAW_G = DRAW_D_G;
   DRAW_B = DRAW_D_B;

   NAME_R = NAME_D_R;
   NAME_G = NAME_D_G;
   NAME_B = NAME_D_B;

   DATE_R = DATE_D_R;
   DATE_G = DATE_D_G;
   DATE_B = DATE_D_B;

   LATE_R = LATE_D_R;
   LATE_G = LATE_D_G;
   LATE_B = LATE_D_B;

   DETE_R = DETE_D_R;
   DETE_G = DETE_D_G;
   DETE_B = DETE_D_B;

   RATI_R = RATI_D_R;
   RATI_G = RATI_D_G;
   RATI_B = RATI_D_B;

   PICK_R = PICK_D_R; 
   PICK_G = PICK_D_G;
   PICK_B = PICK_D_B;

   CURN_R = CURN_D_R;
   CURN_G = CURN_D_G;
   CURN_B = CURN_D_B;

   AXIS_R = AXIS_D_R;
   AXIS_G = AXIS_D_G;
   AXIS_B = AXIS_D_B;
}
Set_Color_2()
{
   BGND_R = 1.0;   
   BGND_G = 1.0;   // WHITE
   BGND_B = 1.0;

   DRAW_R = 0.0;
   DRAW_G = 0.0;   // BLACK
   DRAW_B = 0.0;

   NAME_R = 0.5;
   NAME_G = 0.5;   // GRAY
   NAME_B = 0.5;

   DETE_R = 1.0;
   DETE_G = 0.0;   // RED
   DETE_B = 0.0;

   DATE_R = 0.5;
   DATE_G = 0.5;   // GRAY
   DATE_B = 0.5;

   RATI_R = 0.5;
   RATI_G = 0.5;   // GRAY
   RATI_B = 0.5;

   LATE_R = 0.5;
   LATE_G = 0.5;   // GRAY
   LATE_B = 0.5;

   PICK_R = 1.0; 
   PICK_G = 0.0;   // RED
   PICK_B = 0.0;

   CURN_R = 0.0;
   CURN_G = 0.0;   // BLACK
   CURN_B = 0.0;

   AXIS_R = 0.5;
   AXIS_G = 0.5;   // GRAY
   AXIS_B = 0.5;
}
Set_Color_3()
{
   BGND_R = 0.7;
   BGND_G = 0.9;   // VERY LIGHT BLUE
   BGND_B = 0.9;

   DRAW_R = 0.0;
   DRAW_G = 0.0;   // BLACK
   DRAW_B = 0.0;

   NAME_R = 0.0;
   NAME_G = 0.0;   // BLUE
   NAME_B = 1.0;

   DETE_R = 1.0;
   DETE_G = 0.0;   // RED
   DETE_B = 0.0;

   DATE_R = 0.0;
   DATE_G = 0.0;   // BLUE
   DATE_B = 1.0;

   RATI_R = 0.0;
   RATI_G = 0.0;   // BLUE
   RATI_B = 1.0;

   LATE_R = 0.0;
   LATE_G = 0.0;   // BLUE
   LATE_B = 1.0;

   PICK_R = 1.0; 
   PICK_G = 0.0;   // RED
   PICK_B = 0.0;

   CURN_R = 0.5;
   CURN_G = 0.5;   // RED
   CURN_B = 0.5;

   AXIS_R = 0.5;
   AXIS_G = 0.5;   // GRAY
   AXIS_B = 0.5;
}
Set_Color_4()
{
   BGND_R = 1.0;
   BGND_G = 0.6;   // OKER
   BGND_B = 0.0;

   DRAW_R = 0.0;
   DRAW_G = 0.0;   // BLACK
   DRAW_B = 0.0;

   NAME_R = 0.0;
   NAME_G = 0.0;   // BLUE
   NAME_B = 1.0;

   DETE_R = 1.0;
   DETE_G = 0.0;   // RED
   DETE_B = 0.0;

   DATE_R = 0.0;
   DATE_G = 0.0;   // BLUE
   DATE_B = 1.0;

   RATI_R = 0.0;
   RATI_G = 0.0;   // BLUE
   RATI_B = 1.0;

   LATE_R = 0.0;
   LATE_G = 0.0;   // BLUE
   LATE_B = 1.0;

   PICK_R = 1.0; 
   PICK_G = 0.0;   // RED
   PICK_B = 0.0;

   CURN_R = 0.5;
   CURN_G = 0.5;   // RED
   CURN_B = 0.5;

   AXIS_R = 0.5;
   AXIS_G = 0.5;   // GRAY
   AXIS_B = 0.5;
}
void 
my_menu(int key)
{
  int l;
printf("%d\n",key);
   switch (key) {

   case 27:        // ESC exit
   exit(0);
   break;
   case 117:       // u amplify all channels
   amp=amp+AMP_STEP;
   break;
   case 100:       // d decrease all channel
   amp=amp-AMP_STEP;
   if(amp <= 0.0)
     amp=AMP_STEP;
   break;
   case 83:        // S Hold
   start_stop=1;
   break;
   case 115:       // s Resume
   start_stop=0;
   break;
   case 110:       // n next channel
   current_channel++;
   if(current_channel >= no_to_plot)
     current_channel = 0;
   break;
   case 43:        // + increase amplitude
   amp_single[current_channel]=amp_single[current_channel]+4.0;
   break;
   case 45:        // - decrease amplitude
   amp_single[current_channel]=amp_single[current_channel]-4.0;
   if(amp_single[current_channel] <= 0.0)
     amp_single[current_channel] = 1.0;
   break;
   case 70:        // F filter all
   if(fl == 0.0)
     fl=FL_PRESET;
   if(fh == 0.0)
     fh=FH_PRESET;
   FilterRescale();
   for(l=0;l<no_to_plot;l++)
   {
     sw2[l]=50;
   }

   break;
   case 102:       // f no filter
   fl=0.0;
   fh=0.0;
   amp=1.0;
   for(l=0;l<no_to_plot;l++)
   {
     sw2[l]=0;
     teller[l]=0;
     v[l]=0.0;
     sum[l]=0.0;
   }
   for(l=0;l<no_to_plot;l++)
     amp_single[l]=1.0;
   break;
   case 116:
   trgflg[3][ix[3]]=1;
   break;
   case 104:       // help
   printf("**************************************************************\n");
   printf("* List of key-press functions:                               *\n");
   printf("* ----------------------------                               *\n");
   printf("* h  : list this on screen                                   *\n");
   printf("* esc: exit                                                  *\n");
   printf("* u  : increase amplitude on all channels                    *\n");
   printf("* d  : decrease amplitude on all channels                    *\n");
   printf("* S  : freeze graphics                                       *\n");
   printf("* s  : resume graphics                                       *\n");
   printf("* n  : next channel                                          *\n");
   printf("* +  : increase amplitude on current channel                 *\n");
   printf("* -  : decrease amplitude on current channel                 *\n");
   printf("* F  : turn on pre-set filter                                *\n");
   printf("* f  : turn off filtering                                    *\n");
   printf("* 1  : Set color scheme to default                           *\n");
   printf("* 2  : Color scheme 2                                        *\n");
   printf("* 3  : Color scheme 3                                        *\n");
   printf("* 4  : Color scheme 4                                        *\n");
   printf("**************************************************************\n"); 
   break;
   case 49:        // Set default colors
   Set_Def_Color();
   break;
   case 50:        // Color scheme 2
   Set_Color_2();
   break;
   case 51:        // Color scheme 3
   Set_Color_3();
   break;
   case 52:        // Color scheme 4
   Set_Color_4();
   break;
   default:
      break;
   }
  printf("%4.1f %4.1f %4.1f\n",R,G,B);
}
void 
my_handle_key(unsigned char key, int x, int y)
{
  int l;
printf("%d\n",key);
   switch (key) {

   case 27:        // ESC exit
   exit(1);
   break;
   case 117:       // u amplify all channels
   amp=amp+AMP_STEP;
   break;
   case 100:       // d decrease all channel
   amp=amp-AMP_STEP;
   if(amp <= 0.0)
     amp=AMP_STEP;
   break;
   case 83:        // S Hold
   start_stop=1;
   break;
   case 115:       // s Resume
   start_stop=0;
   break;
   case 110:       // n next channel
   current_channel++;
   if(current_channel >= no_to_plot)
     current_channel = 0;
   break;
   case 43:        // + increase amplitude
   amp_single[current_channel]=amp_single[current_channel]+4.0;
   break;
   case 45:        // - decrease amplitude
   amp_single[current_channel]=amp_single[current_channel]-4.0;
   if(amp_single[current_channel] <= 0.0)
     amp_single[current_channel] = 1.0;
   break;
   case 70:        // F filter all
   if(fl == 0.0)
     fl=FL_PRESET;
   if(fh == 0.0)
     fh=FH_PRESET;
   FilterRescale();
   for(l=0;l<no_to_plot;l++)
   {
     sw2[l]=50;
   }

   break;
   case 102:       // f no filter
   fl=0.0;
   fh=0.0;
   amp=1.0;
   for(l=0;l<no_to_plot;l++)
   {
     sw2[l]=0;
     teller[l]=0;
     v[l]=0.0;
     sum[l]=0.0;
   }
   for(l=0;l<no_to_plot;l++)
     amp_single[l]=1.0;
   break;
   case 116:
   trgflg[3][ix[3]]=1;
   break;
   case 104:       // help
   printf("**************************************************************\n");
   printf("* List of key-press functions:                               *\n");
   printf("* ----------------------------                               *\n");
   printf("* h  : list this on screen                                   *\n");
   printf("* esc: exit                                                  *\n");
   printf("* u  : increase amplitude on all channels                    *\n");
   printf("* d  : decrease amplitude on all channels                    *\n");
   printf("* S  : freeze graphics                                       *\n");
   printf("* s  : resume graphics                                       *\n");
   printf("* n  : next channel                                          *\n");
   printf("* +  : increase amplitude on current channel                 *\n");
   printf("* -  : decrease amplitude on current channel                 *\n");
   printf("* F  : turn on pre-set filter                                *\n");
   printf("* f  : turn off filtering                                    *\n");
   printf("* 1  : Set color scheme to default                           *\n");
   printf("* 2  : Color scheme 2                                        *\n");
   printf("* 3  : Color scheme 3                                        *\n");
   printf("* 4  : Color scheme 4                                        *\n");
   printf("**************************************************************\n"); 
   break;
   case 49:        // Set default colors
   Set_Def_Color();
   break;
   case 50:        // Color scheme 2
   Set_Color_2();
   break;
   case 51:        // Color scheme 3
   Set_Color_3();
   break;
   case 52:        // Color scheme 4
   Set_Color_4();
   break;
   default:
      break;
   }
//  printf("%4.1f %4.1f %4.1f\n",R,G,B);
}
void FilterRescale()
{
  int l;

  amp=1.0;
  for(l=0;l<nch;l++)
  {
    sw4[l]=0;
    teller[l]=0;
    v[l]=0.0;
    sum[l]=0.0;
	factor[l]=0.0;
	promedio[l]=0.0;
//	scale_factor[l]=0.0;
  }
  for(l=0;l<nch;l++)
    amp_single[l]=4.0;
}
/*************************************/
/* RE-INITIALIZE ALL SCALING VALUES  */
/*************************************/
void Rescale()
{
  int l;
  amp=1.0;
  for(l=0;l<nch;l++)
  {
    sw2[l]=0;
	sw4[l]=0;
    teller[l]=0;
    v[l]=0.0;
    sum[l]=0.0;
    avg[l]=0.0;
	factor[l]=0.0;
//	scale_factor[l]=0.0;
	promedio[l]=0.0;
  }
  for(l=0;l<nch;l++)
    amp_single[l]=4.0;
}
void processMousePassiveMotion(int x, int y) {
  printf("pass: %4d,%4d\n",x,y);

}

void processMouseActiveMotion(int x, int y) 
{
  int kanal;
  int space;
  printf("%4d,%4d\n",x,y);
  space = WINH/no_to_plot;
  kanal=(WINH-y)/space-1;
  printf("kanal: %d\n",kanal);
}

void processMouse(int button, int state, int x, int y) 
{
  int kanal;
  int space;
  char syscmd[200];
  char arg1[10];
  int rt1;
  
//  sleep
// if a mouse button is pressed  then
  if ((state == GLUT_DOWN)) 
  {
    if (button == GLUT_LEFT_BUTTON) 
	{
      printf("left: %5d,%5d\n",x,y);
  space = WINH/no_to_plot;
  kanal=y/space;
  printf("kanal: %d\n",kanal);
  sprintf(arg1,"-a %d",kanal);
  sprintf(syscmd,"rtnet -a %d -x 1000 -y 200 -l %s -f %s %s&\n",kanal,STREAMS_SERVER,STATIONS_SERVER,ADDR_SERVER);
#ifdef WIN32
  _spawnl(_P_NOWAIT,"c:\\VS\\RTNET\\debug\\rtnet.exe","rtnet.exe",arg1,"-y 200","129.177.55.30:18000",NULL);
#else
  printf("%s\n",syscmd);
  rt1=system(syscmd);
#endif
	}
		else if (button == GLUT_MIDDLE_BUTTON) {
			printf("midl: %5d,%5d\n",x,y);
		}
		else {
			printf("righ: %5d,%5d\n",x,y);
		}
	}
}


int main(int argc, char **argv)
{

/*****************/
/* COLOR SCHEMES */
/*****************/
   BGND_R = BGND_D_R;
   BGND_G = BGND_D_G;
   BGND_B = BGND_D_B;

   DRAW_R = DRAW_D_R;
   DRAW_G = DRAW_D_G;
   DRAW_B = DRAW_D_B;

   NAME_R = NAME_D_R;
   NAME_G = NAME_D_G;
   NAME_B = NAME_D_B;

   DATE_R = DATE_D_R;
   DATE_G = DATE_D_G;
   DATE_B = DATE_D_B;

   LATE_R = LATE_D_R;
   LATE_G = LATE_D_G;
   LATE_B = LATE_D_B;

   DETE_R = DETE_D_R;
   DETE_G = DETE_D_G;
   DETE_B = DETE_D_B;

   RATI_R = RATI_D_R;
   RATI_G = RATI_D_G;
   RATI_B = RATI_D_B;

   PICK_R = PICK_D_R; 
   PICK_G = PICK_D_G;
   PICK_B = PICK_D_B;

   CURN_R = CURN_D_R;
   CURN_G = CURN_D_G;
   CURN_B = CURN_D_B;

   AXIS_R = AXIS_D_R;
   AXIS_G = AXIS_D_G;
   AXIS_B = AXIS_D_B;


//------------------------------------------------------------------------
// set path to RTQUAKE_TOP as default
//------------------------------------------------------------------------
  topdir_rt = (char*)getenv("RTQUAKE_TOP");
  if (topdir_rt)
  {
    printf("RTNET: RTQUAKE_TOP: %s\n", topdir_rt);
  }else{
    printf("RTNET: RTQUAKE_TOP not defined\n");
    printf("Include setup_rt.bash or setup_rt.csh in your .bashrc or .csh\n");
    exit(0);      
  } 
  fl=0.0;
  fh=0.0;
  
// Allocate and initialize a new connection description SEEDLINK
  slconn = sl_newslcd();

// Process command line parameters
  if (parameter_proc (argc, argv) < 0)
  {
    fprintf(stderr, "Parameter processing failed\n\n");
    fprintf(stderr, "Try '-h' for detailed help\n");
    return -1;
  }

  xold=-(float)(WINL/2);                         // SET X BACK TO START OF VIEWPORT
  xnew=-(float)(WINL/2);


  if(minutter == 0)
  {
    TOTSMP    = TOTALSAMP;
    MINUTES   = TOTALMINU;
  }else{
    switch(minutter)
	{
	  case 1:
	    TOTSMP = 6000;
	    MINUTES= 1.0;
TOTSMP=30000;
	  break;
	  case 2:
	    TOTSMP = 12000;
TOTSMP=60000;
	    MINUTES= 2.0;
	  break;
	  case 3:
	    TOTSMP = 18000;
TOTSMP=90000;
	    MINUTES= 3.0;
	  break;
	  case 4:
	    TOTSMP = 24000;
TOTSMP=120000;
	    MINUTES= 4.0;
	  break;
	  case 5:
	    TOTSMP = 30000;
TOTSMP=150000;
	    MINUTES= 5.0;
	  break;
	  case 10:
	    TOTSMP = 60000;
TOTSMP=300000;
	    MINUTES= 10.0;
	  break;
	  case 15:
	    TOTSMP = 90000;
TOTSMP=450000;
	    MINUTES= 15.0;
	  break;
	  case 30:
            TOTSMP = 180000;
TOTSMP=900000;
	    MINUTES= 30.0;
	  break;
	  case 60:
            TOTSMP = 360000;
	    MINUTES= 60.0;
	  break;
	  default:
            TOTSMP    = TOTALSAMP;
            MINUTES   = TOTALMINU;
	  break;
	}
  }
  if(only_one == 1)
  {
    TOTSMP    = 12000;
    MINUTES   = 2.0;
  }
  Rescale();                                     // RE-INITIALIZE SCALE VALUES
  FilterRescale();
  for(i=0;i<nch;i++)                      // INITIALIZE 
  {
    sw50[i]=0;
    rotate[i]=0;
  }


  /* Make sure everything is shut down and save the state file */
//  if (slconn->link != -1)
//    sl_disconnect (slconn);

//  if (statefile)
//    sl_savestate (slconn, statefile);

//*******************************************************************
// Set up for OpenGL graphics
//*******************************************************************
  glutInit(&argc,argv);

// We're going to animate it, so double buffer 
  if(sindob == 0)
  {
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH );
  }else{
    glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB | GLUT_DEPTH );
  }
// Window position (from top corner), and size (width% and hieght)
  glutInitWindowPosition( xo, yo );

// Window position (from top corner), and size (width% and hieght)
//  glutInitWindowPosition( 0, 0 );

// Set window size in pixels
  glutInitWindowSize( WINL, WINH );

// Create window with title given in parameters
  glutCreateWindow( "REAL TIME DATA" );

  glColor3f(0.0, 0.0, 0.0);
  glRecti(-WINL/2,-WINH/2,WINL/2,WINH/2);

// Detects a key-press. ESC exits program
   glutKeyboardFunc(my_handle_key);

//adding here the mouse processing callbacks
	glutMouseFunc(processMouse);
//	glutMotionFunc(processMouseActiveMotion);
//    glutPassiveMotionFunc(processMousePassiveMotion);

  glutCreateMenu(my_menu);
  glutAddMenuEntry("Increase amplitude all channels", 117);
  glutAddMenuEntry("Decrease amplitude all channels", 100);
  glutAddMenuEntry("Freeze graphics", 83);
  glutAddMenuEntry("Resume graphics", 115);
  glutAddMenuEntry("Set next channel", 110);
  glutAddMenuEntry("Increase amplitude current channel", 43);
  glutAddMenuEntry("Decrease amplitude current channel", 45);
  glutAddMenuEntry("Turn on pre-set filter", 70);
  glutAddMenuEntry("Turn off pre-set filter", 102);
  glutAddMenuEntry("Set default colors", 49);
  glutAddMenuEntry("Color scheme 1", 50);
  glutAddMenuEntry("Color scheme 2", 51);
  glutAddMenuEntry("Color scheme 3", 52);
  glutAddMenuEntry("Quit", 27);
  glutAttachMenu(GLUT_RIGHT_BUTTON);


// call this whenever window needs redrawing
  glutDisplayFunc( drawScene );

  glutReshapeFunc(my_reshape);


// Start the main loop.  glutMainLoop never returns.
  glutMainLoop(  );

  return 0;
}				/* End of main() */



/***************************************************************************
 * packet_handler():
 * Process a received packet based on packet type.
 ***************************************************************************/
static int
packet_handler (char *msrecord, int packet_type, int seqnum, int packet_size)
{
  static SLMSrecord * msr = NULL;
  int trace;
  double dtime;			/* Epoch time */
  double secfrac;		/* Fractional part of epoch time */
  time_t itime;			/* Integer part of epoch time */
  char timestamp[20];
  struct tm *timep;

  /* The following is dependent on the packet type values in libslink.h */
  char *type[]  = { "Data", "Detection", "Calibration", "Timing",
		    "Message", "General", "Request", "Info",
                    "Info (terminated)", "KeepAlive" };

  /* Build a current local time string */

  dtime   = sl_dtime ();
  secfrac = (double) ((double)dtime - (int)dtime);
  itime   = (time_t) dtime;
  timep   = localtime (&itime);
  sprintf (timestamp,  "%04d.%03d.%02d:%02d:%02d.%01.0f",
	    timep->tm_year + 1900, timep->tm_yday + 1, timep->tm_hour,
	    timep->tm_min, timep->tm_sec, secfrac);

  /* Process waveform data */
  if ( packet_type == SLDATA )
    {
      sl_log (0, 1, "%s, seq %d, Received %s blockette:\n",
	      timestamp, seqnum, type[packet_type]);

      sl_msr_parse (slconn->log, msrecord, &msr, 1, 0);

      if ( verbose || ppackets )
	sl_msr_print (slconn->log, msr, ppackets);
    }
  else if ( packet_type == SLKEEP )
    {
      sl_log (0, 2, "Keep alive packet received\n");
    }
  else
    {
      sl_log (0, 1, "%s, seq %d, Received %s blockette\n",
	      timestamp, seqnum, type[packet_type]);
    }

  /* Write packet to dumpfile if defined */
  if ( dumpfile )
    {
      if (fwrite (msrecord, packet_size, 1, outfile) <= 0)
	sl_log (2, 0, "fwrite(): error writing data to %s\n", dumpfile);
//      memcpy(buf,msrecord,seed_size);
	sl_msr_parse (slconn->log, msrecord, &msr, 1, 1);
        trace=unpack_samples (msr);
        return(1);
    }
/* Plot selected data */
if( location )
{
//  printf("%s, seq %d, Received %s blockette:\n",timestamp, seqnum, type[packet_type]);
//  printf("smp:%s\n",timestamp);

  sl_msr_parse (slconn->log, msrecord, &msr, 1, 1);
  if(msr->unpackerr != MSD_UNKNOWNFORMAT)
  {
    trace=unpack_samples (msr,seqnum);
//printf("seqnum: %8d\n",seqnum);
    sprintf(tid[trace],"%s",timestamp);
    return(trace);
  }else{
	printf("Trouble unpacking buffer\n");
	return(0);
  }
}
return(0);

}				/* End of packet_handler() */

/***************************************************************************
 * print_samples:
 * Print samples in the supplied MSrecord with a simple format.
 ***************************************************************************/
int
unpack_samples (SLMSrecord *msr, int seqnum)
{
  char tempo[20];
  char search[20];
  char tmp[80];
  char dd[80];
  int ret2=0;
  int j,pp;
  int retur=-1;
  int cnt,p,l,ret;
//  char loc[3];
  for(p=0;p<20;p++)
    search[p]='\0';
  sprintf(tempo,"%s",msr->fsdh.station);

  strncpy(search,tempo,10);
  tempo[10]='\0';
//printf("%s %s\n",search,tempo);
//  if(search[5] == ' ')search[5]='0';
//  if(search[6] == ' ')search[6]='0';
//printf("Station: %s\n",search);
  for(l=0;l<no_to_plot;l++)
  {
    ret=strcmp(search,toplot[l]);
    if(ret == 0)
    {
//printf("l: %d   %s   %s\n",l,search,toplot[l]);
      retur=l;
      if ( msr->datasamples != NULL )            //EXTRACT DATA FROM MINISEED BUFFER
      {
        if(msr->fsdh.samprate_fact > 0 && msr->fsdh.samprate_mult < 0)
		  msr->fsdh.samprate_fact = (-1 * msr->fsdh.samprate_fact)/msr->fsdh.samprate_mult;
//ret2=strcmp(search,"ARE0 00BHZ");
//if(ret2 == 0)
//		msr->fsdh.samprate_fact=40;

        ns[l]   = msr->numsamples;
        ryrn[l] = msr->fsdh.start_time.year;
        rday[l] = msr->fsdh.start_time.day;
        rhrn[l] = msr->fsdh.start_time.hour;
        rmin[l] = msr->fsdh.start_time.min;
        risec[l]= msr->fsdh.start_time.sec;
        rsecs[l]= msr->fsdh.start_time.fract; 
        sr[l]   = msr->fsdh.samprate_fact;

iryrn[l]=ryrn[l];
irday[l]=rday[l];
irhrn[l]=rhrn[l];
irmin[l]=rmin[l];

        sprintf(tmp,"%4d %3d %2d:%2d:%2d.%4d",iryrn[l],irday[l],irhrn[l],irmin[l],risec[l],rsecs[l]);
        if(tmp[21]==' ')tmp[21]='0';
        if(tmp[20]==' ')tmp[20]='0';
        if(tmp[19]==' ')tmp[19]='0';
        if(tmp[18]==' ')tmp[18]='0';
        if(tmp[15]==' ')tmp[15]='0';

//printf("%2d %2x\n",l,msr->fsdh.dq_flags);
//printf("%s  %4d\n",tmp,ns[l]);
        sprintf(rtid[l],"%s",tmp);
		if(msr->fsdh.dq_flags == 0)
		{
		  time_quality[l]=0;
		}else{
		  time_quality[l]=1;
		}
   
//        printf("NS: %3d %3d %3d   %s  %s  %8d\n",ns[l],msr->fsdh.samprate_fact,msr->fsdh.samprate_mult,msr->fsdh.station,rtid[l],seqnum);
        for(pp=0;pp<22;pp++)
          if(tmp[pp]==' ')tmp[pp]='0';   
        sscanf(tmp,"%15s%f7.4",dd,&ftime[l]);
//        printf("FTIME: %f\n",ftime[l]);



//incrtu
        incr[l]=(float)WINL/(60.0 * MINUTES * (float)msr->fsdh.samprate_fact);
//printf("K:%2d WINL:%d MINUTES:%f SR:%d\n",l,WINL,MINUTES,msr->fsdh.samprate_fact);
//printf("incr: %f\n",incr[l]);
/* COMPUTE NEW FILTER COEF. EACH BUFFER. THEY MAY HAVE DIFFERENT SAMPLERATE */
        eqavst=1.0/(float)msr->fsdh.samprate_fact;
        orden=4;
        if(fl != 0.0 && fh != 0.0)
        {
          design(orden,"BP","BU",30.0,0.3,fl,fh,eqavst);
          for(j=0;j<(nsects*3+1);j++)
          {
            snc[l][j]=sn[j];
            sdc[l][j]=sd[j];
//      printf("%d %f %f\n",j,sn[j],sd[j]);
          }
          cset[l]=nsects;
        }
        for ( cnt = 0; cnt < ns[l]; cnt++ )      // EXTRACT DATA SAMPLES
          seismo[cnt]=*(msr->datasamples + cnt);

        if(sw2[l] < NBSCALE)                     // RE-SCALE ?
        {
          for(p=0;p<ns[l];p++)
          {
            sum[l]=sum[l]+seismo[p];
            teller[l]++;
          }
          avg[l]=sum[l]/teller[l];               // DC

          for(p=0;p<ns[l];p++)                   // AVERAGE OF SIGNAL-DC
            v[l]=v[l]+(abs)(seismo[p]-avg[l]);
          promedio[l]=(float)v[l]/(float)teller[l];

          factor[l] = YSCALE / (100.0 * promedio[l]);
		  if(sw2[l]<(NBSCALE-3))
		    scale_factor[l]=factor[l];

          sw2[l]++;
        }  // sw2.........

      }    // if(msr->data.........
    }      // if(ret......
  }        // for(no_to_plot........ 
//  		if(l==140)
//			printf("unpack2 FAC: %10.7f\n",factor[l]);
  return(retur);
}  /* End of print_samples() */
/***************************************************************************
 * parameter_proc:
 *
 * Process the command line parameters.
 *
 * Returns 0 on success, and -1 on failure
 ***************************************************************************/
static int
parameter_proc (int argcount, char **argvec)
{
  int optind;
  int error = 0;

  char tm[80];  
  
 char *streamfile   = 0;	    /* stream list file for configuring streams */
 char streamsfull[80];
 char *stationfile  = 0;      /* station list file for reading streams */

  char *multiselect = 0;
  char *selectors   = 0;

  if (argcount <= 1)
    error++;

  /* Process all command line arguments */
  for (optind = 1; optind < argcount; optind++)
  {
      if (strcmp (argvec[optind], "-V") == 0)
      {
	  fprintf(stderr, "%s version: %s\n", PACKAGE, VERSION);
	  exit (0);
      }
      else if (strcmp (argvec[optind], "-h") == 0)
      {
	  usage();
	  exit (0);
      }
      else if (strcmp (argvec[optind], "-c") == 0)
      {
          stationfile ="stations.conf";
	  tu_read_stationlist (stationfile) ;
	  exit (0);
      }
      else if (strncmp (argvec[optind], "-v", 2) == 0)
      {
	  verbose += strspn (&argvec[optind][1], "v");
      }
      else if (strcmp (argvec[optind], "-p") == 0)
      {
	  ppackets = 1;
      }
      else if (strcmp (argvec[optind], "-xo") == 0)
      {
        xo = atoi(argvec[++optind]);
      }
      else if (strcmp (argvec[optind], "-yo") == 0)
      {
        yo = atoi(argvec[++optind]);
      }
      else if (strcmp (argvec[optind], "-buf") == 0)
      {
        sindob = atoi(argvec[++optind]);
      }
      else if (strcmp (argvec[optind], "-nt") == 0)
      {
	  slconn->netto = atoi (argvec[++optind]);
      }
      else if (strcmp (argvec[optind], "-nd") == 0)
      {
	  slconn->netdly = atoi (argvec[++optind]);
      }
      else if (strcmp (argvec[optind], "-k") == 0)
      {
	 slconn->keepalive = atoi (argvec[++optind]);
      }
      else if (strcmp (argvec[optind], "-m") == 0)
      {
	  minutter = atoi (argvec[++optind]);
      }
      else if (strcmp (argvec[optind], "-fl") == 0)
      {
	  fl = atof (argvec[++optind]);
      }
      else if (strcmp (argvec[optind], "-fh") == 0)
      {
	  fh = atof (argvec[++optind]);
      }
      else if (strcmp (argvec[optind], "-top") == 0)
      {
	  topdir_rt = argvec[++optind];
          printf("top directory: %s\n",topdir_rt);
      }
      else if (strcmp (argvec[optind], "-o") == 0)
      {
	  dumpfile = getoptval(argcount, argvec, optind++);
      }
      else if (strcmp (argvec[optind], "-g") == 0)
      {
	  location = getoptval(argcount, argvec, optind++);
      }
      else if (strcmp (argvec[optind], "-x") == 0)
      {
	  xlenght = atoi(argvec[++optind]);
          WINL=xlenght;
      }
      else if (strcmp (argvec[optind], "-y") == 0)
      {
	  yheight = atoi(argvec[++optind]);
          WINH=yheight;
      }

      else if (strcmp (argvec[optind], "-n") == 0)
      {
	  no_to_plot = atoi(argvec[++optind]);
          antall=1;
      }
      else if (strcmp (argvec[optind], "-a") == 0)
      {
	  stno = atoi(argvec[++optind]);
          only_one = 1;
          printf("stno: %d\n",stno);
      }

      else if (strcmp (argvec[optind], "-d") == 0)
      {

          demo_version = 1;
      }
      else if (strcmp (argvec[optind], "-sc") == 0)
      {
          auto_scaling = 1;
      }
      else if (strcmp (argvec[optind], "-l") == 0)
      {
	  streamfile = argvec[++optind];
      }
      else if (strcmp (argvec[optind], "-f") == 0)
      {
	  stationfile = argvec[++optind];
      }
      else if (strcmp (argvec[optind], "-s") == 0)
      {
	  selectors = argvec[++optind];
      }
      else if (strcmp (argvec[optind], "-S") == 0)
      {
	  multiselect = argvec[++optind];
          sprintf(window_title,"%s",multiselect);
      }
      else if (strncmp (argvec[optind], "-", 1 ) == 0)
      {
	  fprintf(stderr, "Unknown option: %s\n", argvec[optind]);
	  exit (1);
      }
      else if (!slconn->sladdr)
      {
	  slconn->sladdr = argvec[optind];
	  printf("ADDR: %s\n",argvec[optind]);
	  sprintf(ADDR_SERVER,"%s",argvec[optind]);
      }

      else
      {
	  fprintf(stderr, "Unknown option: %s\n", argvec[optind]);
	  exit (1);
      }
  }

  /* Make sure a server was specified */
  if ( ! slconn->sladdr )
    {
      fprintf(stderr, "No SeedLink server specified\n\n");
      fprintf(stderr, "Usage: %s [options] [host][:port]\n", PACKAGE);
      fprintf(stderr, "Try '-h' for detailed help\n");
      exit (1);
    }

  /* Initialize the verbosity for the sl_log function */
  sl_loginit (verbose, NULL, NULL, NULL, NULL);


  /* Open dumpfile if requested */
  if (dumpfile)
    {
      if (!strcmp (dumpfile, "-"))
	{
	  /* Re-direct all messages to standard error */
//	  sl_loginit (verbose, &print_stderr, NULL, &print_stderr, NULL);

	  outfile = stdout;
	  setvbuf (stdout, NULL, _IONBF, 0);
	}
      else if ((outfile = fopen (dumpfile, "a+")) != NULL)
	{
	  setvbuf (outfile, NULL, _IONBF, 0);
	}
      else
	{
	  sl_log (2, 0, "cannot open dumpfile: %s\n", dumpfile);
	  exit (1);
	}
    }


  
  /* Report the program version */
  sl_log (0, 1, "%s version: %s\n", PACKAGE, VERSION);
  
  /* If errors then report the usage message and quit */
  if (error)
    {
      usage ();
      exit (1);
    }

  /* If verbosity is 2 or greater print detailed packet infor */
  if ( verbose >= 2 )
    ppackets = 1;

  /* Load the stream list from a file if specified */
  if ( streamfile )
  {
    printf("STREAMFILE: %s\n",streamfile);
      sprintf(STREAMS_SERVER,"%s",streamfile);
    sprintf(streamsfull,"%s/par/%s",topdir_rt,streamfile);
    streamfile = streamsfull;
    printf("streamfile: %s\n",streamfile);
    sl_read_streamlist (slconn, streamfile, selectors);
  }else{
    sprintf(streamsfull,"%s/par/streams.conf",topdir_rt);
    streamfile = streamsfull;
    printf("streamfile: %s\n",streamfile);
    sl_read_streamlist (slconn, streamfile, selectors);
  }

  /* Load the station list from a file if specified */
  if ( stationfile )
  {
    tu_read_stationlist (stationfile);
  }else{
    stationfile = "stations.conf";
    tu_read_stationlist (stationfile);
  }
  

  sprintf(STATIONS_SERVER,"%s",stationfile);
  
  if(only_one == 1)
  {
    sprintf(tm,"%s",toplot[stno]);
    printf("tm: %s\n",tm);
    sprintf(toplot[0],"%s",tm);
    no_to_plot = 1;
  }
  
  /* Parse the 'multiselect' string following '-S' */
  if ( multiselect )
    {
      if ( sl_parse_streamlist (slconn, multiselect, selectors) == -1 )
	return -1;
    }
  else if ( !streamfile )
    {			 /* No 'streams' array, assuming uni-station mode */
      sl_setuniparams (slconn, selectors, -1, 0);
    }
  
  /* Attempt to recover sequence numbers from state file */
  if (statefile)
    {
      if (sl_recoverstate (slconn, statefile) < 0)
	{
	  sl_log (2, 0, "state recovery failed\n");
	}
    }
  
  return 0;
}				/* End of parameter_proc() */

/***************************************************************************
 * getoptval:
 * Return the value to a command line option; checking that the value is 
 * itself not an option (starting with '-') and is not past the end of
 * the argument list.
 *
 * argcount: total arguments in argvec
 * argvec: argument list
 * argopt: index of option to process, value is expected to be at argopt+1
 *
 * Returns value on success and exits with error message on failure
 ***************************************************************************/
static char *
getoptval (int argcount, char **argvec, int argopt)
{
  if ( argvec == NULL || argvec[argopt] == NULL ) {
    fprintf (stderr, "getoptval(): NULL option requested\n");
    exit (1);
  }
  
  /* Special case of '-o -' usage */
  if ( (argopt+1) < argcount && strcmp (argvec[argopt], "-o") == 0 )
    if ( strcmp (argvec[argopt+1], "-") == 0 )
      return argvec[argopt+1];
  
  if ( (argopt+1) < argcount && *argvec[argopt+1] != '-' )
    return argvec[argopt+1];

  fprintf (stderr, "Option %s requires a value\n", argvec[argopt]);
  exit (1);
}  /* End of getoptval() */

/***************************************************************************
 * usage:
 * Print the usage message and exit.
 ***************************************************************************/
static void
usage (void)
{


  fprintf (stderr, "\nUsage: %s [options] [host][:port]\n\n", PACKAGE);
  fprintf (stderr,
	   " ## General program options ##\n"
	   " -V              report program version\n"
	   " -h              show this usage message\n"
           " -top text       top directory (default: /home/seimo)\n"
	   " -c              print stations.conf file\n"
	   " -p              print details of data packets\n"
           " -d              print full station name on each seismogram\n"
           " -sc             auto-scaling each new data buffer\n\n"

           "\n"
           " ## Graphics options ##\n"
//           " -g location    location\n"
           " -x   pixels     width window in pixels (default: 1000)\n"
           " -y   pixels     height window in pixels (default: 600)\n"
           " -buf n          0-double 1-single buffering (default: 0)\n"
           " -xo  pixels     position of window x-direction (default: 0)\n"
           " -yo  pixels     position of window y-direction (default: 0)\n"
           " -m   minutes    minutes over screen: 1,2,3,4,5,10,15,30,60 (default: 15)\n"
           "\n"
           " ## Filter options ##\n"
           " -fl  lowpass    lowpass frequency\n"
           " -fh  highpass   highpass frequency\n"
           "\n"
//	   " ## Data saving options ##\n"
//	   " -o dumpfile    write all received records to this file\n"
	   " ## Data stream  selection and station file##\n"
	   " -l listfile     read a stream list from this file  (default: streams.conf)\n"
	   " -f stationfile  read a station list from this file (default: stations.conf)\n"
	   " -n no_to_plot   number of stations to plot\n"
	   " -a station no   which station to plot (0,1,2,3.......)\n"
//	   " -s selectors   selectors for uni-station or default for multi-station\n"
//           " -S streams     select streams for multi-station (requires SeedLink >= 2.5)\n"
//	   "   'streams' = 'stream1[:selectors1],stream2[:selectors2],...'\n"
//	   "        'stream' is in NET_STA format, for example:\n"
//	   "        -S \"IU_KONO:BHE BHN,GE_WLF,MN_AQU:HH?.D\"\n\n"
	   "\n"
	   " [host][:port]   Address of the SeedLink server in host:port format\n"
	   "                   f.ex.: 129.177.xx.yy:18000\n"
	   "                   f.ex.: localhost:18000\n\n");
  
}				/* End of usage() */

#ifndef WIN32
/***************************************************************************
 * term_handler:
 * Signal handler routine. 
 ***************************************************************************/
static void
term_handler (int sig)
{
  sl_terminate (slconn);
}
#endif





/*  Subroutine to apply an iir filter to a data sequence.                   */
/*    The filter is assumed to be stored as second order sections.          */
/*    Filtering is in-place.                                                */
/*    Zero-phase (forward and reverse) is an option.                        */

//appl(sw,chan,nsamp,peker)
void appl(int fnd)
//int      sw;
//int      chan;
//int      nsamp;
//float    *peker;
{
//float    *tpek;
    /* System generated locals */
int      i1;
int      i2;

    /* Local variables */
int      jptr;
int      i;
int      j;
float    b0;
float    b1;
float    b2;
float    a1;
float    a2;
float    x1;
float    x2;
float    y1;
float    y2;
float    output;
//tpek=peker;

/* Parameter adjustments                                                    */
/*    --data;*/

/* Function Body                                                            */
jptr = 1;
i1 = cset[fnd];

for (j = 1; j <= i1; ++j)
{
    x1 = save_x1[fnd][j];
    x2 = save_x2[fnd][j];
    y1 = save_y1[fnd][j];
    y2 = save_y2[fnd][j];
    b0 = snc[fnd][jptr];
    b1 = snc[fnd][jptr + 1];
    b2 = snc[fnd][jptr + 2];
    a1 = sdc[fnd][jptr + 1];
    a2 = sdc[fnd][jptr + 2];
//    i2 = nsamp;
    i2 = ns[fnd];
//    peker=tpek;
    for (i = 0; i < i2; ++i)
    {
//      output = b0 * (*peker) + b1 * x1 + b2 * x2;
      output = b0 * fseismo[i] + b1 * x1 + b2 * x2;

      output -= a1 * y1 + a2 * y2;
      y2 = y1;
      y1 = output;
      x2 = x1;
      x1 = fseismo[i];
      fseismo[i] = output;
//      peker++;
    }

    jptr += 3;

    save_y2[fnd][j]=y2;
    save_y1[fnd][j]=y1;
    save_x2[fnd][j]=x2;
    save_x1[fnd][j]=x1;

}

}
/*  Subroutine to design IIR digital filters from analog prototypes. */
/*  Input Arguments: */
/*  ---------------- */
/*    IORD                Filter order (10 MAXIMUM) */
/*    TYPE                Character*2 variable containing filter type */
/*                          LOWPASS (LP) */
/*                          HIGHPASS (HP) */
/*                          BANDPASS (BP) */
/*                          BANDREJECT (BR) */
/*   APROTO              Character*2 variable designating analog prototype*/
/*                          Butterworth (BU) */
/*                          Bessel (BE) */
/*                          Chebyshev Type I (C1) */
/*                          Chebyshev Type II (C2) */
/*    A                   Chebyshev stopband attenuation factor */
/*    TRBNDW              Chebyshev transition bandwidth (fraction of */
/*                          lowpass prototype passband width) */
/*    FL                  Low-frequency cutoff */
/*    FH                  High-frequency cutoff */
/*    TS                  Sampling interval (in seconds) */
/*  Output Arguments: */
/*  ----------------- */
/*    SN                  Array containing numerator coefficients of */
/*                        second-order sections packed head-to-tail. */
/*    SD                  Array containing denominator coefficients */
/*                        of second-order sections packed head-to-tail. */

void design( iord, type,aproto,  a,trbndw,  fl,  fh,  ts)
int iord;
char *type;
char *aproto;
float a;
float trbndw;
float fl;
float fh;
float ts;
{
    /* System generated locals */
    float r__1;

    /* Local variables */
    complex p[10], z[10];
    char stype[3*10];
    float omegar, ripple;
    float fhw, eps, flw, dcvalue;

    /*  Analog prototype selection */

    if (strncmp(aproto, "BU", 2) == 0) {
    buroots(p, stype, &dcvalue, iord);
    } else if (strncmp(aproto, "BE", 2) == 0) {
    beroots(p, stype, &dcvalue, iord);
    } else if (strncmp(aproto, "C1", 2) == 0) {
    chebparm(a, trbndw, iord, &eps, &ripple);
    c1roots(p, stype, &dcvalue, iord, &eps);
    } else if (strncmp(aproto, "C2", 2) == 0) {
    omegar = trbndw + (float)1.;
    c2roots(p, z, stype, &dcvalue, iord, a, omegar);
    }

    /*  Analog mapping selection */

    if (strncmp(type, "BP", 2) == 0) {
    r__1 = fl * ts / (float)2.;
    flw = warp(&r__1, &c_b12);
    r__1 = fh * ts / (float)2.;
    fhw = warp(&r__1, &c_b12);
    lptbp(p, z, stype, &dcvalue, &flw, &fhw);
    } else if (strncmp(type, "BR", 2) == 0) {
    r__1 = fl * ts / (float)2.;
    flw = warp(&r__1, &c_b12);
    r__1 = fh * ts / (float)2.;
    fhw = warp(&r__1, &c_b12);
    lptbr(p, z, stype, &dcvalue, &flw, &fhw, &sn[1], &sd[1]);
    } else if (strncmp(type, "LP", 2) == 0) {
    r__1 = fh * ts / (float)2.;
    fhw = warp(&r__1, &c_b12);
    lp(p, z, stype, &dcvalue, &sn[1], &sd[1]);
    cutoffs(&sn[1], &sd[1], &fhw);
    } else if (strncmp(type, "HP", 2) == 0) {
    r__1 = fl * ts / (float)2.;
    flw = warp(&r__1, &c_b12);
    lpthp(p, z, stype, &dcvalue, &sn[1], &sd[1]);
    cutoffs(&sn[1], &sd[1], &flw);
    }

    /*  Bilinear analog to digital transformation */
    bilin2(&sn[1], &sd[1]);

    return;
}

/* BUROOTS -- SUBROUTINE TO COMPUTE BUTTERWORTH POLES FOR */
/*   NORMALIZED LOWPASS FILTER */
/* LAST MODIFIED:  SEPTEMBER 7, 1990 */
/*  OUTPUT ARGUMENTS: */
/*  ----------------- */
/*      P              COMPLEX ARRAY CONTAINING POLES */
/*                       CONTAINS ONLY ONE FROM EACH */
/*                       COMPLEX CONJUGATE PAIR, AND */
/*                       ALL REAL POLES */
/*      RTYPE          CHARACTER ARRAY INDICATING 2ND ORDER SECTION */
/*                       TYPE: */
/*                         (SP)  SINGLE REAL POLE */
/*                         (CP)  COMPLEX CONJUGATE POLE PAIR */
/*                         (CPZ) COMPLEX CONJUGATE POLE-ZERO PAIRS */
/*      DCVALUE        MAGNITUDE OF FILTER AT ZERO FREQUENCY */
/*  INPUT ARGUMENTS: */
/*  ---------------- */
/*      IORD           DESIRED FILTER ORDER */
int buroots(p,rtype, dcvalue, iord)
complex *p;
char *rtype;
float *dcvalue;
int iord;
{
    /* System generated locals */
    int i__1, i__2;
    double d__1, d__2;
    complex q__1;

    /* Local variables */
    int half, k;
    float angle, pi;

    /* Parameter adjustments */
    rtype -= 3;
    --p;

    /* Function Body */
    pi = (float)3.14159265;

    half = iord / 2;

    /* TEST FOR ODD ORDER, AND ADD POLE AT -1 */

    nsects = 0;
    if (half << 1 < iord) {
    p[1].r = (float)-1., p[1].i = (float)0.;
    strncpy(rtype + 3, "SP", 2);
    nsects = 1;
    }
    i__1 = half;
    for (k = 1; k <= i__1; ++k) {
    angle = pi * ((float) ((k << 1) - 1)/(float) (iord << 1)+(float).5);
    ++(nsects);
    i__2 = nsects;
    d__1 = cos(angle);
    d__2 = sin(angle);
    q__1.r = d__1, q__1.i = d__2;
    p[i__2].r = q__1.r, p[i__2].i = q__1.i;
        strncpy(rtype + nsects * 3, "CP", 2);
    }
    *dcvalue = (float)1.;
    return TRUE;
}

/*  CHEBPARM - Calculates Chebyshev type I and II design parameters */
/*  INPUT ARGUMENTS */
/*  --------------- */
/*       A                Desired stopband attenuation */
/*                          i.e. max stopband amplitude is 1/ATTEN */
/*       TRBNDW           Transition bandwidth between stop and passbands */
/*                          as a fraction of the passband width */
/*       IORD             Filter order (number of poles) */
/*  OUTPUT ARGUMENTS */
/*  ---------------- */
/*       EPS              Chebyshev passband parameter */
/*       RIPPLE           Passband ripple */

static int chebparm(a, trbndw, iord, eps, ripple)
float a, trbndw;
int iord;
float *eps, *ripple;
{
    /* System generated locals */
    float r__1, r__2;

    /* Local variables */
    float g, alpha, omegar;

    omegar = trbndw + 1.;
    /* Computing 2nd power */
    r__2 = omegar;
    r__1 = omegar + sqrt(r__2 * r__2 - 1.);
    alpha = pow((double)r__1, (double)iord);
    /* Computing 2nd power */
    r__1 = alpha;
    g = (r__1 * r__1 + (float)1.) / (alpha * (float)2.);
    /* Computing 2nd power */
    r__1 = a;
    *eps = sqrt(r__1 * r__1 - (float)1.) / g;
    /* Computing 2nd power */
    r__1 = *eps;
    *ripple = (float)1. / sqrt(r__1 * r__1 + (float)1.);

    return TRUE;
}

/* C1ROOTS -- SUBROUTINE TO COMPUTE CHEBYSHEV TYPE I POLES FOR */
/*   NORMALIZED LOWPASS FILTER */
/* LAST MODIFIED:  SEPTEMBER 7, 1990 */
/*  OUTPUT ARGUMENTS: */
/*  ----------------- */
/*      P              COMPLEX ARRAY CONTAINING POLES */
/*                       CONTAINS ONLY ONE FROM EACH */
/*                       COMPLEX CONJUGATE PAIR, AND */
/*                       ALL REAL POLES */
/*      RTYPE          CHARACTER ARRAY INDICATING 2ND ORDER SECTION */
/*                       TYPE: */
/*                         (SP)  SINGLE REAL POLE */
/*                         (CP)  COMPLEX CONJUGATE POLE PAIR */
/*                         (CPZ) COMPLEX CONJUGATE POLE-ZERO PAIRS */
/*      DCVALUE        RESPONSE OF FILTER AT ZERO FREQUENCY */
/*  INPUT ARGUMENTS: */
/*  ---------------- */
/*      IORD           DESIRED FILTER ORDER */
/*      EPS            CHEBYSHEV PAR RELATED TO PASSBAND RIPPLE */

static int c1roots(p, rtype, dcvalue, iord, eps)
complex *p;
char *rtype;
float *dcvalue;
int iord;
float *eps;
{
    /* System generated locals */
    int i__1, i__2;
    float r__1;
    double d__1;
    complex q__1;

    /* Local variables */
    int half;
    float c;
    int i;
    float gamma, s, angle, omega, sigma, pi;

    /* Parameter adjustments */
    rtype -= 3;
    --p;

    /* Function Body */
    pi = (float)3.14159265;
    half = iord / 2;

    /*  INTERMEDIATE DESIGN PARS */

    gamma = (sqrt(*eps * *eps + (float)1.) + (float)1.) / *eps;
    gamma = log(gamma) / (float) (iord);
    gamma = exp(gamma);
    s = (gamma - (float)1. / gamma) * (float).5;
    c = (gamma + (float)1. / gamma) * (float).5;

    /*  CALCULATE POLES */

    nsects = 0;
    i__1 = half;
    for (i = 1; i <= i__1; ++i) {
    strncpy(rtype + i * 3, "CP", 2);
    angle = (float) ((i << 1) - 1) * pi / (float) (iord << 1);
    sigma = -(double)s * sin(angle);
    omega = c * cos(angle);
    i__2 = i;
    q__1.r = sigma, q__1.i = omega;
    p[i__2].r = q__1.r, p[i__2].i = q__1.i;
    ++(nsects);
    }
    if (half << 1 < iord) {
    strncpy(rtype + (half + 1) * 3, "SP", 2);
    i__1 = half + 1;
    d__1 = -(double)s;
    q__1.r = d__1, q__1.i = (float)0.;
    p[i__1].r = q__1.r, p[i__1].i = q__1.i;
    ++(nsects);
    *dcvalue = (float)1.;
    } else {
    /* Computing 2nd power */
    r__1 = *eps;
    *dcvalue = (float)1. / sqrt(r__1 * r__1 + 1);
    }
    return TRUE;
}

/* C2ROOTS -- SUBROUTINE TO COMPUTE ROOTS FOR NORMALIZED LOWPASS */
/*   CHEBYSHEV TYPE 2 FILTER */
/* LAST MODIFIED:  SEPTEMBER 7, 1990 */
/*  OUTPUT ARGUMENTS: */
/*  ----------------- */
/*      P              COMPLEX ARRAY CONTAINING POLES */
/*                       CONTAINS ONLY ONE FROM EACH */
/*                       COMPLEX CONJUGATE PAIR, AND */
/*                       ALL REAL POLES */
/*      Z              COMPLEX ARRAY CONTAINING ZEROS */
/*                       CONTAINS ONLY ONE FROM EACH */
/*                       COMPLEX CONJUGATE PAIR, AND */
/*                       ALL REAL ZEROS */
/*      RTYPE          CHARACTER ARRAY INDICATING 2ND ORDER SECTION */
/*                       TYPE: */
/*                         (SP)  SINGLE REAL POLE */
/*                         (CP)  COMPLEX CONJUGATE POLE PAIR */
/*                         (CPZ) COMPLEX CONJUGATE POLE-ZERO PAIRS */
/*      DCVALUE        MAGNITUDE OF FILTER AT ZERO FREQUENCY */
/*  INPUT ARGUMENTS: */
/*  ---------------- */
/*      IORD           DESIRED FILTER ORDER */
/*      A              STOPBAND ATTENUATION FACTOR */
/*      OMEGAR         CUTOFF FREQUENCY OF STOPBAND */
/*                     PASSBAND CUTOFF IS AT 1.0 HERTZ */

static int c2roots(p, z, rtype, dcvalue, iord, a, omegar)
complex *p, *z;
char *rtype;
float *dcvalue;
int iord;
float a, omegar;
{
    /* System generated locals */
    int i__1, i__2;
    double d__1;
    complex q__1;

    /* Local variables */
    int half;
    float beta, c;
    int i;
    float gamma, s, alpha, angle, omega, sigma, denom, pi;

    /* Parameter adjustments */
    rtype -= 3;
    --z;
    --p;

    /* Function Body */
    pi = (float)3.14159265;
    half = iord / 2;

/*  INTERMEDIATE DESIGN PARS */

    gamma = a + sqrt(a * a - (float)1.);
    gamma = log(gamma) / (float) (iord);
    gamma = exp(gamma);
    s = (gamma - (float)1. / gamma) * (float).5;
    c = (gamma + (float)1. / gamma) * (float).5;
    nsects = 0;
    i__1 = half;
    for (i = 1; i <= i__1; ++i) {

/*  CALCULATE POLES */

    strncpy(rtype + i * 3, "CPZ", 3);
    angle = (float) ((i << 1) - 1) * pi / (float) (iord << 1);
    alpha = -(double)s * sin(angle);
    beta = c * cos(angle);
    denom = alpha * alpha + beta * beta;
    sigma = omegar * alpha / denom;
    omega = -(double)(omegar) * beta / denom;
    i__2 = i;
    q__1.r = sigma, q__1.i = omega;
    p[i__2].r = q__1.r, p[i__2].i = q__1.i;

/*  CALCULATE ZEROS */

    omega = omegar / cos(angle);
    i__2 = i;
    q__1.r = (float)0., q__1.i = omega;
    z[i__2].r = q__1.r, z[i__2].i = q__1.i;
    ++(nsects);
    }

/*  ODD-ORDER FILTERS */

    if (half << 1 < iord) {
    strncpy(rtype + (half + 1) * 3, "SP", 2);
    i__1 = half + 1;
    d__1 = -(double)(omegar) / s;
    q__1.r = d__1, q__1.i = (float)0.;
    p[i__1].r = q__1.r, p[i__1].i = q__1.i;
    ++(nsects);
    }

/*  DC VALUE */

    *dcvalue = 1.;
    return TRUE;
}

/* Subroutine to convert an prototype lowpass filter to a bandpass filter via
*/
/*    the analog polynomial transformation.  The lowpass filter is */
/*   described in terms of its poles and zeros (as input to this routine).*/
/*    The output consists of the parameters for second order sections. */
/*  Input Arguments: */
/*  ---------------- */
/*    P                       Array containing poles */
/*    Z                       Array containing zeros */
/*    RTYPE                   Character array containing type information */
/*                              (SP) single real pole  or */
/*                              (CP) complex conjugate pole pair  or */
/*                              (CPZ) complex conjugate pole/zero pairs */
/*    DCVALUE                 Zero frequency value of filter */
/*    FL                      Low-frequency cutoff */
/*    FH                      High-frequency cutoff */
/*  Output Arguments: */
/*  ----------------- */
/*    SN                      Numerator polynomials for second order */
/*                              sections. */
/*    SD                      Denominator polynomials for second order */
/*                              sections. */
/*                              This subroutine doubles the number of */
/*                              sections. */

static int lptbp(p, z, rtype, dcvalue, fl, fh)
complex *p, *z;
char *rtype;
float *dcvalue;
float *fl, *fh;
{
    /* System generated locals */
    int i__1, i__2, i__3, i__4, i__5, i__6, i__7;
    double d__1;
    complex q__1, q__2, q__3, q__4, q__5, q__6, q__7, q__8, q__9, q__10;

    /* Local variables */
    int iptr;
    float a, b;
    complex h;
    int i, n;
    complex s;
    float scale;
    complex ctemp, p1, p2;
    float twopi;
    complex z1, z2;
    float pi;

    /* Parameter adjustments */
    rtype -= 3;
    --z;
    --p;

    /* Function Body */
    pi = (float)3.14159265;
    twopi = pi * (float)2.;
    a = twopi * twopi * *fl * *fh;
    b = twopi * (*fh - *fl);

    n = nsects;
    nsects = 0;
    iptr = 1;
    i__1 = n;
    for (i = 1; i <= i__1; ++i) {
    if (strncmp(rtype + i * 3, "CPZ", 3) == 0) {
        i__2 = i;
        q__3.r = b * z[i__2].r, q__3.i = b * z[i__2].i;
            q__2 = cpowi(q__3, 2);
        d__1 = a * (float)4.;
        q__1.r = q__2.r - d__1, q__1.i = q__2.i;
        ctemp.r = q__1.r, ctemp.i = q__1.i;
            q__1 = csqrt1(ctemp);
        ctemp.r = q__1.r, ctemp.i = q__1.i;
        i__2 = i;
        q__3.r = b * z[i__2].r, q__3.i = b * z[i__2].i;
        q__2.r = q__3.r + ctemp.r, q__2.i = q__3.i + ctemp.i;
        q__1.r = q__2.r * (float).5, q__1.i = q__2.i * (float).5;
        z1.r = q__1.r, z1.i = q__1.i;
        i__2 = i;
        q__3.r = b * z[i__2].r, q__3.i = b * z[i__2].i;
        q__2.r = q__3.r - ctemp.r, q__2.i = q__3.i - ctemp.i;
        q__1.r = q__2.r * (float).5, q__1.i = q__2.i * (float).5;
        z2.r = q__1.r, z2.i = q__1.i;
        i__2 = i;
        q__3.r = b * p[i__2].r, q__3.i = b * p[i__2].i;
            q__2 = cpowi(q__3, 2);
        d__1 = a * (float)4.;
        q__1.r = q__2.r - d__1, q__1.i = q__2.i;
        ctemp.r = q__1.r, ctemp.i = q__1.i;
            q__1 = csqrt1(ctemp);
        ctemp.r = q__1.r, ctemp.i = q__1.i;
        i__2 = i;
        q__3.r = b * p[i__2].r, q__3.i = b * p[i__2].i;
        q__2.r = q__3.r + ctemp.r, q__2.i = q__3.i + ctemp.i;
        q__1.r = q__2.r * (float).5, q__1.i = q__2.i * (float).5;
        p1.r = q__1.r, p1.i = q__1.i;
        i__2 = i;
        q__3.r = b * p[i__2].r, q__3.i = b * p[i__2].i;
        q__2.r = q__3.r - ctemp.r, q__2.i = q__3.i - ctemp.i;
        q__1.r = q__2.r * (float).5, q__1.i = q__2.i * (float).5;
        p2.r = q__1.r, p2.i = q__1.i;
            q__2 = conjg(z1);
        q__1.r = z1.r * q__2.r - z1.i * q__2.i, q__1.i = z1.r * q__2.i +
            z1.i * q__2.r;
        sn[iptr] = q__1.r;
        sn[iptr + 1] = z1.r * (float)-2.;
        sn[iptr + 2] = (float)1.;
            q__2 = conjg(p1);
        q__1.r = p1.r * q__2.r - p1.i * q__2.i, q__1.i = p1.r * q__2.i +
            p1.i * q__2.r;
        sd[iptr] = q__1.r;
        sd[iptr + 1] = p1.r * (float)-2.;
        sd[iptr + 2] = (float)1.;
        iptr += 3;
            q__2 = conjg(z2);
        q__1.r = z2.r * q__2.r - z2.i * q__2.i, q__1.i = z2.r * q__2.i +
            z2.i * q__2.r;
        sn[iptr] = q__1.r;
        sn[iptr + 1] = z2.r * (float)-2.;
        sn[iptr + 2] = (float)1.;
            q__2 = conjg(p2);
        q__1.r = p2.r * q__2.r - p2.i * q__2.i, q__1.i = p2.r * q__2.i +
            p2.i * q__2.r;
        sd[iptr] = q__1.r;
        sd[iptr + 1] = p2.r * (float)-2.;
        sd[iptr + 2] = (float)1.;
        iptr += 3;
        nsects += 2;
    } else if (strncmp(rtype + i * 3, "CP", 2) == 0) {
        i__2 = i;
        q__3.r = b * p[i__2].r, q__3.i = b * p[i__2].i;
            q__2 = cpowi(q__3, 2);
        d__1 = a * (float)4.;
        q__1.r = q__2.r - d__1, q__1.i = q__2.i;
        ctemp.r = q__1.r, ctemp.i = q__1.i;
            q__1 = csqrt1(ctemp);
        ctemp.r = q__1.r, ctemp.i = q__1.i;
        i__2 = i;
        q__3.r = b * p[i__2].r, q__3.i = b * p[i__2].i;
        q__2.r = q__3.r + ctemp.r, q__2.i = q__3.i + ctemp.i;
        q__1.r = q__2.r * (float).5, q__1.i = q__2.i * (float).5;
        p1.r = q__1.r, p1.i = q__1.i;
        i__2 = i;
        q__3.r = b * p[i__2].r, q__3.i = b * p[i__2].i;
        q__2.r = q__3.r - ctemp.r, q__2.i = q__3.i - ctemp.i;
        q__1.r = q__2.r * (float).5, q__1.i = q__2.i * (float).5;
        p2.r = q__1.r, p2.i = q__1.i;
        sn[iptr] = (float)0.;
        sn[iptr + 1] = b;
        sn[iptr + 2] = (float)0.;
            q__2 = conjg(p1);
        q__1.r = p1.r * q__2.r - p1.i * q__2.i, q__1.i = p1.r * q__2.i +
            p1.i * q__2.r;
        sd[iptr] = q__1.r;
        sd[iptr + 1] = p1.r * (float)-2.;
        sd[iptr + 2] = (float)1.;
        iptr += 3;
        sn[iptr] = (float)0.;
        sn[iptr + 1] = b;
        sn[iptr + 2] = (float)0.;
            q__2 = conjg(p2);
        q__1.r = p2.r * q__2.r - p2.i * q__2.i, q__1.i = p2.r * q__2.i +
            p2.i * q__2.r;
        sd[iptr] = q__1.r;
        sd[iptr + 1] = p2.r * (float)-2.;
        sd[iptr + 2] = (float)1.;
        iptr += 3;
        nsects += 2;
    } else if (strncmp(rtype + i * 3, "SP", 2) == 0) {
        sn[iptr] = (float)0.;
        sn[iptr + 1] = b;
        sn[iptr + 2] = (float)0.;
        sd[iptr] = a;
        i__2 = i;
        sd[iptr + 1] = -(double)b * p[i__2].r;
        sd[iptr + 2] = (float)1.;
        iptr += 3;
        ++(nsects);
    }
    }
/* Scaling - use fact that the bandpass filter amplitude at sqrt( omega_l **/
/*            equals the amplitude of the lowpass prototype at d.c. */
    d__1 = sqrt(a);
    q__1.r = (float)0., q__1.i = d__1;
    s.r = q__1.r, s.i = q__1.i;
    h.r = (float)1., h.i = (float)0.;

    iptr = 1;
    i__1 = nsects;
    for (i = 1; i <= i__1; ++i) {
    i__2 = iptr + 2;
    q__6.r = sn[i__2] * s.r, q__6.i = sn[i__2] * s.i;
    i__3 = iptr + 1;
    q__5.r = q__6.r + sn[i__3], q__5.i = q__6.i;
    q__4.r = q__5.r * s.r - q__5.i * s.i, q__4.i = q__5.r * s.i + q__5.i *
         s.r;
    i__4 = iptr;
    q__3.r = q__4.r + sn[i__4], q__3.i = q__4.i;
    q__2.r = h.r * q__3.r - h.i * q__3.i, q__2.i = h.r * q__3.i + h.i *
        q__3.r;
    i__5 = iptr + 2;
    q__10.r = sd[i__5] * s.r, q__10.i = sd[i__5] * s.i;
    i__6 = iptr + 1;
    q__9.r = q__10.r + sd[i__6], q__9.i = q__10.i;
    q__8.r = q__9.r * s.r - q__9.i * s.i, q__8.i = q__9.r * s.i + q__9.i *
         s.r;
    i__7 = iptr;
    q__7.r = q__8.r + sd[i__7], q__7.i = q__8.i;
        q__1 = cdiv(q__2, q__7);
    h.r = q__1.r, h.i = q__1.i;
    iptr += 3;
    }
    q__2.r = *dcvalue, q__2.i = (float)0.;
    d__1 = h.r;
    q__5 = conjg(h);
    q__4.r = d__1 * q__5.r, q__4.i = d__1 * q__5.i;
    q__3 = csqrt1(q__4);
    q__1 = cdiv(q__2, q__3);
    scale = q__1.r;
    sn[1] *= scale;
    sn[2] *= scale;
    sn[3] *= scale;
    return TRUE;
}

/*  Subroutine to generate second order section parameterization */
/*    from an pole-zero description for lowpass filters. */
/*  Input Arguments: */
/*  ---------------- */
/*    P                       Array containing poles */
/*    Z                       Array containing zeros */
/*   RTYPE                   Character array containing root type information
*/
/*                              (SP)  Single pole or */
/*                              (CP)  Complex conjugate pole pair */
/*                             (CPZ) Complex conjugate pole and zero pairs*/
/*    DCVALUE                 Zero-frequency value of prototype filter */
/*  Output Arguments: */
/*  ----------------- */
/*    SN                      Numerator polynomials for second order */
/*                              sections. */
/*    SD                      Denominator polynomials for second order */
/*                              sections. */

static int lp(p, z, rtype, dcvalue, sn, sd)
complex *p, *z;
char *rtype;
float *dcvalue;
float *sn, *sd;
{
    /* System generated locals */
    int i__1, i__2, i__3;
    complex q__1, q__2, q__3, q__4;

    /* Local variables */
    int iptr, i;
    float scale;

    /* Parameter adjustments */
    --sd;
    --sn;
    rtype -= 3;
    --z;
    --p;

    /* Function Body */
    iptr = 1;
    i__1 = nsects;
    for (i = 1; i <= i__1; ++i) {
    if (strncmp(rtype + i * 3, "CPZ", 3) == 0) {
        i__2 = i;
        q__2 = conjg(p[i]);
        q__1.r = p[i__2].r * q__2.r - p[i__2].i * q__2.i, q__1.i = p[i__2]
            .r * q__2.i + p[i__2].i * q__2.r;
        i__3 = i;
        q__4 = conjg(z[i]);
        q__3.r = z[i__3].r * q__4.r - z[i__3].i * q__4.i, q__3.i = z[i__3]
            .r * q__4.i + z[i__3].i * q__4.r;
        scale = q__1.r / q__3.r;
        i__2 = i;
        q__2 = conjg(z[i]);
        q__1.r = z[i__2].r * q__2.r - z[i__2].i * q__2.i, q__1.i = z[i__2]
            .r * q__2.i + z[i__2].i * q__2.r;
        sn[iptr] = q__1.r * scale;
        i__2 = i;
        sn[iptr + 1] = z[i__2].r * (float)-2. * scale;
        sn[iptr + 2] = scale * (float)1.;
        i__2 = i;
        q__2 = conjg(p[i]);
        q__1.r = p[i__2].r * q__2.r - p[i__2].i * q__2.i, q__1.i = p[i__2]
            .r * q__2.i + p[i__2].i * q__2.r;
        sd[iptr] = q__1.r;
        i__2 = i;
        sd[iptr + 1] = p[i__2].r * (float)-2.;
        sd[iptr + 2] = (float)1.;
        iptr += 3;
    } else if (strncmp(rtype + i * 3, "CP", 2) == 0) {
        i__2 = i;
        q__2 = conjg(p[i]);
        q__1.r = p[i__2].r * q__2.r - p[i__2].i * q__2.i, q__1.i = p[i__2]
            .r * q__2.i + p[i__2].i * q__2.r;
        scale = q__1.r;
        sn[iptr] = scale;
        sn[iptr + 1] = (float)0.;
        sn[iptr + 2] = (float)0.;
        i__2 = i;
        q__2 = conjg(p[i]);
        q__1.r = p[i__2].r * q__2.r - p[i__2].i * q__2.i, q__1.i = p[i__2]
            .r * q__2.i + p[i__2].i * q__2.r;
        sd[iptr] = q__1.r;
        i__2 = i;
        sd[iptr + 1] = p[i__2].r * (float)-2.;
        sd[iptr + 2] = (float)1.;
        iptr += 3;
    } else if (strncmp(rtype + i * 3, "SP", 2) == 0) {
        i__2 = i;
        scale = -(double)p[i__2].r;
        sn[iptr] = scale;
        sn[iptr + 1] = (float)0.;
        sn[iptr + 2] = (float)0.;
        i__2 = i;
        sd[iptr] = -(double)p[i__2].r;
        sd[iptr + 1] = (float)1.;
        sd[iptr + 2] = (float)0.;
        iptr += 3;
    }
    }
    sn[1] = *dcvalue * sn[1];
    sn[2] = *dcvalue * sn[2];
    sn[3] = *dcvalue * sn[3];
    return TRUE;
}

/*  Subroutine to convert a lowpass filter to a band reject filter */
/*    via an analog polynomial transformation.  The lowpass filter is */
/*   described in terms of its poles and zeros (as input to this routine).*/
/*    The output consists of the parameters for second order sections. */
/*  Input Arguments: */
/*  ---------------- */
/*    P                       Array containing poles */
/*    Z                       Array containing zeros */
/*    RTYPE                   Character array containing type information */
/*                              (SP)  single real pole or */
/*                              (CP)  complex conjugate pole pair */
/*                              (CPZ) complex conjugate pole/zero pairs */
/*    DCVALUE                 Zero-frequency value of prototype filter */
/*                              prior to transformation */
/*    FL                      Low-frequency cutoff */
/*    FH                      High-frequency cutoff */
/*  Output Arguments: */
/*  ----------------- */
/*    SN                      Numerator polynomials for second order */
/*                              sections. */
/*    SD                      Denominator polynomials for second order */
/*                              sections. */
/*    NSECTS                  Number of second order sections following */
/*                              transformation.  The number is doubled. */

static int lptbr(p, z, rtype, dcvalue, fl, fh, sn, sd)
complex *p, *z;
char *rtype;
float *dcvalue;
float *fl, *fh, *sn, *sd;
{
    /* System generated locals */
    int i__1, i__2;
    double d__1;
    complex q__1, q__2, q__3;

    /* Local variables */
    complex cinv;
    int iptr;
    float a, b, h;
    int i, n;
    float scale;
    complex ctemp, p1, p2;
    float twopi;
    complex z1, z2;
    float pi;

    /* Parameter adjustments */
    --sd;
    --sn;
    rtype -= 3;
    --z;
    --p;

    /* Function Body */
    pi = (float)3.14159265;
    twopi = pi * (float)2.;
    a = twopi * twopi * *fl * *fh;
    b = twopi * (*fh - *fl);
    n = nsects;
    nsects = 0;
    iptr = 1;
    i__1 = n;
    for (i = 1; i <= i__1; ++i) {
    if (strncmp(rtype + i * 3, "CPZ", 3) == 0) {
        q__1 = cdiv(c_b43, z[i]);
        cinv.r = q__1.r, cinv.i = q__1.i;
        q__3.r = b * cinv.r, q__3.i = b * cinv.i;
            q__2 = cpowi(q__3, 2);
        d__1 = a * (float)4.;
        q__1.r = q__2.r - d__1, q__1.i = q__2.i;
        ctemp.r = q__1.r, ctemp.i = q__1.i;
        q__1 = csqrt1(ctemp);
        ctemp.r = q__1.r, ctemp.i = q__1.i;
        q__3.r = b * cinv.r, q__3.i = b * cinv.i;
        q__2.r = q__3.r + ctemp.r, q__2.i = q__3.i + ctemp.i;
        q__1.r = q__2.r * (float).5, q__1.i = q__2.i * (float).5;
        z1.r = q__1.r, z1.i = q__1.i;
        q__3.r = b * cinv.r, q__3.i = b * cinv.i;
        q__2.r = q__3.r - ctemp.r, q__2.i = q__3.i - ctemp.i;
        q__1.r = q__2.r * (float).5, q__1.i = q__2.i * (float).5;
        z2.r = q__1.r, z2.i = q__1.i;
        q__1 = cdiv(c_b43, p[i]);
        cinv.r = q__1.r, cinv.i = q__1.i;
        q__3.r = b * cinv.r, q__3.i = b * cinv.i;
            q__2 = cpowi(q__3, 2);
        d__1 = a * (float)4.;
        q__1.r = q__2.r - d__1, q__1.i = q__2.i;
        ctemp.r = q__1.r, ctemp.i = q__1.i;
        q__1 = csqrt1(ctemp);
        ctemp.r = q__1.r, ctemp.i = q__1.i;
        q__3.r = b * cinv.r, q__3.i = b * cinv.i;
        q__2.r = q__3.r + ctemp.r, q__2.i = q__3.i + ctemp.i;
        q__1.r = q__2.r * (float).5, q__1.i = q__2.i * (float).5;
        p1.r = q__1.r, p1.i = q__1.i;
        q__3.r = b * cinv.r, q__3.i = b * cinv.i;
        q__2.r = q__3.r - ctemp.r, q__2.i = q__3.i - ctemp.i;
        q__1.r = q__2.r * (float).5, q__1.i = q__2.i * (float).5;
        p2.r = q__1.r, p2.i = q__1.i;
        q__2 = conjg(z1);
        q__1.r = z1.r * q__2.r - z1.i * q__2.i, q__1.i = z1.r * q__2.i +
            z1.i * q__2.r;
        sn[iptr] = q__1.r;
        sn[iptr + 1] = z1.r * (float)-2.;
        sn[iptr + 2] = (float)1.;
        q__2 = conjg(p1);
        q__1.r = p1.r * q__2.r - p1.i * q__2.i, q__1.i = p1.r * q__2.i +
            p1.i * q__2.r;
        sd[iptr] = q__1.r;
        sd[iptr + 1] = p1.r * (float)-2.;
        sd[iptr + 2] = (float)1.;
        iptr += 3;
        q__2 = conjg(z2);
        q__1.r = z2.r * q__2.r - z2.i * q__2.i, q__1.i = z2.r * q__2.i +
            z2.i * q__2.r;
        sn[iptr] = q__1.r;
        sn[iptr + 1] = z2.r * (float)-2.;
        sn[iptr + 2] = (float)1.;
        q__2 = conjg(p2);
        q__1.r = p2.r * q__2.r - p2.i * q__2.i, q__1.i = p2.r * q__2.i +
            p2.i * q__2.r;
        sd[iptr] = q__1.r;
        sd[iptr + 1] = p2.r * (float)-2.;
        sd[iptr + 2] = (float)1.;
        iptr += 3;
        nsects += 2;
    } else if (strncmp(rtype + i * 3, "CP", 2) == 0) {
        q__1 = cdiv(c_b43, p[i]);
        cinv.r = q__1.r, cinv.i = q__1.i;
        q__3.r = b * cinv.r, q__3.i = b * cinv.i;
            q__2 = cpowi(q__3, 2);
        d__1 = a * (float)4.;
        q__1.r = q__2.r - d__1, q__1.i = q__2.i;
        ctemp.r = q__1.r, ctemp.i = q__1.i;
        q__1 = csqrt1(ctemp);
        ctemp.r = q__1.r, ctemp.i = q__1.i;
        q__3.r = b * cinv.r, q__3.i = b * cinv.i;
        q__2.r = q__3.r + ctemp.r, q__2.i = q__3.i + ctemp.i;
        q__1.r = q__2.r * (float).5, q__1.i = q__2.i * (float).5;
        p1.r = q__1.r, p1.i = q__1.i;
        q__3.r = b * cinv.r, q__3.i = b * cinv.i;
        q__2.r = q__3.r - ctemp.r, q__2.i = q__3.i - ctemp.i;
        q__1.r = q__2.r * (float).5, q__1.i = q__2.i * (float).5;
        p2.r = q__1.r, p2.i = q__1.i;
        sn[iptr] = a;
        sn[iptr + 1] = (float)0.;
        sn[iptr + 2] = (float)1.;
        q__2 = conjg(p1);
        q__1.r = p1.r * q__2.r - p1.i * q__2.i, q__1.i = p1.r * q__2.i +
            p1.i * q__2.r;
        sd[iptr] = q__1.r;
        sd[iptr + 1] = p1.r * (float)-2.;
        sd[iptr + 2] = (float)1.;
        iptr += 3;
        sn[iptr] = a;
        sn[iptr + 1] = (float)0.;
        sn[iptr + 2] = (float)1.;
        q__2 = conjg(p2);
        q__1.r = p2.r * q__2.r - p2.i * q__2.i, q__1.i = p2.r * q__2.i +
            p2.i * q__2.r;
        sd[iptr] = q__1.r;
        sd[iptr + 1] = p2.r * (float)-2.;
        sd[iptr + 2] = (float)1.;
        iptr += 3;
        nsects += 2;
    } else if (strncmp(rtype + i * 3, "SP", 2) == 0) {
        sn[iptr] = a;
        sn[iptr + 1] = (float)0.;
        sn[iptr + 2] = (float)1.;
        i__2 = i;
        sd[iptr] = -(double)a * p[i__2].r;
        sd[iptr + 1] = b;
        i__2 = i;
        sd[iptr + 2] = -(double)p[i__2].r;
        iptr += 3;
        ++(nsects);
    }
    }
/*  Scaling - use the fact that the bandreject filter amplitude  at d.c. */
/*            equals the lowpass prototype amplitude at d.c. */
    h = (float)1.;
    iptr = 1;
    i__1 = nsects;
    for (i = 1; i <= i__1; ++i) {
    h = h * sn[iptr] / sd[iptr];
    iptr += 3;
    }
    scale = *dcvalue / ABS(h);
    sn[1] *= scale;
    sn[2] *= scale;
    sn[3] *= scale;
    return TRUE;
}

/*  Subroutine to alter the cutoff of a filter.  Assumes that the */
/*    filter is structured as second order sections.  Changes */
/*    the cutoffs of normalized lowpass and highpass filters through */
/*    a simple polynomial transformation. */
/*  Input Arguments: */
/*  ---------------- */
/*    F                       New cutoff frequency */
/*  Input/Output Arguments: */
/*  ----------------------- */
/*    SN                      Numerator polynomials for second order */
/*                              sections. */
/*    SD                      Denominator polynomials for second order */
/*                              sections. */

static int cutoffs(sn, sd, f)
float *sn, *sd;
float *f;
{
    /* System generated locals */
    int i__1;

    /* Local variables */
    int iptr, i;
    float scale;

    /* Parameter adjustments */
    --sd;
    --sn;

    /* Function Body */
    scale = *f * (float)6.2831853000000004;

    iptr = 1;
    i__1 = nsects;
    for (i = 1; i <= i__1; ++i) {

    sn[iptr + 1] /= scale;
    sn[iptr + 2] /= scale * scale;
    sd[iptr + 1] /= scale;
    sd[iptr + 2] /= scale * scale;
    iptr += 3;
    }
    return TRUE;
}

/*  Subroutine to convert a lowpass filter to a highpass filter via */
/*    an analog polynomial transformation.  The lowpass filter is */
/*   described in terms of its poles and zeroes (as input to this routine).*/
/*    The output consists of the parameters for second order sections. */
/*  Input Arguments: */
/*  ---------------- */
/*    P                       Array containing poles */
/*    Z                       Array containing zeroes */
/*   RTYPE                   Character array containing root type information
*/
/*                              (SP) single real pole or */
/*                              (CP)  complex conjugate pair */
/*                              (CPZ) complex pole/zero pairs */
/*    DCVALUE                 Zero-frequency value of prototype filter */
/*  Output Arguments: */
/*  ----------------- */
/*    SN                      Numerator polynomials for second order */
/*                              sections. */
/*    SD                      Denominator polynomials for second order */
/*                              sections. */

static int lpthp(p, z, rtype, dcvalue, sn, sd)
complex *p, *z;
char *rtype;
float *dcvalue;
float *sn, *sd;
{
    /* System generated locals */
    int i__1, i__2, i__3;
    complex q__1, q__2, q__3, q__4;

   /* Local variables */
    int iptr, i;
    float scale;

    /* Parameter adjustments */
    --sd;
    --sn;
    rtype -= 3;
    --z;
    --p;

    /* Function Body */
    iptr = 1;
    i__1 = nsects;
    for (i = 1; i <= i__1; ++i) {
    if (strncmp(rtype + i * 3, "CPZ", 3) == 0) {
        i__2 = i;
        q__2 = conjg(p[i]);
        q__1.r = p[i__2].r * q__2.r - p[i__2].i * q__2.i, q__1.i = p[i__2]
            .r * q__2.i + p[i__2].i * q__2.r;
        i__3 = i;
        q__4 = conjg(z[i]);
        q__3.r = z[i__3].r * q__4.r - z[i__3].i * q__4.i, q__3.i = z[i__3]
            .r * q__4.i + z[i__3].i * q__4.r;
        scale = q__1.r / q__3.r;
        sn[iptr] = scale * (float)1.;
        i__2 = i;
        sn[iptr + 1] = z[i__2].r * (float)-2. * scale;
        i__2 = i;
        q__2 = conjg(z[i]);
        q__1.r = z[i__2].r * q__2.r - z[i__2].i * q__2.i, q__1.i = z[i__2]
            .r * q__2.i + z[i__2].i * q__2.r;
        sn[iptr + 2] = q__1.r * scale;
        sd[iptr] = (float)1.;
        i__2 = i;
        sd[iptr + 1] = p[i__2].r * (float)-2.;
        i__2 = i;
        q__2 = conjg(p[i]);
        q__1.r = p[i__2].r * q__2.r - p[i__2].i * q__2.i, q__1.i = p[i__2]
            .r * q__2.i + p[i__2].i * q__2.r;
        sd[iptr + 2] = q__1.r;
        iptr += 3;
    } else if (strncmp(rtype + i * 3, "CP", 2) == 0) {
        i__2 = i;
        q__2 = conjg(p[i]);
        q__1.r = p[i__2].r * q__2.r - p[i__2].i * q__2.i, q__1.i = p[i__2]
            .r * q__2.i + p[i__2].i * q__2.r;
        scale = q__1.r;
        sn[iptr] = (float)0.;
        sn[iptr + 1] = (float)0.;
        sn[iptr + 2] = scale;
        sd[iptr] = (float)1.;
        i__2 = i;
        sd[iptr + 1] = p[i__2].r * (float)-2.;
        i__2 = i;
        q__2 = conjg(p[i]);
        q__1.r = p[i__2].r * q__2.r - p[i__2].i * q__2.i, q__1.i = p[i__2]
            .r * q__2.i + p[i__2].i * q__2.r;
        sd[iptr + 2] = q__1.r;
        iptr += 3;
    } else if (strncmp(rtype + i * 3, "SP", 2) == 0) {
        i__2 = i;
        scale = -(double)p[i__2].r;
        sn[iptr] = (float)0.;
        sn[iptr + 1] = scale;
        sn[iptr + 2] = (float)0.;
        sd[iptr] = (float)1.;
        i__2 = i;
        sd[iptr + 1] = -(double)p[i__2].r;
        sd[iptr + 2] = (float)0.;
        iptr += 3;
    }
    }
    sn[1] *= *dcvalue;
    sn[2] *= *dcvalue;
    sn[3] *= *dcvalue;
    return TRUE;
}

/* Transforms an analog filter to a digital filter via the bilinear transforma
ti*/
/*   Assumes both are stored as second order sections.  The transformation is
*/
/*    done in-place. */
/*  Input Arguments: */
/*  ---------------- */
/*   SN                   Array containing numerator polynomial coefficients f
or*/
/*                           second order sections.  Packed head-to-tail. */
/*   SD                   Array containing denominator polynomial coefficients
 f*/
/*                           second order sections.  Packed head-to-tail. */

static int bilin2(sn, sd)
float *sn, *sd;
{
    /* System generated locals */
    int i__1;

    /* Local variables */
    int iptr, i;
    float scale, a0, a1, a2;

    /* Parameter adjustments */
    --sd;
    --sn;

    /* Function Body */
    iptr = 1;
    i__1 = nsects;
    for (i = 1; i <= i__1; ++i) {
    a0 = sd[iptr];
    a1 = sd[iptr + 1];
    a2 = sd[iptr + 2];
    scale = a2 + a1 + a0;
    sd[iptr] = (float)1.;
    sd[iptr + 1] = (a0 - a2) * (float)2. / scale;
    sd[iptr + 2] = (a2 - a1 + a0) / scale;
    a0 = sn[iptr];
    a1 = sn[iptr + 1];
    a2 = sn[iptr + 2];
    sn[iptr] = (a2 + a1 + a0) / scale;
    sn[iptr + 1] = (a0 - a2) * (float)2. / scale;
    sn[iptr + 2] = (a2 - a1 + a0) / scale;
    iptr += 3;
    }
    return TRUE;
}

/* WARP -- FUNCTION, APPLIES TANGENT FREQUENCY WARPING TO COMPENSATE */
/*         FOR BILINEAR ANALOG -> DIGITAL TRANSFORMATION */
/* ARGUMENTS: */
/* ---------- */
/*      F       ORIGINAL DESIGN FREQUENCY SPECIFICATION (HERTZ) */
/*      TS      SAMPLING INTERVAL (SECONDS) */
/*  LAST MODIFIED:  SEPTEMBER 20, 1990 */

static double warp(f, ts)
float *f, *ts;
{
    /* System generated locals */
    float ret_val;

    /* Local variables */
    float angle, twopi;

    twopi = (float)6.2831853;
    angle = twopi * *f * *ts / (float)2.;
    ret_val = tan(angle) * (float)2. / *ts;
    ret_val /= twopi;
    return ret_val;
}

/* BEROOTS -- SUBROUTINE TO RETURN BESSEL POLES FOR */
/*   NORMALIZED LOWPASS FILTER */
/* LAST MODIFIED:  SEPTEMBER 7, 1990 */
/*  OUTPUT ARGUMENTS: */
/*  ----------------- */
/*      P              COMPLEX ARRAY CONTAINING POLES */
/*                       CONTAINS ONLY ONE FROM EACH */
/*                       COMPLEX CONJUGATE PAIR, AND */
/*                       ALL REAL POLES */
/*      RTYPE          CHARACTER ARRAY INDICATING 2ND ORDER SECTION */
/*                       TYPE: */
/*                         (SP)  SINGLE REAL POLE */
/*                         (CP)  COMPLEX CONJUGATE POLE PAIR */
/*                         (CPZ) COMPLEX CONJUGATE POLE-ZERO PAIRS */
/*      DCVALUE        MAGNITUDE OF FILTER AT ZERO FREQUENCY */
/*  INPUT ARGUMENTS: */
/*  ---------------- */
/*      IORD           DESIRED FILTER ORDER */

static int beroots(p, rtype, dcvalue, iord)
complex *p;
char *rtype;
float *dcvalue;
int iord;
{
    /* Parameter adjustments */
    rtype -= 3;
    --p;

    /* Function Body */
    if (iord == 1) {
    p[1].r = (float)-1., p[1].i = (float)0.;
    strncpy(rtype + 3, "SP", 2);
    } else if (iord == 2) {
    p[1].r = (float)-1.1016013, p[1].i = (float).6360098;
    strncpy(rtype + 3, "CP", 2);
    } else if (iord == 3) {
    p[1].r = (float)-1.0474091, p[1].i = (float).9992645;
    strncpy(rtype + 3, "CP", 2);
    p[2].r = (float)-1.3226758, p[2].i = (float)0.;
    strncpy(rtype + 6, "SP", 2);
    } else if (iord == 4) {
    p[1].r = (float)-.9952088, p[1].i = (float)1.2571058;
    strncpy(rtype + 3, "CP", 2);
    p[2].r = (float)-1.3700679, p[2].i = (float).4102497;
    strncpy(rtype + 6, "CP", 2);
    } else if (iord == 5) {
    p[1].r = (float)-.9576766, p[1].i = (float)1.4711244;
    strncpy(rtype + 3, "CP", 2);
    p[2].r = (float)-1.3808774, p[2].i = (float).7179096;
    strncpy(rtype + 6, "CP", 2);
    p[3].r = (float)-1.502316, p[3].i = (float)0.;
    strncpy(rtype + 9, "SP", 2);
    } else if (iord == 6) {
    p[1].r = (float)-.9306565, p[1].i = (float)1.6618633;
    strncpy(rtype + 3, "CP", 2);
    p[2].r = (float)-1.3818581, p[2].i = (float).9714719;
    strncpy(rtype + 6, "CP", 2);
    p[3].r = (float)-1.5714904, p[3].i = (float).3208964;
    strncpy(rtype + 9, "CP", 2);
    } else if (iord == 7) {
    p[1].r = (float)-.9098678, p[1].i = (float)1.8364514;
    strncpy(rtype + 3, "CP", 2);
    p[2].r = (float)-1.3789032, p[2].i = (float)1.1915667;
    strncpy(rtype + 6, "CP", 2);
    p[3].r = (float)-1.6120388, p[3].i = (float).5892445;
    strncpy(rtype + 9, "CP", 2);
    p[4].r = (float)-1.6843682, p[4].i = (float)0.;
    strncpy(rtype + 12, "SP", 2);
    } else if (iord == 8) {
    p[1].r = (float)-.892871, p[1].i = (float)1.9983286;
    strncpy(rtype + 3, "CP", 2);
    p[2].r = (float)-1.3738431, p[2].i = (float)1.3883585;
    strncpy(rtype + 6, "CP", 2);
    p[3].r = (float)-1.6369417, p[3].i = (float).8227968;
    strncpy(rtype + 9, "CP", 2);
    p[4].r = (float)-1.7574108, p[4].i = (float).2728679;
    strncpy(rtype + 12, "CP", 2);
    }
    nsects = iord - iord / 2;
    *dcvalue = (float)1.;
    return TRUE;
}

static complex cmul(a,b)
complex a,b;
{
    complex c;
    c.r = a.r * b.r - a.i * b.i;
    c.i = a.i * b.r + a.r * b.i;
    return c;
}

static complex cpowi(a,n)
complex a;
int n;
{
    int i;
    complex c;
    c.i = 0.;
    c.r = 1.;
    for (i = 1; i <= n; ++i) {
    c = cmul(c, a);
    }
    return c;

}
static complex csqrt1(z)
complex z;
{   complex c;
    float x,y,w,r;
    if ((z.r == 0.0) && (z.i == 0.0)) {
        c.r=0.0;
        c.i=0.0;
        return c;
    } else {
        x=fabs(z.r);
        y=fabs(z.i);
        if (x >= y) {
            r=y/x;
            w=sqrt(x)*sqrt(0.5*(1.0+sqrt(1.0+r*r)));
        } else {
            r=x/y;
            w=sqrt(y)*sqrt(0.5*(r+sqrt(1.0+r*r)));
        }
        if (z.r >= 0.0) {
            c.r=w;
            c.i=z.i/(2.0*w);
        } else {
            c.i=(z.i >= 0) ? w : -w;
            c.r=z.i/(2.0*c.i);
        }
        return c;
    }
}

static complex conjg(z)
complex z;
{   complex c;
    c.r=z.r;
    c.i = -z.i;
    return c;
}

static complex cdiv(a,b)
complex a,b;
{   complex c;
    float r,den;
    if (fabs(b.r) >= fabs(b.i)) {
        r=b.i/b.r;
        den=b.r+r*b.i;
        c.r=(a.r+r*a.i)/den;
        c.i=(a.i-r*a.r)/den;
    } else {
        r=b.r/b.i;
        den=b.i+r*b.r;
        c.r=(a.r*r+a.i)/den;
        c.i=(a.i*r-a.r)/den;
    }
    return c;
}
//*=========================================================================      
//*                                                                               
//* Routine to convert from day-of-year to date                                   
//*                                                         Leif Kvamme 12-1-85   
void DTE (DOY,DAY,MON,YR)                                          
int DOY;
int *DAY;
int *MON;
int YR;
{
int MTH[14],J,M,N;
for(J=1;J<8;J=J+2)
    MTH[J] = 31;
for(J=8;J<13;J=J+2)
    MTH[J] = 31;
MTH[2] = 28;
MTH[4] = 30;
MTH[6] = 30;
MTH[9] = 30;
MTH[11]= 30;
if ((YR%4) == 0) MTH[2] = 29;
M = 0;                                                                    
for(J=1;J<13;J++)
{
    M = M + MTH[J];
    N = DOY - M;                                                              
    if (N <= 0)
    {                                                       
        *MON = J;
        *DAY = N + MTH[J];                                                     
        goto four;                                                               
    }
}
four:                                                                   
J=0;
}
void TIMSEC (YR,MTH,DAY,HR,MIN,SECS,MSECS)                         
int YR;
int MTH;
int DAY;
int HR;
int MIN;
float SECS;
double *MSECS;                         /* total seconds to be returned      */
{
int DYR;                               /* flag for leap year                */
int IYR;                               /* number of leap-years since 1900   */
int YDY;                               /* number of days in current year    */

if(YR <= 50) YR=YR+100;                /* check for 2000 or more in 2 digits*/

if(YR >= 1900) YR=YR - 1900;
DYR = 0;
if((YR % 4) == 0) DYR = 1;
IYR = YR/4 - DYR;
//                -- Seconds to beginning of     
*MSECS = (float)(IYR*366) + (float)((YR-IYR)*365);
//                -- current year                
*MSECS = *MSECS*86400.0;
//                -- January                     
if (MTH == 1) YDY = DAY;
//                -- February                    
if (MTH == 2) YDY = DAY + 31;
//                -- ....                        
if (MTH == 3) YDY = DAY + DYR + 59;       
if (MTH == 4) YDY = DAY + DYR + 90;                                     
if (MTH == 5) YDY = DAY + DYR + 120;                                    
if (MTH == 6) YDY = DAY + DYR + 151;                                    
if (MTH == 7) YDY = DAY + DYR + 181;                                    
if (MTH == 8) YDY = DAY + DYR + 212;                                    
if (MTH == 9) YDY = DAY + DYR + 243;                                    
if (MTH ==10) YDY = DAY + DYR + 273;                                    
if (MTH ==11) YDY = DAY + DYR + 304;                                    
if (MTH ==12) YDY = DAY + DYR + 334;                                    
*MSECS = *MSECS + (float)(YDY*86400 + HR*3600 + MIN*60) + SECS;
}







