#include <errno.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>
#include <sys/types.h>
#include "gdfontl.h"
#include "gdfontg.h"
#include "gdfonts.h"
#include "gd.h"

#define OK       0
#define NO_INPUT 1
#define TOO_LONG 2

char *topdir        = 0;               // top directory SEISAN
char *topdir_rt     = 0;               // top directory RTQUAKE

static int getLine (char *prmpt, char *buff, size_t sz) {
    int ch, extra;

    // Get line with buffer overrun protection.
    if (prmpt != NULL) {
        printf ("%s", prmpt);
        fflush (stdout);
    }
    if (fgets (buff, sz, stdin) == NULL)
        return NO_INPUT;

    // If it was too long, there'll be no newline. In that case, we flush
    // to end of line so that excess doesn't affect the next call.
    if (buff[strlen(buff)-1] != '\n') {
        extra = 0;
        while (((ch = getchar()) != '\n') && (ch != EOF))
            extra = 1;
        return (extra == 1) ? TOO_LONG : OK;
    }

    // Otherwise remove newline and give string back to caller.
    buff[strlen(buff)-1] = '\0';
    return OK;
}
main()
{
  char cfgfile[200];
  char path[200];
  char start_time[20];
  char stopp_time[20];
  char dbase[20];
  char syscmd[200];
  int  rc;
  int  i;
  int  len;
  char yymm[20];
  char dd[20];
  char hh[20];
  char yr[20];
  char mo[20];
  char mi[20];
  char se[20];  
//------------------------------------------------------------------------
// set path to SEISAN_TOP as default
//------------------------------------------------------------------------
  topdir = (char*)getenv("SEISAN_TOP");
  if(topdir)
  {
    printf("RTPICK: SEISAN_TOP.......................: %s\n", topdir);    
  }else{
    printf("RTPICK: SEISAN_TOP not defined.\n");
    printf("Install SEISAN !\n");
    exit(0);    
  }
//------------------------------------------------------------------------
// set path to RTQUAKE_TOP as default
//------------------------------------------------------------------------
  topdir_rt = (char*)getenv("RTQUAKE_TOP");
  if (topdir_rt)
  {
    printf("RTPICK: RTQUAKE_TOP......................: %s\n", topdir_rt);
  }else{
    printf("RTPICK: RTQUAKE_TOP not defined\n");
    printf("Include setup_rt.bash or setup_rt.csh in your .bashrc or .csh\n");
    exit(0);      
  } 

    


  sprintf(cfgfile,"hyp.out");
  Check_Config(cfgfile);
  sprintf(cfgfile,"res.out");
  Read_Residuals(cfgfile);
  
}
Check_Config(char config[])
{
  FILE *par;
  FILE *sfl;

  char  path[256];
  char  station[50];
  char  resid[25];
  char  linjer[500];
  char  sline[500];
  char  du[10];
  char  ord[100];
  char  keyword[100];
  char  chkpar[500][256];

  int   tel,i,found;
  int   ret;
  int   length;

  par = fopen(config,"r");
  if(par == NULL)
  {
    printf("************RTDET-Read_Config_to_MEM**********************\n");
    printf("* %s does not exist.\n",config);
    printf("* Check your configuration file.                         *\n");
    printf("**********************************************************\n");
    exit(0);
  }
  tel=0;
  sfl = fopen("res.out","w");
  if(sfl == NULL)
  {
    printf("cant open res.out\n");
    exit(0); 
  }

  
  while ( fgets ( linjer, sizeof linjer, par ) != NULL ) /* read a line from file */
  {
//    printf("%3d %s",tel,linjer);
    for(i=0;i<200;i++)
    {
//      if(linjer[i] == 0xa)
//	linjer[i] = '\0';
    }
//    printf("%x\n",linjer[79]);
    if(linjer[79] == ' ')
    {
//      printf("%c%c%c%c%c\n",linjer[63],linjer[64],linjer[65],linjer[66],linjer[67]);
      if(linjer[65] != ' ')
      {
        station[0] = linjer[1];
	station[1] = linjer[2];
	station[2] = linjer[3];
	station[3] = linjer[4];
	station[4] = linjer[5];
	station[5] = '\0';
	resid[0]   = linjer[63];
	resid[1]   = linjer[64];
	resid[2]   = linjer[65];
	resid[3]   = linjer[66];
	resid[4]   = linjer[67];
	resid[5]   = '\0';
	fprintf(sfl,"%2d %s %s\n",tel,station,resid);
	tel++;
      }
    }
  }
  fclose(sfl);
  fclose(par);

}
Read_Residuals(char config[])
{
  FILE *par;
  FILE *out;
  int styleDotted[2], styleDashed[6];

  char  station[50];
  char  resid[25];
  char  linjer[500];
  char  du[10];
  int   num=0;
  float res;
  float ywin;
  float step;
  float zero;
  float seconds;
  char  pngfile[200];
  char  sekund[10];
  int   tel,i,found;
  int   ret;
  int   tmp;
  int    bg,sg1,sg2,sg3,sg4,tl,tt,tk,t5,dg;
  int   xsiz,ysiz,ymax; 
  
  

  par = fopen(config,"r");
  if(par == NULL)
  {
    printf("************RTDET-Read_Config_to_MEM**********************\n");
    printf("* %s does not exist.\n",config);
    printf("* Check your configuration file.                         *\n");
    printf("**********************************************************\n");
    exit(0);
  }
  tel=0;
  xsiz = 260;
  xsiz = 500;
  ysiz = 200;
  ywin = ysiz - 45;
  seconds = 8.0;
  step = ywin/seconds;
  zero = ywin/2;
//  printf("ywin: %5.2f\n",ywin);
//  printf("secs: %5.2f\n",seconds);
//  printf("step: %5.2f\n",step);
//  printf("zero: %5.2f\n",zero);
  sprintf(pngfile,"%s/loc/res.png",topdir_rt);    
  gdImagePtr im;                                           // Declare the image   
  im  = gdImageCreate(xsiz, ysiz);                        // Allocate image: tot_wd pixels width by tot_hg pixels high

  bg  = gdImageColorAllocate(im, 255, 255, 255); // WHITE first color definition defines the background color
  tl  = gdImageColorAllocate(im, 200, 200, 200); // GRAY  color for timelines
  tk  = gdImageColorAllocate(im,   0,   0,   0); // BLACK color for text
  tt  = gdImageColorAllocate(im,   0,   0, 255); // BLUE color for title
  t5  = gdImageColorAllocate(im,   0,   0,   0); // BLACK color for 5 minute marks
  sg1 = gdImageColorAllocate(im, 255,   0,   0); // RED   color seismogram
  sg2 = gdImageColorAllocate(im,   0, 255,   0); // GREEN color seismogram
  sg3 = gdImageColorAllocate(im,   0,   0, 255); // BLUE color seismogram
  sg4 = gdImageColorAllocate(im,   0,   0,   0); // BLACK color seismogram 
  
/* Set up dashed style. Three on, three off. */
styleDashed[0] = tk;
styleDashed[1] = tk;
styleDashed[2] = tk;
styleDashed[3] = gdTransparent;
styleDashed[4] = gdTransparent;
styleDashed[5] = gdTransparent;  
  

  gdImageLine(im, 0, 0, xsiz, 0, sg4);            // draw frame  
  gdImageLine(im, xsiz-1, 0, xsiz-1, ysiz-45, sg4);        // draw frame
  gdImageLine(im, xsiz-1, ysiz-45, 0, ysiz-45, sg4);        // draw frame
  gdImageLine(im, 0, ysiz-45, 0, 0, sg4);            // draw frame   
  for(i=1;i<(int)seconds;i++)
  {
    tmp = (int)((seconds)/2-i);
    if(tmp == 0)
    {
      gdImageSetThickness(im, 2);  
    }else{
      gdImageSetThickness(im, 1);      
    }
    gdImageLine(im, 20, (int)(step*i), xsiz-1, (int)(step*i), sg4);            // draw lines 
    if(tmp >= 0)
    {
      sprintf(sekund,"%2d",(int)(seconds)/2-i);
    }else{
      sprintf(sekund,"%d",(int)(seconds)/2-i);      
    }
    gdImageString(im,gdFontGetLarge(),2,(int)(step*i)-7,sekund,sg3);
  }
  
//  sprintf(station,"Residuals(sec)");
//  gdImageStringUp(im,gdFontGetLarge() , 7, 150, station, sg4);  
  while ( fgets ( linjer, sizeof linjer, par ) != NULL ) /* read a line from file */
  {

//    printf("%s",linjer);
/* Now the dashed line. */
    gdImageSetStyle(im, styleDashed, 6);
    gdImageLine(im, 20+num*15+7, 0, 20+num*15+7, ysiz-45, gdStyled);
    
    sscanf(linjer,"%d %s %f",&num,station,&res);
    gdImageStringUp(im,gdFontGetLarge() , 20+num*15, ysiz-2, station, sg4);
    

    
    ymax = (int)(zero-(step*res));
//    printf("ymax: %d\n",ymax);
    if(ymax > (int)ywin)
      gdImageFilledRectangle(im, 20+num*15, ywin, 20+num*15+15, (int)zero, sg1);
    if(ymax < 0)
      gdImageFilledRectangle(im, 20+num*15, 0, 20+num*15+15, (int)zero, sg1);    
    if(ymax < (int)ywin && ymax > 0)      
      gdImageFilledRectangle(im, 20+num*15, (int)(zero-(step*res)), 20+num*15+15, (int)zero, sg2);
    
//    num++;
  }
//  num++;
gdImageSetStyle(im, styleDashed, 6);
gdImageLine(im, 20+num*15+7, 0, 20+num*15+7, ysiz-45, gdStyled);  
  fclose(par);
//--------------------------------------------------
// write to png file
//--------------------------------------------------
    out = fopen (pngfile, "wb");
    gdImagePng (im, out);
    fclose (out);
    gdImageDestroy(im);                   /* Destroy the image in memory. */    
}