/***************************************************************************
 * rt24.c
 * SeedLink client using libslink (Chad Trabant).
 *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <math.h>		                 // For math routines (such as sqrt & trig).
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>
#include "gd.h"
#include <libslink.h>

#ifdef WIN32                                     // WINDOWS
  #include "glut.h"
  #include <process.h>
#else
  #include <GL/glut.h>                           // LINUX
#endif


#ifndef WIN32                                    
  #include <signal.h>

  static void term_handler (int sig);
#else
  #define strcasecmp _stricmp
#endif

#define PACKAGE "rt24"
#define VERSION LIBSLINK_VERSION

static short int verbose  = 0;
static short int ppackets = 0;
static char * statefile   = 0;	                 /* state file for saving/restoring state */
static char *dumpfile     = 0;	                 /* output file for data dump */
static char *location     = "1";                 // switch to read data for plotting
static FILE *outfile      = 0;                   /* the descriptor for the dumpfile */

#define nch 35                                   // MAX NUMBER OF CHANNELS
#define NBSCALE 8                                // NUMBER OF BUFFERS SCALING

//****************  FILTER  **********************************
/* Type-independant absolute value for C standard types */
#ifndef abs
#define ABS(x) ((x)<0?-(x):(x))
#endif
#define TRUE 1
static  int nsects;
static  float sn[5*3+1],sd[5*3+1];
char *logol       = 0;                 // name of left logo gif file
char *logor       = 0;                 // name of right logo jpg file
float   snc[nch][16];                            // filter coefficients
float   sdc[nch][16];                            // filter coefficients
int     cset[nch];
int     rtdr24_or_rtdrum=1;
static  float save_y2[nch][16];                  // overlap between 2 filter buffers
static  float save_y1[nch][16];
static  float save_x2[nch][16];
static  float save_x1[nch][16];

 char *streamfile   = 0;                         /* stream list file for configuring streams */
 char *stationfile  = 0;                         /* station list file for reading streams */

char *wwwdir      = 0;                           // directory unfiltered heli plots
char *wwwfiltdir  = 0;                           // directory filtered heli plots

char wdir[256];
char wfiltdir[256];

int     to_wi = 1000;
int     fr_hg = 600;
int     mt    = 15;
int     col   = 0;

int rtdrum_counter;
  int   swsl=0;

float   fl;
float   fh;
float   eqavst;
int     orden;
int     ssrate;

typedef struct {float r, i;} complex;

/* Table of constant values */
static  float c_b12 = 2.;
static  complex c_b43 = {1.,0.};


/* Local function prototypes */
static  double warp();
static  int chebparm(), lptbp(), lptbr(), lpthp(), bilin2(), lp();
static  int c1roots(), c2roots();
static  int cutoffs(), beroots();
static  complex cmul(), cpowi(), csqrt(), conjg(), cdiv();

//****************  END FILTER  **********************************


static  SLCD * slconn;	                         // connection parameters 
SLpacket * slpack;
int     seqnum;
int     ptype;
int     fnd=0;

int     printarg = 1;

char *topdir_rt = 0;                          // top directory RTQUAKE

FILE *nr00;
FILE *nr02;
FILE *nr03;


char   syscmd[400];
char   syscmd1[400];

int    sw2[nch];
int    sw3[nch];
int    sw4[nch];
int    sw5[nch];
int    sw1[nch];
int    sw50[nch];
int    rotate[nch];
int    sum[nch];
int    teller[nch];
int    avg[nch];

float amp=1.0;


int k;
int i;
int m;
int j;
int stopwatch = 0;
int stno = 0;
int only_one = 0;

int auto_scaling = 0;
int antall = 0;

int    init_sw=0;
int    ns[nch];

float  promedio[nch];
float  factor[nch];
float  scale_factor[nch];
long   ryrn[nch];
long   rmon[nch];
long   rday[nch];
long   rhrn[nch];
long   rmin[nch];

int    ryrn_int;
int    rmon_int;
int    rday_int;
int    rhrn_int;
int    rmin_int;

int    risec[nch];
float  rsek[nch];
int    rsecs[nch];
float  ftime[nch];
float  stpix[nch];
int    sr[nch];
float  amp_single[nch];
float  latsnitt[nch];
int    trigger[nch];
int    active[nch];

float  csta[nch];
float  clta[nch];
int    hold[nch];
float  rat[nch];

float  v[nch];
float  incr[nch];
char   rtid[nch][80];
char   lat[nch][80];
char   snr[nch][80];
char   tid[nch][80];
int    ix[nch];
int    time_quality[nch];

int    counter[nch];
int    current_channel = 0;
int    filter_rescale_counter = 0;
int    rescale_counter = 0;



  double TIMESTART;
  double TIMESTOPP;
  double TIMEDIFF;

int    start_stop = 0;
char str[80];
char last[80];
int  seismo[1000];
float fseismo[1000];
int RunMode = 1;		                 // Used as a boolean (1 or 0) for "on" and "off"
float xold=-500.0;
float xnew=-500.0;
float yold=0.0;
float ynew=0.0;
int nsold=0;
int ctrs[nch];
char toplot[nch][20];
char tmpfiles[nch][20];
char staname[nch][50];
char stnm[20];
char dum[20];
float ufilt;
float filt;
float unfiltered[nch];
float filtered[nch];
char fullna[nch][80];
float flow;
float fhigh;
float tflow[nch];
float tfhigh[nch];
int no_to_plot;
int limit[nch];

char tags[20][50];
float tags_ix[50];
int tag_index=0;

 

static int packet_handler (char *msrecord, int packet_type,
			    int seqnum, int packet_size);
static int parameter_proc (int argcount, char **argvec);
static char *getoptval (int argcount, char **argvec, int argopt);
static void usage (void);
static void FilterRescale(void);
static void Rescale(void);
static void appl(int fnd);
static int  DET_MEM(int kan);
static void DTE();
static void TIMSEC();
static void strip(int line);
static void bitmap_output(int x, int y, char *string,void *font);
static void STA_LTA();
static int  unpack_samples();
static void design();
static int  buroots();


void drawScene(void)
{
  int ch;

  int ret;
  int DDAY,MMTH;
  int DY;
  int YR;
  int xmax=0;


  double MSECS;
  double MSECSR;
  char cmpfile[80];
  char ctid[80];
  char fullpath[100];
  int ne;

  long   yrn;
  long   mon;
  long   day;
  long   hrn;
  long   min;
  float  sek;
  int    min_int;
  int    day_int;
  int    yrn_int;
  int    mon_int;
  int    hrn_int;
  char dum[50];

  for(k=0;k<80;k++)
    cmpfile[k]='\0';
  /* Loop with the connection manager */
//  while ( sl_collect (slconn, &slpack) )

  if( sl_collect (slconn, &slpack))              // Connect to seedlink server
  {
    ptype  = sl_packettype (slpack);             // get packet type
    seqnum = sl_sequence (slpack);               // get sequence number
    fnd=packet_handler ((char *) &slpack->msrecord, ptype, seqnum, SLRECSIZE);
    if(fnd != -1)                                // PACKET FOUND ?
    {
      ch=fnd;   // YES, SAVE STATION NUMBER
//      printf("%3d\n",ch);
      ctrs[ch]++;
      if(swsl == 0)
      {
	printf("Receiving data from Seedlink server.......\n");
	swsl = 1;
      }
//      printf("%2d packet found NS: %d eqavst: %f sr: %d %s\n",ch,ns[ch],eqavst,ssrate,toplot[ch]);
      fnd=-1;                                    // RESET 'fnd'
    
      for(k=0;k<ns[ch];k++)                      // REMOVE DC
        fseismo[k]=(float)(seismo[k]-avg[ch]);

      sscanf(rtid[ch],"%4d%1c%3d%1c%2d%1c%2d%1c%f7.4",&YR,dum,&DY,dum,&hrn_int,dum,&min_int,dum,&sek);
      hrn = hrn_int;
      min=min_int;
      DTE (DY,&DDAY,&MMTH,YR);
      yrn=YR;
      mon=MMTH;
      day=DDAY;
////////////////////////////////////////////////////////////
      TIMSEC(yrn,mon,day,hrn,min,sek,&MSECS);
      TIMEDIFF = MSECS - 86400.0;
//      printf("%4d %3d %2d %2d %7.4f TIME: %4d %2d %2d %2d %2d %7.4f MSECS: %10.4f\n",YR,DY,hrn,min,sek,yrn,mon,day,hrn,min,sek,MSECS);
////////////////////////////////////////////////////////////
//      printf("TIME: %4d %2d %2d %2d %2d %7.4f\n",yrn,mon,day,hrn,min,sek);
      day_int = day;
      yrn_int = yrn;
      mon_int = mon;
      sprintf(fullpath,"%s/rt/tmp/%s.%4d%2d%2d.%3d",topdir_rt,tmpfiles[ch],yrn_int,mon_int,day_int,DY);
      for(k=0;k<100;k++)
        if(fullpath[k] == ' ') fullpath[k] = '0';
//      printf("fullpath: %s\n",fullpath);

      if ((nr00 = fopen (fullpath,"a")) == NULL) // open file unfiltered data 
      {
        printf("Can't open: %s\n",fullpath);
      }

      ne = ns[ch];
//      printf("ne: %d ne*4: %d rtid: %s\n",ne,ne*4,rtid[ch]);
      hrn_int = hrn;
      min_int = min;
      sscanf(rtid[ch],"%4d%1c%3d%1c%2d%1c%2d%1c%f7.4",&YR,dum,&DY,dum,&hrn_int,dum,&min_int,dum,&sek);

      fwrite(&ne,4,1,nr00);                      // write number of samples
      fwrite(&ssrate,4,1,nr00);                  // write samplerate
      sprintf(ctid,"%s",rtid[ch]);
      fwrite(ctid,22,1,nr00);                    // write timestamp
      ne = ne*4;
      fwrite(seismo,ne,1,nr00);                  // write ne samples
      fclose(nr00);                              // close file


      fl=tflow[ch];
      fh=tfhigh[ch];
      orden=4;
      design(orden,"BP","BU",30.0,0.3,fl,fh,eqavst);
      for(j=0;j<(nsects*3+1);j++)
      {
        snc[ch][j]=sn[j];
        sdc[ch][j]=sd[j];
      }
      cset[ch]=nsects;
      appl(ch);                                  // FILTER ONE BUFFER
      for(k=0;k<ns[ch];k++)
        seismo[k] = (int)fseismo[k];

      sprintf(fullpath,"%s/rt/tmp_filt/%s.%4d%2d%2d.%3d",topdir_rt,tmpfiles[ch],yrn_int,mon_int,day_int,DY);
      yrn = yrn_int;
      mon = mon_int;
      day = day_int;
      for(k=0;k<100;k++)
        if(fullpath[k] == ' ') fullpath[k] = '0';
//        printf("fullpath: %s\n",fullpath);

      if ((nr00 = fopen (fullpath,"a")) == NULL) // open file filtered data 
      {
        printf("Can't open: %s\n",fullpath);
      }

      ne = ns[ch];
//      printf("ne: %d ne*4: %d rtid: %s\n",ne,ne*4,rtid[ch]);
      sscanf(rtid[ch],"%4d%1c%3d%1c%2d%1c%2d%1c%f7.4",&YR,dum,&DY,dum,&hrn_int,dum,&min_int,dum,&sek);
      hrn = hrn_int;
      min = min_int;
//      printf("%4d %3d %2d %2d %7.4f\n",YR,DY,hrn,min,sek);
      fwrite(&ne,4,1,nr00);                      // write number of samples
      fwrite(&ssrate,4,1,nr00);                  // write samplerate
      sprintf(ctid,"%s",rtid[ch]);
      fwrite(ctid,22,1,nr00);                    // write timestamp
      ne = ne*4;
      fwrite(seismo,ne,1,nr00);                  // write ne samples
      fclose(nr00);                              // close file
//printf("%4d %4d\n",ctrs[ch],limit[ch]);
      if(ctrs[ch] >= limit[ch])
      {
        printf("%2d   ",ch);
        for(i=0;i<15;i++)
          printf(" %3d",ctrs[i]);
        printf(" %s\n",staname[ch]);
        for(i=0;i<50;i++)
	{
	  if(staname[ch][i] == ' ')
	    staname[ch][i]='_';
	}

	if(rtdr24_or_rtdrum == 0)
	{
          sprintf(syscmd1,"%s/bin/rtdr24 -logol %s -logor %s -flt -top %s -n %10.4f -fl %8.3f -fh %8.3f -col %d -to_wi %d -fr_hg %d -mt %d -gain %f -upd 15 -comp %s -name %s",topdir_rt,logol,logor,topdir_rt,MSECS,fl,fh,col,to_wi,fr_hg,mt,filtered[ch],tmpfiles[ch],staname[ch]);
          printf("%s\n",syscmd1);
          ret=system(syscmd1);

          sprintf(syscmd1,"%s/bin/rtdr24 -logol %s -logor %s -top %s -n %10.f  -fl %8.3f -fh %8.3f -col %d -to_wi %d -fr_hg %d -mt %d -gain %f -upd 15 -comp %s -name %s &",topdir_rt,logol,logor,topdir_rt,MSECS,fl,fh,col,to_wi,fr_hg,mt,unfiltered[ch],tmpfiles[ch],staname[ch]);
          printf("%s\n",syscmd1);
          ret=system(syscmd1);
          ctrs[ch]=0;
	}else{
	  sprintf(syscmd1,"%s/bin/rtdrum -logol %s -logor %s -flt -top %s -col %d -to_wi %d -fr_hg %d -mt %d -gain %f -upd 15 -comp %s -name %s",topdir_rt,logol,logor,topdir_rt,col,to_wi,fr_hg,mt,filtered[ch],tmpfiles[ch],staname[ch]);
          printf("%s\n",syscmd1);
          ret=system(syscmd1);

          sprintf(syscmd1,"%s/bin/rtdrum -logol %s -logor %s -top %s -col %d -to_wi %d -fr_hg %d -mt %d -gain %f -upd 15 -comp %s -name %s &",topdir_rt,logol,logor,topdir_rt,col,to_wi,fr_hg,mt,unfiltered[ch],tmpfiles[ch],staname[ch]);
          printf("%s\n",syscmd1);
          ret=system(syscmd1);
          ctrs[ch]=0;
	  
	  sprintf(syscmd1,"%s/bin/rthplt &",topdir_rt);
	  ret=system(syscmd1);
	}
      
      }

      sscanf(tid[ch],"%4d%1c%3d%1c%2d%1c%2d%1c%f4.1",&YR,dum,&DY,dum,&hrn_int,dum,&min_int,dum,&sek);
      hrn = hrn_int;
      min = min_int;

//-- convert from doy to date  
      DTE (DY,&DDAY,&MMTH,YR);
      yrn=YR;
      mon=MMTH;
      day=DDAY;

      TIMSEC(yrn,mon,day,hrn,min,sek,&MSECS);

      latsnitt[ch]=latsnitt[ch]+MSECSR-MSECS;
      counter[ch]++;

    }
  }

}


tu_read_stationlist ( const char * streamfile)
{
  FILE *streamfp;
  char linje[80];
  char station[80];
  char navn[80];
  char tempo[80];
  char filnam1[80];
  char filnavn[80];
  int i,l,p,no;
  int sw7=0;
  int br=0;
/* Open the station list file */
  sprintf(filnam1,"%s/par/%s",topdir_rt,streamfile);
//  printf("Stationfile: %s\n",filnam1);
  while(sw7 == 0)
  {
    if ((streamfp = fopen (filnam1, "rb")) == NULL)
    {
//      printf("Can't open: %s\n",filnam1);
//      exit(0);
      sw7=0;
    }else{
      sw7=1;
    }
  }
  if(antall == 1)
  {
    no = no_to_plot;
  }else{
    no = 200;
  }
  i=0;
  l=0;
  for(i=0;i<no;i++)
  {
    if((fgets(linje, 70, streamfp)) != NULL)
    {
//      printf("%2d %s",i,linje);

      sprintf(tempo,"%s",linje);
      for(l=0;l<80;l++)
      {
        station[l]='\0';
        navn[l]='\0';
        filnavn[l]='\0';
      }
      strncpy(station,linje,10);
      strncpy(filnavn,station,10);
      for(l=0;l<10;l++)
      {
        if(filnavn[l] == ' ')
          filnavn[l] = '_';
      }
//      printf("filnavn: %s station: %s\n",filnavn,station);
      sprintf(tmpfiles[i],"%s",filnavn);

      sprintf(toplot[i], "%s",station);
//      printf("toplot: %s\n",toplot[i]);
      for(p=0;p<70;p++)
	linje[p]=linje[p+10];
      sscanf(linje," %f %f %f %f %50[^\n]\n",&ufilt,&filt,&flow,&fhigh,stnm);
//printf("%s %f %f %f %f %s\n",toplot[i],ufilt,filt,flow,fhigh,stnm);
      unfiltered[i]=ufilt;
      filtered[i]=filt;
      tflow[i]=flow;
      tfhigh[i]=fhigh;
      sprintf(staname[i],"%s",stnm);
      for(l=0;l<40;l++)
        navn[l]=linje[l+11];
      sprintf(fullna[i],"%s",navn);
      for(l=0;l<40;l++)
        if(fullna[i][l] == '\n') fullna[i][l] = '\0';
//	  printf("%s\n",fullna[i]);
    }else{
      break;
    }
  }
//        exit(0);
//  printf("End of file. File contains: %2d items\n",i);
  no_to_plot = i;
  fclose(streamfp);
}

void FilterRescale()
{
  int l;

  amp=1.0;
  for(l=0;l<nch;l++)
  {
    sw4[l]=0;
    teller[l]=0;
    v[l]=0.0;
    sum[l]=0.0;
	factor[l]=0.0;
	promedio[l]=0.0;
//	scale_factor[l]=0.0;
  }
  for(l=0;l<nch;l++)
    amp_single[l]=4.0;
}
/*************************************/
/* RE-INITIALIZE ALL SCALING VALUES  */
/*************************************/
void Rescale()
{
  int l;
  amp=1.0;
  for(l=0;l<nch;l++)
  {
    sw2[l]=0;
	sw4[l]=0;
    teller[l]=0;
    v[l]=0.0;
    sum[l]=0.0;
    avg[l]=0.0;
	factor[l]=0.0;
//	scale_factor[l]=0.0;
	promedio[l]=0.0;
  }
  for(l=0;l<nch;l++)
    amp_single[l]=4.0;
}






int main (int argc, char **argv)
{

//------------------------------------------------------------------------
// set path to RTQUAKE_TOP as default
//------------------------------------------------------------------------
  topdir_rt = (char*)getenv("RTQUAKE_TOP");
  if (topdir_rt)
  {
    printf("RT24: RTQUAKE_TOP: %s\n", topdir_rt);
  }else{
    printf("RT24: RTQUAKE_TOP not defined\n");
    printf("Include setup_rt.bash or setup_rt.csh in your .bashrc or .csh\n");
    exit(0);      
  } 
  sprintf(wdir,"%s/rt/png",topdir_rt);
  wwwdir=wdir;
  sprintf(wfiltdir,"%s/rt/png_filt",topdir_rt);
  wwwfiltdir=wfiltdir;

  fl=0.0;
  fh=0.0;
  
// Allocate and initialize a new connection description SEEDLINK
  slconn = sl_newslcd();

// Process command line parameters
  if (parameter_proc (argc, argv) < 0)
  {
    fprintf(stderr, "Parameter processing failed\n\n");
    fprintf(stderr, "Try '-h' for detailed help\n");
    return -1;
  }
  /* initialize random seed: */
  srand ( time(NULL) );

  Rescale();                                     // RE-INITIALIZE SCALE VALUES
  FilterRescale();
  for(i=0;i<nch;i++)                             // INITIALIZE 
  {
    ctrs[i]=0;
    sw50[i]=0;
    rotate[i]=0;
    limit[i] = i + 10 + i;
  }


  rtdrum_counter = 0;

  for(;;)
  {
    drawScene();
    tu_read_stationlist (stationfile);
    rtdrum_counter++;
  }
  return 0;
}				/* End of main() */



/***************************************************************************
 * packet_handler():
 * Process a received packet based on packet type.
 ***************************************************************************/
static int
packet_handler (char *msrecord, int packet_type, int seqnum, int packet_size)
{
  static SLMSrecord * msr = NULL;
  int trace;
  double dtime;			/* Epoch time */
  double secfrac;		/* Fractional part of epoch time */
  time_t itime;			/* Integer part of epoch time */
  char timestamp[20];
  struct tm *timep;

  /* The following is dependent on the packet type values in libslink.h */
  char *type[]  = { "Data", "Detection", "Calibration", "Timing",
		    "Message", "General", "Request", "Info",
                    "Info (terminated)", "KeepAlive" };

  /* Build a current local time string */

  dtime   = sl_dtime ();
  secfrac = (double) ((double)dtime - (int)dtime);
  itime   = (time_t) dtime;
  timep   = localtime (&itime);
  sprintf (timestamp,  "%04d.%03d.%02d:%02d:%02d.%01.0f",
	    timep->tm_year + 1900, timep->tm_yday + 1, timep->tm_hour,
	    timep->tm_min, timep->tm_sec, secfrac);

  /* Process waveform data */
  if ( packet_type == SLDATA )
    {
      sl_log (0, 1, "%s, seq %d, Received %s blockette:\n",
	      timestamp, seqnum, type[packet_type]);

      sl_msr_parse (slconn->log, msrecord, &msr, 1, 0);

      if ( verbose || ppackets )
	sl_msr_print (slconn->log, msr, ppackets);
    }
  else if ( packet_type == SLKEEP )
    {
      sl_log (0, 2, "Keep alive packet received\n");
    }
  else
    {
      sl_log (0, 1, "%s, seq %d, Received %s blockette\n",
	      timestamp, seqnum, type[packet_type]);
    }

  /* Write packet to dumpfile if defined */
  if ( dumpfile )
    {
      if (fwrite (msrecord, packet_size, 1, outfile) <= 0)
	sl_log (2, 0, "fwrite(): error writing data to %s\n", dumpfile);
//      memcpy(buf,msrecord,seed_size);
	sl_msr_parse (slconn->log, msrecord, &msr, 1, 1);
        trace=unpack_samples (msr);
        return(1);
    }
/* Plot selected data */
if( location )
{
//  printf("%s, seq %d, Received %s blockette:\n",timestamp, seqnum, type[packet_type]);
//  printf("smp:%s\n",timestamp);

  sl_msr_parse (slconn->log, msrecord, &msr, 1, 1);
  if(msr->unpackerr != MSD_UNKNOWNFORMAT)
  {
    trace=unpack_samples (msr,seqnum);
//printf("seqnum: %8d\n",seqnum);
    sprintf(tid[trace],"%s",timestamp);
    return(trace);
  }else{
	printf("Trouble unpacking buffer\n");
	return(0);
  }
}
return(0);

}				/* End of packet_handler() */

/***************************************************************************
 * print_samples:
 * Print samples in the supplied MSrecord with a simple format.
 ***************************************************************************/
int
unpack_samples (SLMSrecord *msr, int seqnum)
{
  char tempo[20];
  char search[20];
  char tmp[80];
  char dd[80];
  int ret2=0;
  int j,pp;
  int retur=-1;
  int cnt,p,l,ret;

  for(p=0;p<20;p++)
    search[p]='\0';
  sprintf(tempo,"%s",msr->fsdh.station);

  strncpy(search,tempo,10);
//  if(search[5] == ' ')search[5]='0';
//  if(search[6] == ' ')search[6]='0';
  for(l=0;l<no_to_plot;l++)
  {
//    printf("%s %s\n",search,toplot[l]);
    ret=strcmp(search,toplot[l]);
    if(ret == 0)
    {
      retur=l;
      if ( msr->datasamples != NULL )            //EXTRACT DATA FROM MINISEED BUFFER
      {
        if(msr->fsdh.samprate_fact > 0 && msr->fsdh.samprate_mult < 0)
		  msr->fsdh.samprate_fact = (-1 * msr->fsdh.samprate_fact)/msr->fsdh.samprate_mult;

        ns[l]   = msr->numsamples;
        ryrn[l] = msr->fsdh.start_time.year;
        rday[l] = msr->fsdh.start_time.day;
        rhrn[l] = msr->fsdh.start_time.hour;
        rmin[l] = msr->fsdh.start_time.min;
        risec[l]= msr->fsdh.start_time.sec;
        rsecs[l]= msr->fsdh.start_time.fract; 
        sr[l]   = msr->fsdh.samprate_fact;

        ryrn_int = ryrn[l];
        rday_int = rday[l];
        rhrn_int = rhrn[l];
        rmin_int = rmin[l];

        sprintf(tmp,"%4d %3d %2d:%2d:%2d.%4d",ryrn_int,rday_int,rhrn_int,rmin_int,risec[l],rsecs[l]);
        if(tmp[21]==' ')tmp[21]='0';
        if(tmp[20]==' ')tmp[20]='0';
        if(tmp[19]==' ')tmp[19]='0';
        if(tmp[18]==' ')tmp[18]='0';
        if(tmp[15]==' ')tmp[15]='0';

        sprintf(rtid[l],"%s",tmp);
        if(msr->fsdh.dq_flags == 0)
        {
          time_quality[l]=0;
        }else{
          time_quality[l]=1;
        }
        for(pp=0;pp<22;pp++)
          if(tmp[pp]==' ')tmp[pp]='0';   
        sscanf(tmp,"%15s%f7.4",dd,&ftime[l]);
//        printf("FTIME: %f\n",ftime[l]);

/* COMPUTE NEW FILTER COEF. EACH BUFFER. THEY MAY HAVE DIFFERENT SAMPLERATE */
        eqavst=1.0/(float)msr->fsdh.samprate_fact;

        ssrate=msr->fsdh.samprate_fact;

        for ( cnt = 0; cnt < ns[l]; cnt++ )      // EXTRACT DATA SAMPLES
          seismo[cnt]=*(msr->datasamples + cnt);
        if(sw2[l] < NBSCALE)                     // RE-SCALE ?
        {
          for(p=0;p<ns[l];p++)
          {
            sum[l]=sum[l]+seismo[p];
            teller[l]++;
          }
          avg[l]=sum[l]/teller[l];               // DC
          sw2[l]++;
        }  // sw2.........
      }    // if(msr->data.........
    }      // if(ret......
  }        // for(no_to_plot........ 
  return(retur);
}  /* End of print_samples() */
/***************************************************************************
 * parameter_proc:
 *
 * Process the command line parameters.
 *
 * Returns 0 on success, and -1 on failure
 ***************************************************************************/
static int
parameter_proc (int argcount, char **argvec)
{
  int optind;
  int error = 0;

  char tm[80];  
 char streamsfull[80];
  
// char *streamfile   = 0;      /* stream list file for configuring streams */
// char *stationfile  = 0;      /* station list file for reading streams */

  char *multiselect = 0;
  char *selectors   = 0;

  if (argcount <= 1)
    error++;

  /* Process all command line arguments */
  for (optind = 1; optind < argcount; optind++)
  {
      if (strcmp (argvec[optind], "-V") == 0)
      {
	  fprintf(stderr, "%s version: %s\n", PACKAGE, VERSION);
	  exit (0);
      }
      else if (strcmp (argvec[optind], "-h") == 0)
      {
	  usage();
	  exit (0);
      }
//      else if (strcmp (argvec[optind], "-c") == 0)
//      {
//          stationfile ="stations.conf";
//	  tu_read_stationlist (stationfile) ;
//	  exit (0);
//      }
      else if (strncmp (argvec[optind], "-v", 2) == 0)
      {
	  verbose += strspn (&argvec[optind][1], "v");
      }
//      else if (strcmp (argvec[optind], "-p") == 0)
//      {
//	  ppackets = 1;
//      }
//      else if (strcmp (argvec[optind], "-nt") == 0)
//      {
//	  slconn->netto = atoi (argvec[++optind]);
//      }
//      else if (strcmp (argvec[optind], "-nd") == 0)
//      {
//	  slconn->netdly = atoi (argvec[++optind]);
//      }
//      else if (strcmp (argvec[optind], "-k") == 0)
//     {
//	 slconn->keepalive = atoi (argvec[++optind]);
//      }
      else if (strcmp (argvec[optind], "-fl") == 0)
      {
	  fl = atof (argvec[++optind]);
      }
      else if (strcmp (argvec[optind], "-fh") == 0)
      {
	  fh = atof (argvec[++optind]);
      }
      else if (strcmp (argvec[optind], "-logol") == 0)
      {
	  logol = argvec[++optind];
      }
      else if (strcmp (argvec[optind], "-logor") == 0)
	{
	  logor = argvec[++optind];
	}
      else if (strcmp (argvec[optind], "-heli") == 0)
      {
	  rtdr24_or_rtdrum = atoi (argvec[++optind]);
      }
//      else if (strcmp (argvec[optind], "-top") == 0)
//      {
//	  topdir_rt = argvec[++optind];
//          printf("top directory: %s\n",topdir_rt);
//      }

//      else if (strcmp (argvec[optind], "-o") == 0)
//      {
//	  dumpfile = getoptval(argcount, argvec, optind++);
//      }
//      else if (strcmp (argvec[optind], "-g") == 0)
//      {
//	  location = getoptval(argcount, argvec, optind++);
//      }

//      else if (strcmp (argvec[optind], "-n") == 0)
//      {
//	  no_to_plot = atoi(argvec[++optind]);
//          antall=1;
//      }
//      else if (strcmp (argvec[optind], "-a") == 0)
//      {
//	  stno = atoi(argvec[++optind]);
//          only_one = 1;
//          printf("stno: %d\n",stno);
//      }

//      else if (strcmp (argvec[optind], "-sc") == 0)
//      {
//          auto_scaling = 1;
//      }
      else if (strcmp (argvec[optind], "-l") == 0)
      {
	  streamfile = argvec[++optind];
      }
      else if (strcmp (argvec[optind], "-f") == 0)
      {
	  stationfile = argvec[++optind];
      }
//      else if (strcmp (argvec[optind], "-s") == 0)
//      {
//	  selectors = argvec[++optind];
//      }
//      else if (strcmp (argvec[optind], "-S") == 0)
//      {
//	  multiselect = argvec[++optind];

//      }
      else if (strcmp (argvec[optind], "-col") == 0)
      {
        col = atoi(argvec[++optind]);
      }
      else if (strcmp (argvec[optind], "-to_wi") == 0)
      {
        to_wi = atoi(argvec[++optind]);
      }
      else if (strcmp (argvec[optind], "-fr_hg") == 0)
      {
        fr_hg = atoi(argvec[++optind]);
      }
      else if (strcmp (argvec[optind], "-mt") == 0)
      {
        mt = atoi(argvec[++optind]);
      }
      else if (strncmp (argvec[optind], "-", 1 ) == 0)
      {
	  fprintf(stderr, "Unknown option: %s\n", argvec[optind]);
	  exit (1);
      }
      else if (!slconn->sladdr)
      {
	  slconn->sladdr = argvec[optind];
      }

      else
      {
	  fprintf(stderr, "Unknown option: %s\n", argvec[optind]);
	  exit (1);
      }
  }

  /* Make sure a server was specified */
  if ( ! slconn->sladdr )
    {
      fprintf(stderr, "No SeedLink server specified\n\n");
      fprintf(stderr, "Usage: %s [options] [host][:port]\n", PACKAGE);
      fprintf(stderr, "Try '-h' for detailed help\n");
      exit (1);
    }

  /* Initialize the verbosity for the sl_log function */
  sl_loginit (verbose, NULL, NULL, NULL, NULL);


  /* Open dumpfile if requested */
  if (dumpfile)
    {
      if (!strcmp (dumpfile, "-"))
	{
	  /* Re-direct all messages to standard error */
//	  sl_loginit (verbose, &print_stderr, NULL, &print_stderr, NULL);

	  outfile = stdout;
	  setvbuf (stdout, NULL, _IONBF, 0);
	}
      else if ((outfile = fopen (dumpfile, "a+")) != NULL)
	{
	  setvbuf (outfile, NULL, _IONBF, 0);
	}
      else
	{
	  sl_log (2, 0, "cannot open dumpfile: %s\n", dumpfile);
	  exit (1);
	}
    }


  
  /* Report the program version */
  sl_log (0, 1, "%s version: %s\n", PACKAGE, VERSION);
  
  /* If errors then report the usage message and quit */
  if (error)
    {
      usage ();
      exit (1);
    }

  /* If verbosity is 2 or greater print detailed packet infor */
  if ( verbose >= 2 )
    ppackets = 1;

  /* Load the stream list from a file if specified */
  if ( streamfile )
  {
    sprintf(streamsfull,"%s/par/%s",topdir_rt,streamfile);
    streamfile = streamsfull;
    printf("streamfile1: %s\n",streamfile);
    sl_read_streamlist (slconn, streamfile, selectors);
  }else{
    sprintf(streamsfull,"%s/par/streams.conf",topdir_rt);
    streamfile = streamsfull;
    printf("streamfile2: %s\n",streamfile);
    sl_read_streamlist (slconn, streamfile, selectors);
  }

  /* Load the station list from a file if specified */
  if ( stationfile )
  {
    tu_read_stationlist (stationfile);
  }else{
    stationfile = "stations.conf";
    tu_read_stationlist (stationfile);
  }
  if(only_one == 1)
  {
    sprintf(tm,"%s",toplot[stno]);
    printf("tm: %s\n",tm);
    sprintf(toplot[0],"%s",tm);
    no_to_plot = 1;
  }
  
  /* Parse the 'multiselect' string following '-S' */
  if ( multiselect )
    {
      if ( sl_parse_streamlist (slconn, multiselect, selectors) == -1 )
	return -1;
    }
  else if ( !streamfile )
    {			 /* No 'streams' array, assuming uni-station mode */
      sl_setuniparams (slconn, selectors, -1, 0);
    }
  
  /* Attempt to recover sequence numbers from state file */
  if (statefile)
    {
      if (sl_recoverstate (slconn, statefile) < 0)
	{
	  sl_log (2, 0, "state recovery failed\n");
	}
    }
  
  return 0;
}				/* End of parameter_proc() */

/***************************************************************************
 * getoptval:
 * Return the value to a command line option; checking that the value is 
 * itself not an option (starting with '-') and is not past the end of
 * the argument list.
 *
 * argcount: total arguments in argvec
 * argvec: argument list
 * argopt: index of option to process, value is expected to be at argopt+1
 *
 * Returns value on success and exits with error message on failure
 ***************************************************************************/
static char *
getoptval (int argcount, char **argvec, int argopt)
{
  if ( argvec == NULL || argvec[argopt] == NULL ) {
    fprintf (stderr, "getoptval(): NULL option requested\n");
    exit (1);
  }
  
  /* Special case of '-o -' usage */
  if ( (argopt+1) < argcount && strcmp (argvec[argopt], "-o") == 0 )
    if ( strcmp (argvec[argopt+1], "-") == 0 )
      return argvec[argopt+1];
  
  if ( (argopt+1) < argcount && *argvec[argopt+1] != '-' )
    return argvec[argopt+1];

  fprintf (stderr, "Option %s requires a value\n", argvec[argopt]);
  exit (1);
}  /* End of getoptval() */

/***************************************************************************
 * usage:
 * Print the usage message and exit.
 ***************************************************************************/
static void
usage (void)
{


  fprintf (stderr, "\nCommand: %s [options] [host][:port]\n\n", PACKAGE);
  fprintf (stderr,
	   " Options:\n"
	   " -h              show this usage message\n"
	   " -to_wi          total width of plot in pixels\n"
	   " -fr_hg          total height of plot in pixels\n"
	   " -logo1          name of left-side logo (gif file)\n"
           " -logo2          name of right-side logo (jpg file)\n" 
           " -heli           0-One helicorder plot showing last 24 hours.\n"
	   "                 1-One helicorder plot per day.\n"
	   "                 (default 1). See manual for how to use.\n"
           " -mt             minutes across frame\n"
           " -col            color scheme\n"

           "\n"
           "\n"
           "\n"

	   " ## Data stream  selection and station file##\n"
	   " -l listfile     read a stream list from this file  (default: streams.conf)\n"
	   " -f stationfile  read a station list from this file (default: stations.conf)\n"
	   "\n"
	   " [host][:port]   Address of the SeedLink server in host:port format\n"
	   "                   f.ex.: 129.177.xx.yy:18000\n"
	   "                   f.ex.: localhost:18000\n\n");
  
}				/* End of usage() */

#ifndef WIN32
/***************************************************************************
 * term_handler:
 * Signal handler routine. 
 ***************************************************************************/
static void
term_handler (int sig)
{
  sl_terminate (slconn);
}
#endif





/*  Subroutine to apply an iir filter to a data sequence.                   */
/*    The filter is assumed to be stored as second order sections.          */
/*    Filtering is in-place.                                                */
/*    Zero-phase (forward and reverse) is an option.                        */

//appl(sw,chan,nsamp,peker)
void appl(int fnd)
//int      sw;
//int      chan;
//int      nsamp;
//float    *peker;
{
//float    *tpek;
    /* System generated locals */
int      i1;
int      i2;

    /* Local variables */
int      jptr;
int      i;
int      j;
float    b0;
float    b1;
float    b2;
float    a1;
float    a2;
float    x1;
float    x2;
float    y1;
float    y2;
float    output;
//tpek=peker;

/* Parameter adjustments                                                    */
/*    --data;*/

/* Function Body                                                            */
jptr = 1;
i1 = cset[fnd];

for (j = 1; j <= i1; ++j)
{
    x1 = save_x1[fnd][j];
    x2 = save_x2[fnd][j];
    y1 = save_y1[fnd][j];
    y2 = save_y2[fnd][j];
    b0 = snc[fnd][jptr];
    b1 = snc[fnd][jptr + 1];
    b2 = snc[fnd][jptr + 2];
    a1 = sdc[fnd][jptr + 1];
    a2 = sdc[fnd][jptr + 2];
//    i2 = nsamp;
    i2 = ns[fnd];
//    peker=tpek;
    for (i = 0; i < i2; ++i)
    {
//      output = b0 * (*peker) + b1 * x1 + b2 * x2;
      output = b0 * fseismo[i] + b1 * x1 + b2 * x2;

      output -= a1 * y1 + a2 * y2;
      y2 = y1;
      y1 = output;
      x2 = x1;
      x1 = fseismo[i];
      fseismo[i] = output;
//      peker++;
    }

    jptr += 3;

    save_y2[fnd][j]=y2;
    save_y1[fnd][j]=y1;
    save_x2[fnd][j]=x2;
    save_x1[fnd][j]=x1;

}

}
/*  Subroutine to design IIR digital filters from analog prototypes. */
/*  Input Arguments: */
/*  ---------------- */
/*    IORD                Filter order (10 MAXIMUM) */
/*    TYPE                Character*2 variable containing filter type */
/*                          LOWPASS (LP) */
/*                          HIGHPASS (HP) */
/*                          BANDPASS (BP) */
/*                          BANDREJECT (BR) */
/*   APROTO              Character*2 variable designating analog prototype*/
/*                          Butterworth (BU) */
/*                          Bessel (BE) */
/*                          Chebyshev Type I (C1) */
/*                          Chebyshev Type II (C2) */
/*    A                   Chebyshev stopband attenuation factor */
/*    TRBNDW              Chebyshev transition bandwidth (fraction of */
/*                          lowpass prototype passband width) */
/*    FL                  Low-frequency cutoff */
/*    FH                  High-frequency cutoff */
/*    TS                  Sampling interval (in seconds) */
/*  Output Arguments: */
/*  ----------------- */
/*    SN                  Array containing numerator coefficients of */
/*                        second-order sections packed head-to-tail. */
/*    SD                  Array containing denominator coefficients */
/*                        of second-order sections packed head-to-tail. */

void design( iord, type,aproto,  a,trbndw,  fl,  fh,  ts)
int iord;
char *type;
char *aproto;
float a;
float trbndw;
float fl;
float fh;
float ts;
{
    /* System generated locals */
    float r__1;

    /* Local variables */
    complex p[10], z[10];
    char stype[3*10];
    float omegar, ripple;
    float fhw, eps, flw, dcvalue;

    /*  Analog prototype selection */

    if (strncmp(aproto, "BU", 2) == 0) {
    buroots(p, stype, &dcvalue, iord);
    } else if (strncmp(aproto, "BE", 2) == 0) {
    beroots(p, stype, &dcvalue, iord);
    } else if (strncmp(aproto, "C1", 2) == 0) {
    chebparm(a, trbndw, iord, &eps, &ripple);
    c1roots(p, stype, &dcvalue, iord, &eps);
    } else if (strncmp(aproto, "C2", 2) == 0) {
    omegar = trbndw + (float)1.;
    c2roots(p, z, stype, &dcvalue, iord, a, omegar);
    }

    /*  Analog mapping selection */

    if (strncmp(type, "BP", 2) == 0) {
    r__1 = fl * ts / (float)2.;
    flw = warp(&r__1, &c_b12);
    r__1 = fh * ts / (float)2.;
    fhw = warp(&r__1, &c_b12);
    lptbp(p, z, stype, &dcvalue, &flw, &fhw);
    } else if (strncmp(type, "BR", 2) == 0) {
    r__1 = fl * ts / (float)2.;
    flw = warp(&r__1, &c_b12);
    r__1 = fh * ts / (float)2.;
    fhw = warp(&r__1, &c_b12);
    lptbr(p, z, stype, &dcvalue, &flw, &fhw, &sn[1], &sd[1]);
    } else if (strncmp(type, "LP", 2) == 0) {
    r__1 = fh * ts / (float)2.;
    fhw = warp(&r__1, &c_b12);
    lp(p, z, stype, &dcvalue, &sn[1], &sd[1]);
    cutoffs(&sn[1], &sd[1], &fhw);
    } else if (strncmp(type, "HP", 2) == 0) {
    r__1 = fl * ts / (float)2.;
    flw = warp(&r__1, &c_b12);
    lpthp(p, z, stype, &dcvalue, &sn[1], &sd[1]);
    cutoffs(&sn[1], &sd[1], &flw);
    }

    /*  Bilinear analog to digital transformation */
    bilin2(&sn[1], &sd[1]);

    return;
}

/* BUROOTS -- SUBROUTINE TO COMPUTE BUTTERWORTH POLES FOR */
/*   NORMALIZED LOWPASS FILTER */
/* LAST MODIFIED:  SEPTEMBER 7, 1990 */
/*  OUTPUT ARGUMENTS: */
/*  ----------------- */
/*      P              COMPLEX ARRAY CONTAINING POLES */
/*                       CONTAINS ONLY ONE FROM EACH */
/*                       COMPLEX CONJUGATE PAIR, AND */
/*                       ALL REAL POLES */
/*      RTYPE          CHARACTER ARRAY INDICATING 2ND ORDER SECTION */
/*                       TYPE: */
/*                         (SP)  SINGLE REAL POLE */
/*                         (CP)  COMPLEX CONJUGATE POLE PAIR */
/*                         (CPZ) COMPLEX CONJUGATE POLE-ZERO PAIRS */
/*      DCVALUE        MAGNITUDE OF FILTER AT ZERO FREQUENCY */
/*  INPUT ARGUMENTS: */
/*  ---------------- */
/*      IORD           DESIRED FILTER ORDER */
int buroots(p,rtype, dcvalue, iord)
complex *p;
char *rtype;
float *dcvalue;
int iord;
{
    /* System generated locals */
    int i__1, i__2;
    double d__1, d__2;
    complex q__1;

    /* Local variables */
    int half, k;
    float angle, pi;

    /* Parameter adjustments */
    rtype -= 3;
    --p;

    /* Function Body */
    pi = (float)3.14159265;

    half = iord / 2;

    /* TEST FOR ODD ORDER, AND ADD POLE AT -1 */

    nsects = 0;
    if (half << 1 < iord) {
    p[1].r = (float)-1., p[1].i = (float)0.;
    strncpy(rtype + 3, "SP", 2);
    nsects = 1;
    }
    i__1 = half;
    for (k = 1; k <= i__1; ++k) {
    angle = pi * ((float) ((k << 1) - 1)/(float) (iord << 1)+(float).5);
    ++(nsects);
    i__2 = nsects;
    d__1 = cos(angle);
    d__2 = sin(angle);
    q__1.r = d__1, q__1.i = d__2;
    p[i__2].r = q__1.r, p[i__2].i = q__1.i;
        strncpy(rtype + nsects * 3, "CP", 2);
    }
    *dcvalue = (float)1.;
    return TRUE;
}

/*  CHEBPARM - Calculates Chebyshev type I and II design parameters */
/*  INPUT ARGUMENTS */
/*  --------------- */
/*       A                Desired stopband attenuation */
/*                          i.e. max stopband amplitude is 1/ATTEN */
/*       TRBNDW           Transition bandwidth between stop and passbands */
/*                          as a fraction of the passband width */
/*       IORD             Filter order (number of poles) */
/*  OUTPUT ARGUMENTS */
/*  ---------------- */
/*       EPS              Chebyshev passband parameter */
/*       RIPPLE           Passband ripple */

static int chebparm(a, trbndw, iord, eps, ripple)
float a, trbndw;
int iord;
float *eps, *ripple;
{
    /* System generated locals */
    float r__1, r__2;

    /* Local variables */
    float g, alpha, omegar;

    omegar = trbndw + 1.;
    /* Computing 2nd power */
    r__2 = omegar;
    r__1 = omegar + sqrt(r__2 * r__2 - 1.);
    alpha = pow((double)r__1, (double)iord);
    /* Computing 2nd power */
    r__1 = alpha;
    g = (r__1 * r__1 + (float)1.) / (alpha * (float)2.);
    /* Computing 2nd power */
    r__1 = a;
    *eps = sqrt(r__1 * r__1 - (float)1.) / g;
    /* Computing 2nd power */
    r__1 = *eps;
    *ripple = (float)1. / sqrt(r__1 * r__1 + (float)1.);

    return TRUE;
}

/* C1ROOTS -- SUBROUTINE TO COMPUTE CHEBYSHEV TYPE I POLES FOR */
/*   NORMALIZED LOWPASS FILTER */
/* LAST MODIFIED:  SEPTEMBER 7, 1990 */
/*  OUTPUT ARGUMENTS: */
/*  ----------------- */
/*      P              COMPLEX ARRAY CONTAINING POLES */
/*                       CONTAINS ONLY ONE FROM EACH */
/*                       COMPLEX CONJUGATE PAIR, AND */
/*                       ALL REAL POLES */
/*      RTYPE          CHARACTER ARRAY INDICATING 2ND ORDER SECTION */
/*                       TYPE: */
/*                         (SP)  SINGLE REAL POLE */
/*                         (CP)  COMPLEX CONJUGATE POLE PAIR */
/*                         (CPZ) COMPLEX CONJUGATE POLE-ZERO PAIRS */
/*      DCVALUE        RESPONSE OF FILTER AT ZERO FREQUENCY */
/*  INPUT ARGUMENTS: */
/*  ---------------- */
/*      IORD           DESIRED FILTER ORDER */
/*      EPS            CHEBYSHEV PAR RELATED TO PASSBAND RIPPLE */

static int c1roots(p, rtype, dcvalue, iord, eps)
complex *p;
char *rtype;
float *dcvalue;
int iord;
float *eps;
{
    /* System generated locals */
    int i__1, i__2;
    float r__1;
    double d__1;
    complex q__1;

    /* Local variables */
    int half;
    float c;
    int i;
    float gamma, s, angle, omega, sigma, pi;

    /* Parameter adjustments */
    rtype -= 3;
    --p;

    /* Function Body */
    pi = (float)3.14159265;
    half = iord / 2;

    /*  INTERMEDIATE DESIGN PARS */

    gamma = (sqrt(*eps * *eps + (float)1.) + (float)1.) / *eps;
    gamma = log(gamma) / (float) (iord);
    gamma = exp(gamma);
    s = (gamma - (float)1. / gamma) * (float).5;
    c = (gamma + (float)1. / gamma) * (float).5;

    /*  CALCULATE POLES */

    nsects = 0;
    i__1 = half;
    for (i = 1; i <= i__1; ++i) {
    strncpy(rtype + i * 3, "CP", 2);
    angle = (float) ((i << 1) - 1) * pi / (float) (iord << 1);
    sigma = -(double)s * sin(angle);
    omega = c * cos(angle);
    i__2 = i;
    q__1.r = sigma, q__1.i = omega;
    p[i__2].r = q__1.r, p[i__2].i = q__1.i;
    ++(nsects);
    }
    if (half << 1 < iord) {
    strncpy(rtype + (half + 1) * 3, "SP", 2);
    i__1 = half + 1;
    d__1 = -(double)s;
    q__1.r = d__1, q__1.i = (float)0.;
    p[i__1].r = q__1.r, p[i__1].i = q__1.i;
    ++(nsects);
    *dcvalue = (float)1.;
    } else {
    /* Computing 2nd power */
    r__1 = *eps;
    *dcvalue = (float)1. / sqrt(r__1 * r__1 + 1);
    }
    return TRUE;
}

/* C2ROOTS -- SUBROUTINE TO COMPUTE ROOTS FOR NORMALIZED LOWPASS */
/*   CHEBYSHEV TYPE 2 FILTER */
/* LAST MODIFIED:  SEPTEMBER 7, 1990 */
/*  OUTPUT ARGUMENTS: */
/*  ----------------- */
/*      P              COMPLEX ARRAY CONTAINING POLES */
/*                       CONTAINS ONLY ONE FROM EACH */
/*                       COMPLEX CONJUGATE PAIR, AND */
/*                       ALL REAL POLES */
/*      Z              COMPLEX ARRAY CONTAINING ZEROS */
/*                       CONTAINS ONLY ONE FROM EACH */
/*                       COMPLEX CONJUGATE PAIR, AND */
/*                       ALL REAL ZEROS */
/*      RTYPE          CHARACTER ARRAY INDICATING 2ND ORDER SECTION */
/*                       TYPE: */
/*                         (SP)  SINGLE REAL POLE */
/*                         (CP)  COMPLEX CONJUGATE POLE PAIR */
/*                         (CPZ) COMPLEX CONJUGATE POLE-ZERO PAIRS */
/*      DCVALUE        MAGNITUDE OF FILTER AT ZERO FREQUENCY */
/*  INPUT ARGUMENTS: */
/*  ---------------- */
/*      IORD           DESIRED FILTER ORDER */
/*      A              STOPBAND ATTENUATION FACTOR */
/*      OMEGAR         CUTOFF FREQUENCY OF STOPBAND */
/*                     PASSBAND CUTOFF IS AT 1.0 HERTZ */

static int c2roots(p, z, rtype, dcvalue, iord, a, omegar)
complex *p, *z;
char *rtype;
float *dcvalue;
int iord;
float a, omegar;
{
    /* System generated locals */
    int i__1, i__2;
    double d__1;
    complex q__1;

    /* Local variables */
    int half;
    float beta, c;
    int i;
    float gamma, s, alpha, angle, omega, sigma, denom, pi;

    /* Parameter adjustments */
    rtype -= 3;
    --z;
    --p;

    /* Function Body */
    pi = (float)3.14159265;
    half = iord / 2;

/*  INTERMEDIATE DESIGN PARS */

    gamma = a + sqrt(a * a - (float)1.);
    gamma = log(gamma) / (float) (iord);
    gamma = exp(gamma);
    s = (gamma - (float)1. / gamma) * (float).5;
    c = (gamma + (float)1. / gamma) * (float).5;
    nsects = 0;
    i__1 = half;
    for (i = 1; i <= i__1; ++i) {

/*  CALCULATE POLES */

    strncpy(rtype + i * 3, "CPZ", 3);
    angle = (float) ((i << 1) - 1) * pi / (float) (iord << 1);
    alpha = -(double)s * sin(angle);
    beta = c * cos(angle);
    denom = alpha * alpha + beta * beta;
    sigma = omegar * alpha / denom;
    omega = -(double)(omegar) * beta / denom;
    i__2 = i;
    q__1.r = sigma, q__1.i = omega;
    p[i__2].r = q__1.r, p[i__2].i = q__1.i;

/*  CALCULATE ZEROS */

    omega = omegar / cos(angle);
    i__2 = i;
    q__1.r = (float)0., q__1.i = omega;
    z[i__2].r = q__1.r, z[i__2].i = q__1.i;
    ++(nsects);
    }

/*  ODD-ORDER FILTERS */

    if (half << 1 < iord) {
    strncpy(rtype + (half + 1) * 3, "SP", 2);
    i__1 = half + 1;
    d__1 = -(double)(omegar) / s;
    q__1.r = d__1, q__1.i = (float)0.;
    p[i__1].r = q__1.r, p[i__1].i = q__1.i;
    ++(nsects);
    }

/*  DC VALUE */

    *dcvalue = 1.;
    return TRUE;
}

/* Subroutine to convert an prototype lowpass filter to a bandpass filter via
*/
/*    the analog polynomial transformation.  The lowpass filter is */
/*   described in terms of its poles and zeros (as input to this routine).*/
/*    The output consists of the parameters for second order sections. */
/*  Input Arguments: */
/*  ---------------- */
/*    P                       Array containing poles */
/*    Z                       Array containing zeros */
/*    RTYPE                   Character array containing type information */
/*                              (SP) single real pole  or */
/*                              (CP) complex conjugate pole pair  or */
/*                              (CPZ) complex conjugate pole/zero pairs */
/*    DCVALUE                 Zero frequency value of filter */
/*    FL                      Low-frequency cutoff */
/*    FH                      High-frequency cutoff */
/*  Output Arguments: */
/*  ----------------- */
/*    SN                      Numerator polynomials for second order */
/*                              sections. */
/*    SD                      Denominator polynomials for second order */
/*                              sections. */
/*                              This subroutine doubles the number of */
/*                              sections. */

static int lptbp(p, z, rtype, dcvalue, fl, fh)
complex *p, *z;
char *rtype;
float *dcvalue;
float *fl, *fh;
{
    /* System generated locals */
    int i__1, i__2, i__3, i__4, i__5, i__6, i__7;
    double d__1;
    complex q__1, q__2, q__3, q__4, q__5, q__6, q__7, q__8, q__9, q__10;

    /* Local variables */
    int iptr;
    float a, b;
    complex h;
    int i, n;
    complex s;
    float scale;
    complex ctemp, p1, p2;
    float twopi;
    complex z1, z2;
    float pi;

    /* Parameter adjustments */
    rtype -= 3;
    --z;
    --p;

    /* Function Body */
    pi = (float)3.14159265;
    twopi = pi * (float)2.;
    a = twopi * twopi * *fl * *fh;
    b = twopi * (*fh - *fl);

    n = nsects;
    nsects = 0;
    iptr = 1;
    i__1 = n;
    for (i = 1; i <= i__1; ++i) {
    if (strncmp(rtype + i * 3, "CPZ", 3) == 0) {
        i__2 = i;
        q__3.r = b * z[i__2].r, q__3.i = b * z[i__2].i;
            q__2 = cpowi(q__3, 2);
        d__1 = a * (float)4.;
        q__1.r = q__2.r - d__1, q__1.i = q__2.i;
        ctemp.r = q__1.r, ctemp.i = q__1.i;
            q__1 = csqrt(ctemp);
        ctemp.r = q__1.r, ctemp.i = q__1.i;
        i__2 = i;
        q__3.r = b * z[i__2].r, q__3.i = b * z[i__2].i;
        q__2.r = q__3.r + ctemp.r, q__2.i = q__3.i + ctemp.i;
        q__1.r = q__2.r * (float).5, q__1.i = q__2.i * (float).5;
        z1.r = q__1.r, z1.i = q__1.i;
        i__2 = i;
        q__3.r = b * z[i__2].r, q__3.i = b * z[i__2].i;
        q__2.r = q__3.r - ctemp.r, q__2.i = q__3.i - ctemp.i;
        q__1.r = q__2.r * (float).5, q__1.i = q__2.i * (float).5;
        z2.r = q__1.r, z2.i = q__1.i;
        i__2 = i;
        q__3.r = b * p[i__2].r, q__3.i = b * p[i__2].i;
            q__2 = cpowi(q__3, 2);
        d__1 = a * (float)4.;
        q__1.r = q__2.r - d__1, q__1.i = q__2.i;
        ctemp.r = q__1.r, ctemp.i = q__1.i;
            q__1 = csqrt(ctemp);
        ctemp.r = q__1.r, ctemp.i = q__1.i;
        i__2 = i;
        q__3.r = b * p[i__2].r, q__3.i = b * p[i__2].i;
        q__2.r = q__3.r + ctemp.r, q__2.i = q__3.i + ctemp.i;
        q__1.r = q__2.r * (float).5, q__1.i = q__2.i * (float).5;
        p1.r = q__1.r, p1.i = q__1.i;
        i__2 = i;
        q__3.r = b * p[i__2].r, q__3.i = b * p[i__2].i;
        q__2.r = q__3.r - ctemp.r, q__2.i = q__3.i - ctemp.i;
        q__1.r = q__2.r * (float).5, q__1.i = q__2.i * (float).5;
        p2.r = q__1.r, p2.i = q__1.i;
            q__2 = conjg(z1);
        q__1.r = z1.r * q__2.r - z1.i * q__2.i, q__1.i = z1.r * q__2.i +
            z1.i * q__2.r;
        sn[iptr] = q__1.r;
        sn[iptr + 1] = z1.r * (float)-2.;
        sn[iptr + 2] = (float)1.;
            q__2 = conjg(p1);
        q__1.r = p1.r * q__2.r - p1.i * q__2.i, q__1.i = p1.r * q__2.i +
            p1.i * q__2.r;
        sd[iptr] = q__1.r;
        sd[iptr + 1] = p1.r * (float)-2.;
        sd[iptr + 2] = (float)1.;
        iptr += 3;
            q__2 = conjg(z2);
        q__1.r = z2.r * q__2.r - z2.i * q__2.i, q__1.i = z2.r * q__2.i +
            z2.i * q__2.r;
        sn[iptr] = q__1.r;
        sn[iptr + 1] = z2.r * (float)-2.;
        sn[iptr + 2] = (float)1.;
            q__2 = conjg(p2);
        q__1.r = p2.r * q__2.r - p2.i * q__2.i, q__1.i = p2.r * q__2.i +
            p2.i * q__2.r;
        sd[iptr] = q__1.r;
        sd[iptr + 1] = p2.r * (float)-2.;
        sd[iptr + 2] = (float)1.;
        iptr += 3;
        nsects += 2;
    } else if (strncmp(rtype + i * 3, "CP", 2) == 0) {
        i__2 = i;
        q__3.r = b * p[i__2].r, q__3.i = b * p[i__2].i;
            q__2 = cpowi(q__3, 2);
        d__1 = a * (float)4.;
        q__1.r = q__2.r - d__1, q__1.i = q__2.i;
        ctemp.r = q__1.r, ctemp.i = q__1.i;
            q__1 = csqrt(ctemp);
        ctemp.r = q__1.r, ctemp.i = q__1.i;
        i__2 = i;
        q__3.r = b * p[i__2].r, q__3.i = b * p[i__2].i;
        q__2.r = q__3.r + ctemp.r, q__2.i = q__3.i + ctemp.i;
        q__1.r = q__2.r * (float).5, q__1.i = q__2.i * (float).5;
        p1.r = q__1.r, p1.i = q__1.i;
        i__2 = i;
        q__3.r = b * p[i__2].r, q__3.i = b * p[i__2].i;
        q__2.r = q__3.r - ctemp.r, q__2.i = q__3.i - ctemp.i;
        q__1.r = q__2.r * (float).5, q__1.i = q__2.i * (float).5;
        p2.r = q__1.r, p2.i = q__1.i;
        sn[iptr] = (float)0.;
        sn[iptr + 1] = b;
        sn[iptr + 2] = (float)0.;
            q__2 = conjg(p1);
        q__1.r = p1.r * q__2.r - p1.i * q__2.i, q__1.i = p1.r * q__2.i +
            p1.i * q__2.r;
        sd[iptr] = q__1.r;
        sd[iptr + 1] = p1.r * (float)-2.;
        sd[iptr + 2] = (float)1.;
        iptr += 3;
        sn[iptr] = (float)0.;
        sn[iptr + 1] = b;
        sn[iptr + 2] = (float)0.;
            q__2 = conjg(p2);
        q__1.r = p2.r * q__2.r - p2.i * q__2.i, q__1.i = p2.r * q__2.i +
            p2.i * q__2.r;
        sd[iptr] = q__1.r;
        sd[iptr + 1] = p2.r * (float)-2.;
        sd[iptr + 2] = (float)1.;
        iptr += 3;
        nsects += 2;
    } else if (strncmp(rtype + i * 3, "SP", 2) == 0) {
        sn[iptr] = (float)0.;
        sn[iptr + 1] = b;
        sn[iptr + 2] = (float)0.;
        sd[iptr] = a;
        i__2 = i;
        sd[iptr + 1] = -(double)b * p[i__2].r;
        sd[iptr + 2] = (float)1.;
        iptr += 3;
        ++(nsects);
    }
    }
/* Scaling - use fact that the bandpass filter amplitude at sqrt( omega_l **/
/*            equals the amplitude of the lowpass prototype at d.c. */
    d__1 = sqrt(a);
    q__1.r = (float)0., q__1.i = d__1;
    s.r = q__1.r, s.i = q__1.i;
    h.r = (float)1., h.i = (float)0.;

    iptr = 1;
    i__1 = nsects;
    for (i = 1; i <= i__1; ++i) {
    i__2 = iptr + 2;
    q__6.r = sn[i__2] * s.r, q__6.i = sn[i__2] * s.i;
    i__3 = iptr + 1;
    q__5.r = q__6.r + sn[i__3], q__5.i = q__6.i;
    q__4.r = q__5.r * s.r - q__5.i * s.i, q__4.i = q__5.r * s.i + q__5.i *
         s.r;
    i__4 = iptr;
    q__3.r = q__4.r + sn[i__4], q__3.i = q__4.i;
    q__2.r = h.r * q__3.r - h.i * q__3.i, q__2.i = h.r * q__3.i + h.i *
        q__3.r;
    i__5 = iptr + 2;
    q__10.r = sd[i__5] * s.r, q__10.i = sd[i__5] * s.i;
    i__6 = iptr + 1;
    q__9.r = q__10.r + sd[i__6], q__9.i = q__10.i;
    q__8.r = q__9.r * s.r - q__9.i * s.i, q__8.i = q__9.r * s.i + q__9.i *
         s.r;
    i__7 = iptr;
    q__7.r = q__8.r + sd[i__7], q__7.i = q__8.i;
        q__1 = cdiv(q__2, q__7);
    h.r = q__1.r, h.i = q__1.i;
    iptr += 3;
    }
    q__2.r = *dcvalue, q__2.i = (float)0.;
    d__1 = h.r;
    q__5 = conjg(h);
    q__4.r = d__1 * q__5.r, q__4.i = d__1 * q__5.i;
    q__3 = csqrt(q__4);
    q__1 = cdiv(q__2, q__3);
    scale = q__1.r;
    sn[1] *= scale;
    sn[2] *= scale;
    sn[3] *= scale;
    return TRUE;
}

/*  Subroutine to generate second order section parameterization */
/*    from an pole-zero description for lowpass filters. */
/*  Input Arguments: */
/*  ---------------- */
/*    P                       Array containing poles */
/*    Z                       Array containing zeros */
/*   RTYPE                   Character array containing root type information
*/
/*                              (SP)  Single pole or */
/*                              (CP)  Complex conjugate pole pair */
/*                             (CPZ) Complex conjugate pole and zero pairs*/
/*    DCVALUE                 Zero-frequency value of prototype filter */
/*  Output Arguments: */
/*  ----------------- */
/*    SN                      Numerator polynomials for second order */
/*                              sections. */
/*    SD                      Denominator polynomials for second order */
/*                              sections. */

static int lp(p, z, rtype, dcvalue, sn, sd)
complex *p, *z;
char *rtype;
float *dcvalue;
float *sn, *sd;
{
    /* System generated locals */
    int i__1, i__2, i__3;
    complex q__1, q__2, q__3, q__4;

    /* Local variables */
    int iptr, i;
    float scale;

    /* Parameter adjustments */
    --sd;
    --sn;
    rtype -= 3;
    --z;
    --p;

    /* Function Body */
    iptr = 1;
    i__1 = nsects;
    for (i = 1; i <= i__1; ++i) {
    if (strncmp(rtype + i * 3, "CPZ", 3) == 0) {
        i__2 = i;
        q__2 = conjg(p[i]);
        q__1.r = p[i__2].r * q__2.r - p[i__2].i * q__2.i, q__1.i = p[i__2]
            .r * q__2.i + p[i__2].i * q__2.r;
        i__3 = i;
        q__4 = conjg(z[i]);
        q__3.r = z[i__3].r * q__4.r - z[i__3].i * q__4.i, q__3.i = z[i__3]
            .r * q__4.i + z[i__3].i * q__4.r;
        scale = q__1.r / q__3.r;
        i__2 = i;
        q__2 = conjg(z[i]);
        q__1.r = z[i__2].r * q__2.r - z[i__2].i * q__2.i, q__1.i = z[i__2]
            .r * q__2.i + z[i__2].i * q__2.r;
        sn[iptr] = q__1.r * scale;
        i__2 = i;
        sn[iptr + 1] = z[i__2].r * (float)-2. * scale;
        sn[iptr + 2] = scale * (float)1.;
        i__2 = i;
        q__2 = conjg(p[i]);
        q__1.r = p[i__2].r * q__2.r - p[i__2].i * q__2.i, q__1.i = p[i__2]
            .r * q__2.i + p[i__2].i * q__2.r;
        sd[iptr] = q__1.r;
        i__2 = i;
        sd[iptr + 1] = p[i__2].r * (float)-2.;
        sd[iptr + 2] = (float)1.;
        iptr += 3;
    } else if (strncmp(rtype + i * 3, "CP", 2) == 0) {
        i__2 = i;
        q__2 = conjg(p[i]);
        q__1.r = p[i__2].r * q__2.r - p[i__2].i * q__2.i, q__1.i = p[i__2]
            .r * q__2.i + p[i__2].i * q__2.r;
        scale = q__1.r;
        sn[iptr] = scale;
        sn[iptr + 1] = (float)0.;
        sn[iptr + 2] = (float)0.;
        i__2 = i;
        q__2 = conjg(p[i]);
        q__1.r = p[i__2].r * q__2.r - p[i__2].i * q__2.i, q__1.i = p[i__2]
            .r * q__2.i + p[i__2].i * q__2.r;
        sd[iptr] = q__1.r;
        i__2 = i;
        sd[iptr + 1] = p[i__2].r * (float)-2.;
        sd[iptr + 2] = (float)1.;
        iptr += 3;
    } else if (strncmp(rtype + i * 3, "SP", 2) == 0) {
        i__2 = i;
        scale = -(double)p[i__2].r;
        sn[iptr] = scale;
        sn[iptr + 1] = (float)0.;
        sn[iptr + 2] = (float)0.;
        i__2 = i;
        sd[iptr] = -(double)p[i__2].r;
        sd[iptr + 1] = (float)1.;
        sd[iptr + 2] = (float)0.;
        iptr += 3;
    }
    }
    sn[1] = *dcvalue * sn[1];
    sn[2] = *dcvalue * sn[2];
    sn[3] = *dcvalue * sn[3];
    return TRUE;
}

/*  Subroutine to convert a lowpass filter to a band reject filter */
/*    via an analog polynomial transformation.  The lowpass filter is */
/*   described in terms of its poles and zeros (as input to this routine).*/
/*    The output consists of the parameters for second order sections. */
/*  Input Arguments: */
/*  ---------------- */
/*    P                       Array containing poles */
/*    Z                       Array containing zeros */
/*    RTYPE                   Character array containing type information */
/*                              (SP)  single real pole or */
/*                              (CP)  complex conjugate pole pair */
/*                              (CPZ) complex conjugate pole/zero pairs */
/*    DCVALUE                 Zero-frequency value of prototype filter */
/*                              prior to transformation */
/*    FL                      Low-frequency cutoff */
/*    FH                      High-frequency cutoff */
/*  Output Arguments: */
/*  ----------------- */
/*    SN                      Numerator polynomials for second order */
/*                              sections. */
/*    SD                      Denominator polynomials for second order */
/*                              sections. */
/*    NSECTS                  Number of second order sections following */
/*                              transformation.  The number is doubled. */

static int lptbr(p, z, rtype, dcvalue, fl, fh, sn, sd)
complex *p, *z;
char *rtype;
float *dcvalue;
float *fl, *fh, *sn, *sd;
{
    /* System generated locals */
    int i__1, i__2;
    double d__1;
    complex q__1, q__2, q__3;

    /* Local variables */
    complex cinv;
    int iptr;
    float a, b, h;
    int i, n;
    float scale;
    complex ctemp, p1, p2;
    float twopi;
    complex z1, z2;
    float pi;

    /* Parameter adjustments */
    --sd;
    --sn;
    rtype -= 3;
    --z;
    --p;

    /* Function Body */
    pi = (float)3.14159265;
    twopi = pi * (float)2.;
    a = twopi * twopi * *fl * *fh;
    b = twopi * (*fh - *fl);
    n = nsects;
    nsects = 0;
    iptr = 1;
    i__1 = n;
    for (i = 1; i <= i__1; ++i) {
    if (strncmp(rtype + i * 3, "CPZ", 3) == 0) {
        q__1 = cdiv(c_b43, z[i]);
        cinv.r = q__1.r, cinv.i = q__1.i;
        q__3.r = b * cinv.r, q__3.i = b * cinv.i;
            q__2 = cpowi(q__3, 2);
        d__1 = a * (float)4.;
        q__1.r = q__2.r - d__1, q__1.i = q__2.i;
        ctemp.r = q__1.r, ctemp.i = q__1.i;
        q__1 = csqrt(ctemp);
        ctemp.r = q__1.r, ctemp.i = q__1.i;
        q__3.r = b * cinv.r, q__3.i = b * cinv.i;
        q__2.r = q__3.r + ctemp.r, q__2.i = q__3.i + ctemp.i;
        q__1.r = q__2.r * (float).5, q__1.i = q__2.i * (float).5;
        z1.r = q__1.r, z1.i = q__1.i;
        q__3.r = b * cinv.r, q__3.i = b * cinv.i;
        q__2.r = q__3.r - ctemp.r, q__2.i = q__3.i - ctemp.i;
        q__1.r = q__2.r * (float).5, q__1.i = q__2.i * (float).5;
        z2.r = q__1.r, z2.i = q__1.i;
        q__1 = cdiv(c_b43, p[i]);
        cinv.r = q__1.r, cinv.i = q__1.i;
        q__3.r = b * cinv.r, q__3.i = b * cinv.i;
            q__2 = cpowi(q__3, 2);
        d__1 = a * (float)4.;
        q__1.r = q__2.r - d__1, q__1.i = q__2.i;
        ctemp.r = q__1.r, ctemp.i = q__1.i;
        q__1 = csqrt(ctemp);
        ctemp.r = q__1.r, ctemp.i = q__1.i;
        q__3.r = b * cinv.r, q__3.i = b * cinv.i;
        q__2.r = q__3.r + ctemp.r, q__2.i = q__3.i + ctemp.i;
        q__1.r = q__2.r * (float).5, q__1.i = q__2.i * (float).5;
        p1.r = q__1.r, p1.i = q__1.i;
        q__3.r = b * cinv.r, q__3.i = b * cinv.i;
        q__2.r = q__3.r - ctemp.r, q__2.i = q__3.i - ctemp.i;
        q__1.r = q__2.r * (float).5, q__1.i = q__2.i * (float).5;
        p2.r = q__1.r, p2.i = q__1.i;
        q__2 = conjg(z1);
        q__1.r = z1.r * q__2.r - z1.i * q__2.i, q__1.i = z1.r * q__2.i +
            z1.i * q__2.r;
        sn[iptr] = q__1.r;
        sn[iptr + 1] = z1.r * (float)-2.;
        sn[iptr + 2] = (float)1.;
        q__2 = conjg(p1);
        q__1.r = p1.r * q__2.r - p1.i * q__2.i, q__1.i = p1.r * q__2.i +
            p1.i * q__2.r;
        sd[iptr] = q__1.r;
        sd[iptr + 1] = p1.r * (float)-2.;
        sd[iptr + 2] = (float)1.;
        iptr += 3;
        q__2 = conjg(z2);
        q__1.r = z2.r * q__2.r - z2.i * q__2.i, q__1.i = z2.r * q__2.i +
            z2.i * q__2.r;
        sn[iptr] = q__1.r;
        sn[iptr + 1] = z2.r * (float)-2.;
        sn[iptr + 2] = (float)1.;
        q__2 = conjg(p2);
        q__1.r = p2.r * q__2.r - p2.i * q__2.i, q__1.i = p2.r * q__2.i +
            p2.i * q__2.r;
        sd[iptr] = q__1.r;
        sd[iptr + 1] = p2.r * (float)-2.;
        sd[iptr + 2] = (float)1.;
        iptr += 3;
        nsects += 2;
    } else if (strncmp(rtype + i * 3, "CP", 2) == 0) {
        q__1 = cdiv(c_b43, p[i]);
        cinv.r = q__1.r, cinv.i = q__1.i;
        q__3.r = b * cinv.r, q__3.i = b * cinv.i;
            q__2 = cpowi(q__3, 2);
        d__1 = a * (float)4.;
        q__1.r = q__2.r - d__1, q__1.i = q__2.i;
        ctemp.r = q__1.r, ctemp.i = q__1.i;
        q__1 = csqrt(ctemp);
        ctemp.r = q__1.r, ctemp.i = q__1.i;
        q__3.r = b * cinv.r, q__3.i = b * cinv.i;
        q__2.r = q__3.r + ctemp.r, q__2.i = q__3.i + ctemp.i;
        q__1.r = q__2.r * (float).5, q__1.i = q__2.i * (float).5;
        p1.r = q__1.r, p1.i = q__1.i;
        q__3.r = b * cinv.r, q__3.i = b * cinv.i;
        q__2.r = q__3.r - ctemp.r, q__2.i = q__3.i - ctemp.i;
        q__1.r = q__2.r * (float).5, q__1.i = q__2.i * (float).5;
        p2.r = q__1.r, p2.i = q__1.i;
        sn[iptr] = a;
        sn[iptr + 1] = (float)0.;
        sn[iptr + 2] = (float)1.;
        q__2 = conjg(p1);
        q__1.r = p1.r * q__2.r - p1.i * q__2.i, q__1.i = p1.r * q__2.i +
            p1.i * q__2.r;
        sd[iptr] = q__1.r;
        sd[iptr + 1] = p1.r * (float)-2.;
        sd[iptr + 2] = (float)1.;
        iptr += 3;
        sn[iptr] = a;
        sn[iptr + 1] = (float)0.;
        sn[iptr + 2] = (float)1.;
        q__2 = conjg(p2);
        q__1.r = p2.r * q__2.r - p2.i * q__2.i, q__1.i = p2.r * q__2.i +
            p2.i * q__2.r;
        sd[iptr] = q__1.r;
        sd[iptr + 1] = p2.r * (float)-2.;
        sd[iptr + 2] = (float)1.;
        iptr += 3;
        nsects += 2;
    } else if (strncmp(rtype + i * 3, "SP", 2) == 0) {
        sn[iptr] = a;
        sn[iptr + 1] = (float)0.;
        sn[iptr + 2] = (float)1.;
        i__2 = i;
        sd[iptr] = -(double)a * p[i__2].r;
        sd[iptr + 1] = b;
        i__2 = i;
        sd[iptr + 2] = -(double)p[i__2].r;
        iptr += 3;
        ++(nsects);
    }
    }
/*  Scaling - use the fact that the bandreject filter amplitude  at d.c. */
/*            equals the lowpass prototype amplitude at d.c. */
    h = (float)1.;
    iptr = 1;
    i__1 = nsects;
    for (i = 1; i <= i__1; ++i) {
    h = h * sn[iptr] / sd[iptr];
    iptr += 3;
    }
    scale = *dcvalue / ABS(h);
    sn[1] *= scale;
    sn[2] *= scale;
    sn[3] *= scale;
    return TRUE;
}

/*  Subroutine to alter the cutoff of a filter.  Assumes that the */
/*    filter is structured as second order sections.  Changes */
/*    the cutoffs of normalized lowpass and highpass filters through */
/*    a simple polynomial transformation. */
/*  Input Arguments: */
/*  ---------------- */
/*    F                       New cutoff frequency */
/*  Input/Output Arguments: */
/*  ----------------------- */
/*    SN                      Numerator polynomials for second order */
/*                              sections. */
/*    SD                      Denominator polynomials for second order */
/*                              sections. */

static int cutoffs(sn, sd, f)
float *sn, *sd;
float *f;
{
    /* System generated locals */
    int i__1;

    /* Local variables */
    int iptr, i;
    float scale;

    /* Parameter adjustments */
    --sd;
    --sn;

    /* Function Body */
    scale = *f * (float)6.2831853000000004;

    iptr = 1;
    i__1 = nsects;
    for (i = 1; i <= i__1; ++i) {

    sn[iptr + 1] /= scale;
    sn[iptr + 2] /= scale * scale;
    sd[iptr + 1] /= scale;
    sd[iptr + 2] /= scale * scale;
    iptr += 3;
    }
    return TRUE;
}

/*  Subroutine to convert a lowpass filter to a highpass filter via */
/*    an analog polynomial transformation.  The lowpass filter is */
/*   described in terms of its poles and zeroes (as input to this routine).*/
/*    The output consists of the parameters for second order sections. */
/*  Input Arguments: */
/*  ---------------- */
/*    P                       Array containing poles */
/*    Z                       Array containing zeroes */
/*   RTYPE                   Character array containing root type information
*/
/*                              (SP) single real pole or */
/*                              (CP)  complex conjugate pair */
/*                              (CPZ) complex pole/zero pairs */
/*    DCVALUE                 Zero-frequency value of prototype filter */
/*  Output Arguments: */
/*  ----------------- */
/*    SN                      Numerator polynomials for second order */
/*                              sections. */
/*    SD                      Denominator polynomials for second order */
/*                              sections. */

static int lpthp(p, z, rtype, dcvalue, sn, sd)
complex *p, *z;
char *rtype;
float *dcvalue;
float *sn, *sd;
{
    /* System generated locals */
    int i__1, i__2, i__3;
    complex q__1, q__2, q__3, q__4;

   /* Local variables */
    int iptr, i;
    float scale;

    /* Parameter adjustments */
    --sd;
    --sn;
    rtype -= 3;
    --z;
    --p;

    /* Function Body */
    iptr = 1;
    i__1 = nsects;
    for (i = 1; i <= i__1; ++i) {
    if (strncmp(rtype + i * 3, "CPZ", 3) == 0) {
        i__2 = i;
        q__2 = conjg(p[i]);
        q__1.r = p[i__2].r * q__2.r - p[i__2].i * q__2.i, q__1.i = p[i__2]
            .r * q__2.i + p[i__2].i * q__2.r;
        i__3 = i;
        q__4 = conjg(z[i]);
        q__3.r = z[i__3].r * q__4.r - z[i__3].i * q__4.i, q__3.i = z[i__3]
            .r * q__4.i + z[i__3].i * q__4.r;
        scale = q__1.r / q__3.r;
        sn[iptr] = scale * (float)1.;
        i__2 = i;
        sn[iptr + 1] = z[i__2].r * (float)-2. * scale;
        i__2 = i;
        q__2 = conjg(z[i]);
        q__1.r = z[i__2].r * q__2.r - z[i__2].i * q__2.i, q__1.i = z[i__2]
            .r * q__2.i + z[i__2].i * q__2.r;
        sn[iptr + 2] = q__1.r * scale;
        sd[iptr] = (float)1.;
        i__2 = i;
        sd[iptr + 1] = p[i__2].r * (float)-2.;
        i__2 = i;
        q__2 = conjg(p[i]);
        q__1.r = p[i__2].r * q__2.r - p[i__2].i * q__2.i, q__1.i = p[i__2]
            .r * q__2.i + p[i__2].i * q__2.r;
        sd[iptr + 2] = q__1.r;
        iptr += 3;
    } else if (strncmp(rtype + i * 3, "CP", 2) == 0) {
        i__2 = i;
        q__2 = conjg(p[i]);
        q__1.r = p[i__2].r * q__2.r - p[i__2].i * q__2.i, q__1.i = p[i__2]
            .r * q__2.i + p[i__2].i * q__2.r;
        scale = q__1.r;
        sn[iptr] = (float)0.;
        sn[iptr + 1] = (float)0.;
        sn[iptr + 2] = scale;
        sd[iptr] = (float)1.;
        i__2 = i;
        sd[iptr + 1] = p[i__2].r * (float)-2.;
        i__2 = i;
        q__2 = conjg(p[i]);
        q__1.r = p[i__2].r * q__2.r - p[i__2].i * q__2.i, q__1.i = p[i__2]
            .r * q__2.i + p[i__2].i * q__2.r;
        sd[iptr + 2] = q__1.r;
        iptr += 3;
    } else if (strncmp(rtype + i * 3, "SP", 2) == 0) {
        i__2 = i;
        scale = -(double)p[i__2].r;
        sn[iptr] = (float)0.;
        sn[iptr + 1] = scale;
        sn[iptr + 2] = (float)0.;
        sd[iptr] = (float)1.;
        i__2 = i;
        sd[iptr + 1] = -(double)p[i__2].r;
        sd[iptr + 2] = (float)0.;
        iptr += 3;
    }
    }
    sn[1] *= *dcvalue;
    sn[2] *= *dcvalue;
    sn[3] *= *dcvalue;
    return TRUE;
}

/* Transforms an analog filter to a digital filter via the bilinear transforma
ti*/
/*   Assumes both are stored as second order sections.  The transformation is
*/
/*    done in-place. */
/*  Input Arguments: */
/*  ---------------- */
/*   SN                   Array containing numerator polynomial coefficients f
or*/
/*                           second order sections.  Packed head-to-tail. */
/*   SD                   Array containing denominator polynomial coefficients
 f*/
/*                           second order sections.  Packed head-to-tail. */

static int bilin2(sn, sd)
float *sn, *sd;
{
    /* System generated locals */
    int i__1;

    /* Local variables */
    int iptr, i;
    float scale, a0, a1, a2;

    /* Parameter adjustments */
    --sd;
    --sn;

    /* Function Body */
    iptr = 1;
    i__1 = nsects;
    for (i = 1; i <= i__1; ++i) {
    a0 = sd[iptr];
    a1 = sd[iptr + 1];
    a2 = sd[iptr + 2];
    scale = a2 + a1 + a0;
    sd[iptr] = (float)1.;
    sd[iptr + 1] = (a0 - a2) * (float)2. / scale;
    sd[iptr + 2] = (a2 - a1 + a0) / scale;
    a0 = sn[iptr];
    a1 = sn[iptr + 1];
    a2 = sn[iptr + 2];
    sn[iptr] = (a2 + a1 + a0) / scale;
    sn[iptr + 1] = (a0 - a2) * (float)2. / scale;
    sn[iptr + 2] = (a2 - a1 + a0) / scale;
    iptr += 3;
    }
    return TRUE;
}

/* WARP -- FUNCTION, APPLIES TANGENT FREQUENCY WARPING TO COMPENSATE */
/*         FOR BILINEAR ANALOG -> DIGITAL TRANSFORMATION */
/* ARGUMENTS: */
/* ---------- */
/*      F       ORIGINAL DESIGN FREQUENCY SPECIFICATION (HERTZ) */
/*      TS      SAMPLING INTERVAL (SECONDS) */
/*  LAST MODIFIED:  SEPTEMBER 20, 1990 */

static double warp(f, ts)
float *f, *ts;
{
    /* System generated locals */
    float ret_val;

    /* Local variables */
    float angle, twopi;

    twopi = (float)6.2831853;
    angle = twopi * *f * *ts / (float)2.;
    ret_val = tan(angle) * (float)2. / *ts;
    ret_val /= twopi;
    return ret_val;
}

/* BEROOTS -- SUBROUTINE TO RETURN BESSEL POLES FOR */
/*   NORMALIZED LOWPASS FILTER */
/* LAST MODIFIED:  SEPTEMBER 7, 1990 */
/*  OUTPUT ARGUMENTS: */
/*  ----------------- */
/*      P              COMPLEX ARRAY CONTAINING POLES */
/*                       CONTAINS ONLY ONE FROM EACH */
/*                       COMPLEX CONJUGATE PAIR, AND */
/*                       ALL REAL POLES */
/*      RTYPE          CHARACTER ARRAY INDICATING 2ND ORDER SECTION */
/*                       TYPE: */
/*                         (SP)  SINGLE REAL POLE */
/*                         (CP)  COMPLEX CONJUGATE POLE PAIR */
/*                         (CPZ) COMPLEX CONJUGATE POLE-ZERO PAIRS */
/*      DCVALUE        MAGNITUDE OF FILTER AT ZERO FREQUENCY */
/*  INPUT ARGUMENTS: */
/*  ---------------- */
/*      IORD           DESIRED FILTER ORDER */

static int beroots(p, rtype, dcvalue, iord)
complex *p;
char *rtype;
float *dcvalue;
int iord;
{
    /* Parameter adjustments */
    rtype -= 3;
    --p;

    /* Function Body */
    if (iord == 1) {
    p[1].r = (float)-1., p[1].i = (float)0.;
    strncpy(rtype + 3, "SP", 2);
    } else if (iord == 2) {
    p[1].r = (float)-1.1016013, p[1].i = (float).6360098;
    strncpy(rtype + 3, "CP", 2);
    } else if (iord == 3) {
    p[1].r = (float)-1.0474091, p[1].i = (float).9992645;
    strncpy(rtype + 3, "CP", 2);
    p[2].r = (float)-1.3226758, p[2].i = (float)0.;
    strncpy(rtype + 6, "SP", 2);
    } else if (iord == 4) {
    p[1].r = (float)-.9952088, p[1].i = (float)1.2571058;
    strncpy(rtype + 3, "CP", 2);
    p[2].r = (float)-1.3700679, p[2].i = (float).4102497;
    strncpy(rtype + 6, "CP", 2);
    } else if (iord == 5) {
    p[1].r = (float)-.9576766, p[1].i = (float)1.4711244;
    strncpy(rtype + 3, "CP", 2);
    p[2].r = (float)-1.3808774, p[2].i = (float).7179096;
    strncpy(rtype + 6, "CP", 2);
    p[3].r = (float)-1.502316, p[3].i = (float)0.;
    strncpy(rtype + 9, "SP", 2);
    } else if (iord == 6) {
    p[1].r = (float)-.9306565, p[1].i = (float)1.6618633;
    strncpy(rtype + 3, "CP", 2);
    p[2].r = (float)-1.3818581, p[2].i = (float).9714719;
    strncpy(rtype + 6, "CP", 2);
    p[3].r = (float)-1.5714904, p[3].i = (float).3208964;
    strncpy(rtype + 9, "CP", 2);
    } else if (iord == 7) {
    p[1].r = (float)-.9098678, p[1].i = (float)1.8364514;
    strncpy(rtype + 3, "CP", 2);
    p[2].r = (float)-1.3789032, p[2].i = (float)1.1915667;
    strncpy(rtype + 6, "CP", 2);
    p[3].r = (float)-1.6120388, p[3].i = (float).5892445;
    strncpy(rtype + 9, "CP", 2);
    p[4].r = (float)-1.6843682, p[4].i = (float)0.;
    strncpy(rtype + 12, "SP", 2);
    } else if (iord == 8) {
    p[1].r = (float)-.892871, p[1].i = (float)1.9983286;
    strncpy(rtype + 3, "CP", 2);
    p[2].r = (float)-1.3738431, p[2].i = (float)1.3883585;
    strncpy(rtype + 6, "CP", 2);
    p[3].r = (float)-1.6369417, p[3].i = (float).8227968;
    strncpy(rtype + 9, "CP", 2);
    p[4].r = (float)-1.7574108, p[4].i = (float).2728679;
    strncpy(rtype + 12, "CP", 2);
    }
    nsects = iord - iord / 2;
    *dcvalue = (float)1.;
    return TRUE;
}

static complex cmul(a,b)
complex a,b;
{
    complex c;
    c.r = a.r * b.r - a.i * b.i;
    c.i = a.i * b.r + a.r * b.i;
    return c;
}

static complex cpowi(a,n)
complex a;
int n;
{
    int i;
    complex c;
    c.i = 0.;
    c.r = 1.;
    for (i = 1; i <= n; ++i) {
    c = cmul(c, a);
    }
    return c;

}
static complex csqrt(z)
complex z;
{   complex c;
    float x,y,w,r;
    if ((z.r == 0.0) && (z.i == 0.0)) {
        c.r=0.0;
        c.i=0.0;
        return c;
    } else {
        x=fabs(z.r);
        y=fabs(z.i);
        if (x >= y) {
            r=y/x;
            w=sqrt(x)*sqrt(0.5*(1.0+sqrt(1.0+r*r)));
        } else {
            r=x/y;
            w=sqrt(y)*sqrt(0.5*(r+sqrt(1.0+r*r)));
        }
        if (z.r >= 0.0) {
            c.r=w;
            c.i=z.i/(2.0*w);
        } else {
            c.i=(z.i >= 0) ? w : -w;
            c.r=z.i/(2.0*c.i);
        }
        return c;
    }
}

static complex conjg(z)
complex z;
{   complex c;
    c.r=z.r;
    c.i = -z.i;
    return c;
}

static complex cdiv(a,b)
complex a,b;
{   complex c;
    float r,den;
    if (fabs(b.r) >= fabs(b.i)) {
        r=b.i/b.r;
        den=b.r+r*b.i;
        c.r=(a.r+r*a.i)/den;
        c.i=(a.i-r*a.r)/den;
    } else {
        r=b.r/b.i;
        den=b.i+r*b.r;
        c.r=(a.r*r+a.i)/den;
        c.i=(a.i*r-a.r)/den;
    }
    return c;
}
//*=========================================================================      
//*                                                                               
//* Routine to convert from day-of-year to date                                   
//*                                                         Leif Kvamme 12-1-85   
void DTE (DOY,DAY,MON,YR)                                          
int DOY;
int *DAY;
int *MON;
int YR;
{
int MTH[14],J,M,N;
for(J=1;J<8;J=J+2)
    MTH[J] = 31;
for(J=8;J<13;J=J+2)
    MTH[J] = 31;
MTH[2] = 28;
MTH[4] = 30;
MTH[6] = 30;
MTH[9] = 30;
MTH[11]= 30;
if ((YR%4) == 0) MTH[2] = 29;
M = 0;                                                                    
for(J=1;J<13;J++)
{
    M = M + MTH[J];
    N = DOY - M;                                                              
    if (N <= 0)
    {                                                       
        *MON = J;
        *DAY = N + MTH[J];                                                     
        goto four;                                                               
    }
}
four:                                                                   
J=0;
}
void TIMSEC (YR,MTH,DAY,HR,MIN,SECS,MSECS)                         
int YR;
int MTH;
int DAY;
int HR;
int MIN;
float SECS;
double *MSECS;                         /* total seconds to be returned      */
{
int DYR;                               /* flag for leap year                */
int IYR;                               /* number of leap-years since 1900   */
int YDY;                               /* number of days in current year    */

if(YR <= 50) YR=YR+100;                /* check for 2000 or more in 2 digits*/

if(YR >= 1900) YR=YR - 1900;
DYR = 0;
if((YR % 4) == 0) DYR = 1;
IYR = YR/4 - DYR;
//                -- Seconds to beginning of     
*MSECS = (float)(IYR*366) + (float)((YR-IYR)*365);
//                -- current year                
*MSECS = *MSECS*86400.0;
//                -- January                     
if (MTH == 1) YDY = DAY;
//                -- February                    
if (MTH == 2) YDY = DAY + 31;
//                -- ....                        
if (MTH == 3) YDY = DAY + DYR + 59;       
if (MTH == 4) YDY = DAY + DYR + 90;                                     
if (MTH == 5) YDY = DAY + DYR + 120;                                    
if (MTH == 6) YDY = DAY + DYR + 151;                                    
if (MTH == 7) YDY = DAY + DYR + 181;                                    
if (MTH == 8) YDY = DAY + DYR + 212;                                    
if (MTH == 9) YDY = DAY + DYR + 243;                                    
if (MTH ==10) YDY = DAY + DYR + 273;                                    
if (MTH ==11) YDY = DAY + DYR + 304;                                    
if (MTH ==12) YDY = DAY + DYR + 334;                                    
*MSECS = *MSECS + (float)(YDY*86400 + HR*3600 + MIN*60) + SECS;
}











